/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.exoservices;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.eclipse.core.portal.model.INode;

/**
 * Represents a data object to hold the content of configuration.xml which is the
 * configuration file used to configure services in the eXo Platform. Although this class
 * is not aware of XStream, please do not change the name of the private fields since
 * XStream is dependent on them to serialize from/to xml.
 * 
 * @author Hatim Khan (from Tuan Nguyen work)
 */
public class Configuration implements INode
{
  private boolean mReadOnly = false;
  private List globalParam = null;
  private List serviceConfigurations = null;
  private List importConfigurations = null;
  private List removeServices = null;
  private transient Object mParent = null;

  /**
   * @return Returns a list of GlobalParam instances
   */
  public List getGlobalParam()
  {
    return globalParam;
  }

  /**
   * @param globalParam The globalParam to set.
   */
  public void setGlobalParam(List globalParam)
  {
    this.globalParam = globalParam;
  }

  /**
   * @return Returns a list of Strings
   */
  public List getImportConfigurations()
  {
    return importConfigurations;
  }

  /**
   * @param importConfigurations The importConfigurations to set.
   */
  public void setImportConfigurations(List importConfigurations)
  {
    this.importConfigurations = importConfigurations;
  }

  /**
   * @return Returns a list of Strings.
   */
  public List getRemoveServices()
  {
    return removeServices;
  }

  /**
   * @param removeServices The removeServices to set.
   */
  public void setRemoveServices(List removeServices)
  {
    this.removeServices = removeServices;
  }

  /**
   * @return Returns a list of ServiceConfiguration instances
   */
  public List getServiceConfigurations()
  {
    return serviceConfigurations;
  }

  /**
   * @param serviceConfigurations The serviceConfigurations to set.
   */
  public void setServiceConfigurations(List serviceConfigurations)
  {
    this.serviceConfigurations = serviceConfigurations;
  }

  public void updateAppearIn(String pFile)
  {
    // lets update the global param first
    if ((globalParam != null) && (globalParam.size() > 0))
    {
      GlobalParam gp = (GlobalParam) globalParam.get(0);
      List list = gp.getInitParams();
      if (list != null)
      {
        Iterator iterator = list.iterator();
        while (iterator.hasNext())
        {
          ValuesParam param = (ValuesParam) iterator.next();
          param.setAppearIn(pFile);
        }
      }
    }

    // now we will let the services know about which file they appeared in
    if (serviceConfigurations != null)
    {
      Iterator iterator = serviceConfigurations.iterator();
      while (iterator.hasNext())
      {
        ServiceConfiguration sc = (ServiceConfiguration) iterator.next();
        sc.setAppearIn(pFile);
      }
    }
  }

  public Object[] getChildren()
  {
    List result = new ArrayList();
    // lets update the global param first
    if ((globalParam != null) && (globalParam.size() > 0))
    {
      GlobalParam gp = (GlobalParam) globalParam.get(0);
      gp.setParent(this);
      result.add(gp);
    }

    // now we will let the services know about which file they appeared in
    if (serviceConfigurations != null)
    {
      Iterator iterator = serviceConfigurations.iterator();
      while (iterator.hasNext())
      {
        ServiceConfiguration sc = (ServiceConfiguration) iterator.next();
        sc.setParent(this);
        result.add(sc);
      }
    }

    return result.toArray();
  }

  /**
   * @return Returns the readOnly.
   */
  public boolean isReadOnly()
  {
    return mReadOnly;
  }

  /**
   * @param pReadOnly The readOnly to set.
   */
  public void setReadOnly(boolean pReadOnly)
  {
    mReadOnly = pReadOnly;
    if (pReadOnly)
    {
      // it is assumed that when a configuration is marked as read only, then all of its
      // global parameters are sticky by default
      markGlobalParamsSticky();
    }
  }

  public void markGlobalParamsSticky()
  {
    // lets update the global param
    if ((globalParam != null) && (globalParam.size() > 0))
    {
      GlobalParam gp = (GlobalParam) globalParam.get(0);
      List list = gp.getInitParams();
      if (list != null)
      {
        Iterator iterator = list.iterator();
        while (iterator.hasNext())
        {
          ValuesParam param = (ValuesParam) iterator.next();
          param.setSticky(true);
        }
      }
    }
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#getParent()
   */
  public Object getParent()
  {
    return mParent;
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#setParent(java.lang.Object)
   */
  public void setParent(Object pParent)
  {
    mParent = pParent;
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#removeChild(java.lang.Object)
   */
  public boolean removeChild(Object pChild)
  {
    if (pChild == null) return false;
    boolean removed = false;
    if ((globalParam != null) && (globalParam.remove(pChild)))
    {
      removed = true;
    }
    else if ((serviceConfigurations != null) && (serviceConfigurations.remove(pChild)))
    {
      removed = true;
    }

    // we disconnect the child from the parent if we are able to remove it
    if (removed)
    {
      if (pChild instanceof INode) ((INode) pChild).setParent(null);
    }

    return removed;
  }

  public List getServicesKeys()
  {
    List result = new ArrayList();
    if (serviceConfigurations != null)
    {
      Iterator iterator = serviceConfigurations.iterator();
      while (iterator.hasNext())
      {
        ServiceConfiguration service = (ServiceConfiguration) iterator.next();
        String key = service.getKey();
        if ((key == null) || (key.trim().length() < 1))
        {
          key = service.getType();
        }
        if ((key != null) && (key.trim().length() > 0))
        {
          result.add(key);
        }
      }
    }

    return result;
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#addChild(java.lang.Object)
   */
  public void addChild(Object pChild)
  {
    if (pChild == null) return;
    if (pChild instanceof INode)
    {
      INode node = (INode) pChild;
      node.setParent(this);
    }

    // when the child is a global param
    if (pChild instanceof GlobalParam)
    {
      GlobalParam newGlobal = (GlobalParam) pChild;
      if (globalParam == null) globalParam = new ArrayList();
      if (globalParam.size() == 0) globalParam.add(newGlobal);
      else
      {
        GlobalParam existingGlobalParam = (GlobalParam) globalParam.get(0);
        List existingInitParams = existingGlobalParam.getInitParams();
        if (existingInitParams == null)
        {
          existingInitParams = new ArrayList();
          existingGlobalParam.setInitParams(existingInitParams);
        }
        
        List newInitParams = newGlobal.getInitParams();
        if (newInitParams != null)
        {
          existingInitParams.addAll(newInitParams);
        }
      }
    }
    
    // when the child is service configuration
    if (pChild instanceof ServiceConfiguration)
    {
      ServiceConfiguration newService = (ServiceConfiguration) pChild;
      if (serviceConfigurations == null) serviceConfigurations = new ArrayList();
      serviceConfigurations.add(newService);
    }    
  }
}