/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.exoservices;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.eclipse.core.common.Utils;
import org.exoplatform.eclipse.core.portal.model.INode;

/**
 * An abstract representation of parameter to the service configuration. Although this
 * class is not aware of XStream, please do not change the name of the private fields
 * since XStream is dependent on them to serialize from/to xml.
 * 
 * Note: here is some comments about the sticky property for this parameter.
 * 
 * A parameter that is sticky means that it can not be removed. However, it can be edited
 * or replaced with a parameter that has the same name and once it is replaced with a new
 * parameter that parameter should become sticky too. History: the main drive for creating
 * such a property is the fact that if a initParam of a global parameter appeared in a
 * configuration.xml that is in a jar file, then there is no supported way in eXo Platform
 * to remove it, however, the parameter can be overriden by another initParam of a global
 * parameter (the initParam must have the same name, though).
 * 
 * @author Hatim Khan (from Tuan Nguyen work)
 */
abstract public class Parameter implements INode
{
  private boolean mSticky = false;
  private String description;
  private String name;
  private transient Object mParent = null;

  public String getDescription()
  {
    return description;
  }

  public void setDescription(String description)
  {
    this.description = description;
  }

  public String getName()
  {
    return name;
  }

  public void setName(String name)
  {
    this.name = name;
  }

  /**
   * @return Returns the parent.
   */
  public Object getParent()
  {
    return mParent;
  }

  /**
   * @param pParent The parent to set.
   */
  public void setParent(Object pParent)
  {
    mParent = pParent;
  }

  public Object[] getChildren()
  {
    return null;
  }

  protected void addAfterAttachingToParent(List pContainer, List pChildren)
  {
    if (pChildren == null) return;
    Iterator iterator = pChildren.iterator();
    while (iterator.hasNext())
    {
      INode p = (INode) iterator.next();
      p.setParent(this);
      pContainer.add(p);
    }
  }

  /**
   * A parameter that is sticky means that it can not be removed. However, it can be
   * edited or replaced with a parameter that has the same name and once it is replaced
   * with a new parameter that parameter should become sticky too. History: the main drive
   * for creating such a property is the fact that if a initParam of a global parameter
   * appeared in a configuration.xml that is in a jar file, then there is no supported way
   * in eXo Platform to remove it, however, the parameter can be overriden by another
   * initParam of a global parameter (the initParam must have the same name, though).
   * 
   * @return Returns the sticky flag.
   */
  public boolean isSticky()
  {
    return mSticky;
  }

  /**
   * @param pSticky The sticky to set.
   */
  public void setSticky(boolean pSticky)
  {
    mSticky = pSticky;
  }

  protected void populateCopy(Parameter pParameter)
  {
    if (pParameter == null) return;
    pParameter.setName(name);
    pParameter.setDescription(description);
    pParameter.setSticky(mSticky);
  }

  abstract public Object getCopy();

  public boolean isEquivalent(Object pObject)
  {
    if ((pObject == null) || !(pObject instanceof Parameter)) return false;
    Parameter parameter = (Parameter) pObject;

    if (mSticky != parameter.mSticky) return false;
    boolean nameSame = Utils.isEquivalentString(this.name, parameter.name);
    boolean descSame = Utils.isEquivalentString(this.description, parameter.description);

    return nameSame && descSame;
  }

  /**
   * A helper method that takes two lists which contain elements of type 'Parameter' and
   * checks if they are equivalent.
   * 
   * @param p1 list of 'Parameter' elements
   * @param p2 list of 'Parameter' elements
   * @return true if the two list are equivalent, false otherwise
   */
  public static boolean isEquivalentParameterList(List p1, List p2)
  {
    if ((p1 == null) && (p2 == null)) return true;
    if (p1 == null) return false;
    if (p2 == null) return false;

    if (p1.size() != p2.size()) return false;
    for (int i = 0, sizei = p1.size(); i < sizei; ++i)
    {
      Object o1 = p1.get(i);
      Object o2 = p2.get(i);
      if (!(o1 instanceof Parameter) || !(o2 instanceof Parameter)) return false;
      Parameter v1 = (Parameter) o1;
      Parameter v2 = (Parameter) o2;
      if (!(v1.isEquivalent(v2))) return false;
    }

    return true;
  }

  /**
   * A helper method that returns a copy of a given list of 'Parameter' elements.
   * 
   * @param p1 list of 'Parameter' elements
   * @return a copy of the given list, only elements of type 'Parameter' will be copied
   */
  public static List getCopyParameterList(List p1)
  {
    if (p1 == null) return null;
    List result = new ArrayList();

    for (Iterator iterator = p1.iterator(); iterator.hasNext();)
    {
      Object o1 = iterator.next();
      if (o1 instanceof Parameter)
      {
        Parameter v1 = (Parameter) o1;
        result.add(v1.getCopy());
      }
    }

    return result;
  }
}