/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.exoservices;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * This class is used to merge two or more services together. The order in which you call
 * the add() method will affect the merge operation. This class is NOT responsible for
 * loading the configurations from xml nor is it responsible for importing the
 * configuration specified in the xml of another configuration using the "import" element.
 * 
 * A typical usage would be to call : add() multiple times then call getMergedServices()
 * which will return a configuration instance that contains the result after merging.
 * 
 * @author Hatim Khan
 */
public class ServicesMerger
{
  private Map mGlobalParams = new HashMap();
  private Map mServices = new HashMap();
  private List mRemoveServices = new ArrayList();

  /**
   * Use this method to include the configuration for merging.
   * 
   * @param pConfig the configuration to include for merging
   */
  public void add(Configuration pConfig)
  {
    if (pConfig == null) return;
    mergeGlobalParam(pConfig.getGlobalParam());
    mergeServices(pConfig.getServiceConfigurations());
    List removeServices = pConfig.getRemoveServices();
    if (removeServices != null) mRemoveServices.addAll(removeServices);
  }

  /**
   * Use this method to include a list of configuration for merging.
   * 
   * @param pConfigs a list of Configuration instances
   */
  public void add(List pConfigs)
  {
    if (pConfigs == null) return;
    Iterator iterator = pConfigs.iterator();
    while (iterator.hasNext())
    {
      add((Configuration) iterator.next());
    }
  }

  public Configuration getMergedServices()
  {
    removeServices();
    Configuration config = new Configuration();
    GlobalParam gp = new GlobalParam();
    gp.setInitParams(convertToList(mGlobalParams.values()));
    List list = new ArrayList();
    list.add(gp);
    config.setGlobalParam(list);
    config.setServiceConfigurations(convertToList(mServices.values()));

    return config;
  }

  /**
   * @param pGlobalParam the list should only contain one instance of GlobalParam that
   *          instance in turn should have a list of ValuesParam instances
   */
  private void mergeGlobalParam(List pGlobalParam)
  {
    if (pGlobalParam == null) return;
    if (pGlobalParam.isEmpty()) return;
    GlobalParam gp = (GlobalParam) pGlobalParam.get(0);
    List initParams = gp.getInitParams();
    if (initParams == null) return;
    Iterator iterator = initParams.iterator();
    while (iterator.hasNext())
    {
      ValuesParam params = (ValuesParam) iterator.next();
      if (params.getName() == null) continue;
      // we need to maintain the sticky state of parameter
      // even if we replace it
      ValuesParam existingParams = (ValuesParam) mGlobalParams.get(params.getName());
      if ((existingParams != null) && (existingParams.isSticky()))
      {
        params.setSticky(true);
      }
      mGlobalParams.put(params.getName(), params);
    }
  }

  /**
   * @param pServices a list of ServicesConfiguration instances to merge
   */
  private void mergeServices(List pServices)
  {
    if (pServices == null) return;
    Iterator iterator = pServices.iterator();
    while (iterator.hasNext())
    {
      ServiceConfiguration service = (ServiceConfiguration) iterator.next();
      String key = service.getKey();
      if (key == null) key = service.getType();
      if (key == null) continue;
      mServices.put(key, service);
    }
  }

  private void removeServices()
  {
    Iterator iterator = mRemoveServices.iterator();
    while (iterator.hasNext())
    {
      String key = (String) iterator.next();
      mServices.remove(key);
    }
  }

  private List convertToList(Collection c)
  {
    List result = new ArrayList();
    if (c == null) return result;
    Iterator iterator = c.iterator();
    while (iterator.hasNext())
    {
      result.add(iterator.next());
    }

    return result;
  }
}