/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.eclipse.core.exoservices;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.eclipse.core.common.Utils;
import org.exoplatform.eclipse.core.portal.model.INode;
import org.exoplatform.eclipse.core.portal.model.ValueHolder;

/**
 * Represent a parameter that can hold a list of string values, a name and a desscription.
 * Although this class is not aware of XStream, please do not change the name of the
 * private fields since XStream is dependent on them to serialize from/to xml.
 * 
 * @author Hatim Khan (from Tuan Nguyen work)
 */
public class ValuesParam extends Parameter
{
  private List values;
  private String appearIn;
  private transient List mValueHolders = null;

  /**
   * @return Returns a list of Strings
   */
  public List getValues()
  {
    return values;
  }

  /**
   * @param values The values to set.
   */
  public void setValues(List values)
  {
    this.values = values;
    clearValueHolders();
    createValueHolders();
  }

  /**
   * @return Returns the appearIn.
   */
  public String getAppearIn()
  {
    return appearIn;
  }

  /**
   * @param appearIn The appearIn to set.
   */
  public void setAppearIn(String appearIn)
  {
    this.appearIn = appearIn;
  }

  private void clearValueHolders()
  {
    if (mValueHolders != null)
    {
      Iterator iterator = mValueHolders.iterator();
      while (iterator.hasNext())
      {
        ValueHolder value = (ValueHolder) iterator.next();
        value.setParent(null);
      }
      mValueHolders.clear();
    }
  }

  private void createValueHolders()
  {
    if (mValueHolders == null) mValueHolders = new ArrayList();
    if (values != null)
    {
      Iterator iterator = values.iterator();
      while (iterator.hasNext())
      {
        ValueHolder value = new ValueHolder();
        value.setValue((String) iterator.next());
        value.setParent(this);
        mValueHolders.add(value);
      }
    }
  }

  public Object[] getChildren()
  {
    if (mValueHolders == null) createValueHolders();
    List result = new ArrayList();
    if (mValueHolders != null)
    {
      result.addAll(mValueHolders);
    }
    return result.toArray();
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#removeChild(java.lang.Object)
   */
  public boolean removeChild(Object pChild)
  {
    boolean removed = false;

    if ((pChild == null) || !(pChild instanceof ValueHolder) || (mValueHolders == null) || (values == null))
        return false;
    ValueHolder child = (ValueHolder) pChild;
    String childValue = child.getValue();
    if (childValue == null) return false;

    // when we move a value holder we need to make sure we are removing the value in the
    // same index
    int index = mValueHolders.indexOf(child);
    if (index < 0) return false;
    if (values == null) return false;
    String valueAtIndex = (String) values.get(index);
    if (childValue.equals(valueAtIndex))
    {
      // we have a match so delete both the child and the value
      mValueHolders.remove(child);
      values.remove(index);
      removed = true;
    }

    // we disconnect the child from the parent if we are able to remove it
    if (removed)
    {
      if (pChild instanceof INode) ((INode) pChild).setParent(null);
    }

    return removed;
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#addChild(java.lang.Object)
   */
  public void addChild(Object pChild)
  {
    if (pChild == null) return;
    if (pChild instanceof INode)
    {
      INode node = (INode) pChild;
      node.setParent(this);
    }

    if (pChild instanceof ValueHolder)
    {
      ValueHolder newValueHolder = (ValueHolder) pChild;
      if (mValueHolders == null) createValueHolders();
      String newValue = newValueHolder.getValue();
      if (newValue != null)
      {
        mValueHolders.add(newValueHolder);
        // we also need to add the value to the value list
        if (values == null) values = new ArrayList();
        values.add(newValue);
      }
    }
  }

  public Object getCopy()
  {
    ValuesParam copy = new ValuesParam();
    populateCopy(copy);
    copy.setAppearIn(appearIn);
    List copyValues = new ArrayList();
    if (values != null)
    {
      copyValues.addAll(values);
      copy.setValues(copyValues);
    }

    return copy;
  }

  public boolean isEquivalent(Object pObject)
  {
    boolean paramSame = super.isEquivalent(pObject);
    if ((!paramSame) || !(pObject instanceof ValuesParam)) return false;
    
    ValuesParam param = (ValuesParam) pObject;
    boolean appearSame = Utils.isEquivalentString(this.appearIn, param.appearIn);
    if (!appearSame) return false;

    return isEquivalentValues(param.values);
  }

  public boolean isEquivalentValues(List p1)
  {
    if ((p1 == null) && (this.values == null)) return true;
    if (p1 == null) return false;
    if (this.values == null) return false;

    if (p1.size() != values.size()) return false;
    for (int i = 0, sizei = p1.size(); i < sizei; ++i)
    {
      String v1 = (String) p1.get(i);
      String v2 = (String) this.values.get(i);
      if (!(v1.equals(v2))) return false;
    }

    return true;
  }
}