/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.filesprovider;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;

import java.io.IOException;
import java.io.InputStream;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

/**
 * This class uses the same code as the one in
 * org.eclipse.ui.wizards.datatransfer the only difference here is that this
 * class supports empty directories while the one that comes with the
 * org.eclipse.ui.wizards.datatransfer package does not. Also, most of the
 * comments here are coming from the original class.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ZipFileStructureProvider implements IImportStructureProvider
{
  public static final String CLASS_VERSION =
    "$Id: ZipFileStructureProvider.java,v 1.1 2004/04/19 03:45:47 hatimk Exp $"; //$NON-NLS-1$
  private ZipFile mSourceFile;
  private ZipEntry mRoot = new ZipEntry("/"); //$NON-NLS-1$
  private Map mChildren;
  private Map mDirectoryEntryCache = new HashMap();

  /**
   * The constructor method. Creates a <code>ZipFileStructureProvider</code>,
   * which will operate on the passed zip file.
   *
   * @param pSourceFile the zip file to walk through
   */
  public ZipFileStructureProvider(ZipFile pSourceFile)
  {
    super();
    mSourceFile = pSourceFile;
  }

  /**   
   * Initializes this object's children table based on the contents of the
   * specified source file.
   */
  protected void initialize()
  {
    mChildren = new HashMap(1000);

    Enumeration entries = mSourceFile.entries();
    while (entries.hasMoreElements())
    {
      ZipEntry entry = (ZipEntry) entries.nextElement();

      IPath path = new Path(entry.getName()).addTrailingSeparator();
      int pathSegmentCount = path.segmentCount();

      if (!entry.isDirectory())
      {
        for (int i = 1; i < pathSegmentCount; i++)
        {
          createContainer(path.uptoSegment(i));
        }

        createFile(entry);
      }
      else
      {
        for (int i = 1; i < (pathSegmentCount + 1); i++)
        {
          createContainer(path.uptoSegment(i));
        }
      }
    }
  }

  /**   
   * Creates a new container zip entry with the specified name, iff it has not
   * already been created.
   *
   * @param pPathName a container path that should end with "/"
   */
  protected void createContainer(IPath pPathName)
  {
    if (mDirectoryEntryCache.containsKey(pPathName))
    {
      return;
    }

    ZipEntry parent;
    if (pPathName.segmentCount() == 1)
    {
      parent = mRoot;
    }
    else
    {
      parent =
        (ZipEntry) mDirectoryEntryCache.get(pPathName.removeLastSegments(1));
    }

    ZipEntry newEntry = new ZipEntry(pPathName.toString());
    mDirectoryEntryCache.put(pPathName, newEntry);
    addToChildren(parent, newEntry);
  }

  /**   
   * Adds the specified child to the internal collection of the parent's
   * children.
   *
   * @param parent the parent
   * @param child the child
   */
  protected void addToChildren(ZipEntry parent, ZipEntry child)
  {
    List childList = (List) mChildren.get(parent);
    if (childList == null)
    {
      childList = new ArrayList();
      mChildren.put(parent, childList);
    }

    childList.add(child);
  }

  /**   
   * Creates a new file zip entry with the specified name.
   *
   * @param entry the entry
   */
  protected void createFile(ZipEntry entry)
  {
    IPath pathname = new Path(entry.getName());
    ZipEntry parent;
    if (pathname.segmentCount() == 1)
    {
      parent = mRoot;
    }
    else
    {
      parent =
        (ZipEntry) mDirectoryEntryCache.get(pathname.removeLastSegments(1));
    }

    addToChildren(parent, entry);
  }

  /**   
   * @see org.eclipse.ui.wizards.datatransfer.IImportStructureProvider#getChildren(java.lang.Object)
   */
  public List getChildren(Object pElement)
  {
    if (mChildren == null)
    {
      initialize();
    }

    List children = (List) mChildren.get(pElement);

    return (children == null) ? new ArrayList() : children;
  }

  /**   
   * @see org.eclipse.ui.wizards.datatransfer.IImportStructureProvider#getContents(java.lang.Object)
   */
  public InputStream getContents(Object pElement)
  {
    try
    {
      return mSourceFile.getInputStream((ZipEntry) pElement);
    }
    catch (IOException e)
    {
      return null;
    }
  }

  /**   
   * @see org.eclipse.ui.wizards.datatransfer.IImportStructureProvider#getFullPath(java.lang.Object)
   */
  public String getFullPath(Object pElement)
  {
    return ((ZipEntry) pElement).getName();
  }

  /**   
   * @see org.eclipse.ui.wizards.datatransfer.IImportStructureProvider#getLabel(java.lang.Object)
   */
  public String getLabel(Object pElement)
  {
    if (pElement.equals(mRoot))
    {
      return ((ZipEntry) pElement).getName();
    }

    return new Path(((ZipEntry) pElement).getName()).lastSegment();
  }

  /**   
   * @see org.eclipse.ui.wizards.datatransfer.IImportStructureProvider#isFolder(java.lang.Object)
   */
  public boolean isFolder(Object pElement)
  {
    return ((ZipEntry) pElement).isDirectory();
  }

  /**   
   * Returns the entry that this importer uses as the root sentinel.
   *
   * @return ZipEntry the root
   */
  public ZipEntry getRoot()
  {
    return mRoot;
  }

  /**   
   * Returns the zip file that this provider provides structure for.
   *
   * @return the zip file that this provider provides structure for
   */
  public ZipFile getZipFile()
  {
    return mSourceFile;
  }
}
