/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.launching;


import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.debug.core.sourcelookup.ISourceContainer;
import org.eclipse.debug.core.sourcelookup.containers.DirectorySourceContainer;
import org.eclipse.debug.core.sourcelookup.containers.ExternalArchiveSourceContainer;
import org.eclipse.jdt.core.IJavaModel;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.launching.IRuntimeClasspathEntry;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jdt.launching.LibraryLocation;
import org.eclipse.jdt.launching.sourcelookup.containers.JavaProjectSourceContainer;
import org.eclipse.jdt.launching.sourcelookup.containers.PackageFragmentRootSourceContainer;
import org.exoplatform.eclipse.core.ExoCorePlugin;

import java.io.File;
import java.util.HashSet;
import java.util.Set;

/**
 * A collection of static methods to help build the source lookup containers.
 * 
 * @author Hatim Khan
 */
public class ExoSourceLookupUtil
{

  /**
   * Translates the given runtime classpath entry into the associated source container. If
   * the given runtime classpath entry is not part of an already existing external jar
   * file, then it must have a source path otherwise null will be returned.
   * 
   * @param pEntry entry to translate
   * @exception CoreException if unable to expand the path
   */
  public static ISourceContainer translate(IRuntimeClasspathEntry pEntry)
  {
    boolean considerSourceAttachments = true;
    ISourceContainer container = null;
    IRuntimeClasspathEntry entry = pEntry;

    if ((entry == null) || (entry.getLocation() == null)) return null;

    switch (entry.getType())
    {
      case IRuntimeClasspathEntry.ARCHIVE:
        IPackageFragmentRoot root = getPackageFragmentRoot(entry, considerSourceAttachments);
        String path = entry.getSourceAttachmentLocation();
        if (root == null && path == null && considerSourceAttachments)
        {
          // use the pkg frag root it there is no source attachment
          root = getPackageFragmentRoot(entry, false);
        }
        if (root == null)
        {
          if (path != null)
          {
            File file = new File(path);
            if (file.exists())
            {
              if (file.isDirectory())
              {
                container = new DirectorySourceContainer(file, true);
              }
              else
              {
                // it must be a zip or a jar file that contains the source code
                container = new ExternalArchiveSourceContainer(path, true);
              }
            }
          }
        }
        else
        {
          container = new PackageFragmentRootSourceContainer(root);
        }
        break;
      case IRuntimeClasspathEntry.PROJECT:
        IResource resource = entry.getResource();
        if (resource != null && resource.getType() == IResource.PROJECT)
        {
          container = new JavaProjectSourceContainer(JavaCore.create((IProject) resource));
        }
        break;
      default:
        // no other classpath types are valid in a resolved path
        break;
    }
    return container;
  }

  /**
   * Translates the given runtime classpath entries into the associated source containers
   * and return the result in a set. If any given runtime classpath entry is not part of
   * an already existing external jar file, then it must have a source path otherwise its
   * souce container will not be included in the returned set.
   * 
   * @param pEntry entry to translate
   * @exception CoreException if unable to expand the path
   */
  public static Set translate(IRuntimeClasspathEntry[] pEntries)
  {
    Set result = new HashSet();
    if (pEntries != null)
    {
      for (int i = 0, sizei = pEntries.length; i < sizei; ++i)
      {
        ISourceContainer container = translate(pEntries[i]);
        if (container != null)
        {
          result.add(container);
        }
      }
    }
    return result;
  }

  public static Set getVMInstallSourceContainers(IVMInstall pVMInstall)
  {
    Set result = new HashSet();

    LibraryLocation[] locations = JavaRuntime.getLibraryLocations(pVMInstall);

    if ((locations == null) || (locations.length < 1))
    {
      return result;
    }

    for (int i = 0, sizei = locations.length; i < sizei; ++i)
    {
      IPath libraryPath = locations[i].getSystemLibraryPath();
      IPath sourcePath = locations[i].getSystemLibrarySourcePath();
      IPath sourceRootPath = locations[i].getPackageRootPath();

      IRuntimeClasspathEntry entry = JavaRuntime.newArchiveRuntimeClasspathEntry(libraryPath);
      entry.setSourceAttachmentPath(sourcePath);
      entry.setSourceAttachmentRootPath(sourceRootPath);

      ISourceContainer container = translate(entry);
      if (container != null)
      {
        result.add(container);
      }
    }

    return result;
  }

  /**
   * Given a list of java projects, returns the equivalent set of ISourceContainers
   * 
   * @param pJavaProjects java projects (portlet projects)
   * @return a set of ISourceContainer
   * @throws CoreException when things go wrong
   */
  public static Set getPortletProjectsSourceContainers(IJavaProject[] pJavaProjects) throws CoreException
  {
    Set result = new HashSet();

    if (pJavaProjects == null)
    {
      return result;
    }

    // for each java project returns its java source location
    for (int i = 0, sizei = pJavaProjects.length; i < sizei; ++i)
    {
      ISourceContainer container = new JavaProjectSourceContainer(pJavaProjects[i]);
      result.add(container);
    }

    return result;
  }

  /**
   * Returns whether the given objects are equal, allowing for <code>null</code>.
   * 
   * @param a
   * @param b
   * @return whether the given objects are equal, allowing for <code>null</code>
   */
  private static boolean equalOrNull(Object a, Object b)
  {
    if (a == null)
    {
      return b == null;
    }
    if (b == null)
    {
      return false;
    }
    return a.equals(b);
  }

  /**
   * Returns whether the source attachments of the given package fragment root and runtime
   * classpath entry are equal.
   * 
   * @param root package fragment root
   * @param entry runtime classpath entry
   * @return whether the source attachments of the given package fragment root and runtime
   *         classpath entry are equal
   * @throws JavaModelException
   */
  private static boolean isSourceAttachmentEqual(IPackageFragmentRoot root, IRuntimeClasspathEntry entry)
      throws JavaModelException
  {
    return equalOrNull(root.getSourceAttachmentPath(), entry.getSourceAttachmentPath());
  }

  /**
   * Determines if the given archive runtime classpath entry exists in the workspace as a
   * package fragment root. Returns the associated package fragment root possible,
   * otherwise <code>null</code>.
   * 
   * @param entry archive runtime classpath entry
   * @param considerSourceAttachment whether the source attachments should be considered
   *          comparing against package fragment roots
   * @return package fragment root or <code>null</code>
   */
  private static IPackageFragmentRoot getPackageFragmentRoot(IRuntimeClasspathEntry entry,
      boolean considerSourceAttachment)
  {
    IResource resource = entry.getResource();
    if (resource == null)
    {
      // Check all package fragment roots for case of external archive.
      // External jars are shared, so it does not matter which project it
      // originates from
      IJavaModel model = JavaCore.create(ResourcesPlugin.getWorkspace().getRoot());
      try
      {
        IJavaProject[] jps = model.getJavaProjects();
        for (int i = 0; i < jps.length; i++)
        {
          IJavaProject jp = jps[i];
          IProject p = jp.getProject();
          if (p.isOpen())
          {
            IPackageFragmentRoot[] allRoots = jp.getPackageFragmentRoots();
            for (int j = 0; j < allRoots.length; j++)
            {
              IPackageFragmentRoot root = allRoots[j];
              if (root.isExternal() && root.getPath().equals(new Path(entry.getLocation())))
              {
                if (!considerSourceAttachment || isSourceAttachmentEqual(root, entry))
                {
                  // use package fragment root
                  return root;
                }
              }
            }
          }
        }
      }
      catch (JavaModelException e)
      {
        ExoCorePlugin.logError(e);
      }
    }
    else
    {
      // check if the archive is a package fragment root
      IProject project = resource.getProject();
      IJavaProject jp = JavaCore.create(project);
      try
      {
        if (project.isOpen() && jp.exists())
        {
          IPackageFragmentRoot root = jp.getPackageFragmentRoot(resource);
          IPackageFragmentRoot[] allRoots = jp.getPackageFragmentRoots();
          for (int j = 0; j < allRoots.length; j++)
          {
            if (allRoots[j].equals(root))
            {
              // ensure source attachment paths match
              if (!considerSourceAttachment || isSourceAttachmentEqual(root, entry))
              {
                // use package fragment root
                return root;
              }
            }
          }

        }
        // check all other java projects to see if another project references
        // the archive
        IJavaModel model = JavaCore.create(ResourcesPlugin.getWorkspace().getRoot());
        IJavaProject[] jps = model.getJavaProjects();
        for (int i = 0; i < jps.length; i++)
        {
          IJavaProject jp1 = jps[i];
          IProject p = jp1.getProject();
          if (p.isOpen())
          {
            IPackageFragmentRoot[] allRoots = jp1.getPackageFragmentRoots();
            for (int j = 0; j < allRoots.length; j++)
            {
              IPackageFragmentRoot root = allRoots[j];
              if (!root.isExternal() && root.getPath().equals(entry.getPath()))
              {
                if (!considerSourceAttachment || isSourceAttachmentEqual(root, entry))
                {
                  // use package fragment root
                  return root;
                }
              }
            }
          }
        }
      }
      catch (JavaModelException e)
      {
        ExoCorePlugin.logError(e);
      }
    }
    return null;
  }

}