/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.launching.delegate;


import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.jdt.launching.AbstractJavaLaunchConfigurationDelegate;
import org.eclipse.jdt.launching.ExecutionArguments;
import org.eclipse.jdt.launching.IRuntimeClasspathEntry;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.IVMRunner;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jdt.launching.VMRunnerConfiguration;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.launching.ExoLaunching;
import org.exoplatform.eclipse.core.launching.IExoLaunchingConfigurationConstants;
import org.exoplatform.eclipse.core.launching.IExoTomcatLocation;
import org.exoplatform.eclipse.core.launching.IExoTomcatRuntimeConfiguration;
import org.exoplatform.eclipse.core.launching.IExoTomcatRuntimeConfigurationProvider;
import org.exoplatform.eclipse.core.launching.VMArgument;
import org.exoplatform.eclipse.core.launching.VMArgumentsContainer;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * A launch configuration delegate that understands how to launch eXo Platform with
 * Tomcat.
 * 
 * @author Hatim Khan
 * @version $Revision: 1.2 $
 */
public class ExoTomcatLaunchConfigurationDelegate extends AbstractJavaLaunchConfigurationDelegate
{
  public static final String CLASS_VERSION = "$Id: ExoTomcatLaunchConfigurationDelegate.java,v 1.2 2004/10/10 05:39:05 hatimk Exp $"; //$NON-NLS-1$

  /**
   * @see org.eclipse.jdt.launching.AbstractJavaLaunchConfigurationDelegate#launch(org.eclipse.debug.core.ILaunchConfiguration,
   *      java.lang.String, org.eclipse.debug.core.ILaunch,
   *      org.eclipse.core.runtime.IProgressMonitor)
   */
  public void launch(ILaunchConfiguration pConfig, String pMode, ILaunch pLaunch, IProgressMonitor pMonitor)
      throws CoreException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    pMonitor.beginTask("starting eXo Platform with Tomcat  ", 3);

    boolean shutdown = isShutdownLaunch(pConfig);
    IExoTomcatLocation exoTomcatLocation = getLocation(pConfig);
    IVMInstall vminstall = verifyVMInstall(pConfig);
    IExoTomcatRuntimeConfigurationProvider provider = getConfigurationProvider(pConfig);

    pMonitor.subTask("preparing Tomcat configuration, this might take a few seconds .. ");

    IExoTomcatRuntimeConfiguration config = provider.getConfiguration(exoTomcatLocation, vminstall);
    String mainTypeName = config.getMainTypeName();
    String startupArgs = getString(config.getStartupProgramArguments(), " ");
    String shutdownArgs = getString(config.getShutdownProgramArguments(), " ");
    IRuntimeClasspathEntry[] classpathEntries = config.getClasspathLibraries();
    VMArgument[] vmarguments = config.getVMArguments();
    VMArgumentsContainer container = new VMArgumentsContainer();
    container.appendVMArguments(vmarguments);

    String[] classpath = resolveRuntimeClasspathEntries(classpathEntries, pConfig);
    String pgmArgs = (shutdown) ? shutdownArgs : startupArgs;
    String vmArgs = container.toString();

    // check for cancellation
    if (pMonitor.isCanceled())
    {
      return;
    }

    pMonitor.subTask("preparing the JRE ");

    IVMRunner runner = vminstall.getVMRunner(pMode);
    if (runner == null)
    {
      throw new CoreException(
          ExoCorePlugin.createErrorStatus("The selected VM does not support debugging or running Java applications. Please choose a different JVM.", null));
    }

    String workingDirName = config.getWorkingDirectory().toOSString();

    // Program & VM args
    ExecutionArguments execArgs = new ExecutionArguments(vmArgs, pgmArgs);

    // VM-specific attributes
    Map vmAttributesMap = getVMSpecificAttributesMap(pConfig);

    // Environment variables
    String[] envp = DebugPlugin.getDefault().getLaunchManager().getEnvironment(pConfig);

    // Create VM config
    VMRunnerConfiguration runConfig = new VMRunnerConfiguration(mainTypeName, classpath);
    runConfig.setProgramArguments(execArgs.getProgramArgumentsArray());
    runConfig.setVMArguments(execArgs.getVMArgumentsArray());
    runConfig.setWorkingDirectory(workingDirName);
    runConfig.setVMSpecificAttributesMap(vmAttributesMap);
    runConfig.setEnvironment(envp);

    // check for cancellation
    if (pMonitor.isCanceled())
    {
      return;
    }

    pMonitor.worked(1);

    // check for cancellation
    if (pMonitor.isCanceled())
    {
      return;
    }

    // Launch the configuration - 1 unit of work
    runner.run(runConfig, pLaunch, new SubProgressMonitor(pMonitor, 2));

    // check for cancellation
    if (pMonitor.isCanceled())
    {
      return;
    }

    pMonitor.done();
  }

  public static boolean isShutdownLaunch(ILaunchConfiguration pConfig) throws CoreException
  {
    boolean shutdown = pConfig.getAttribute(IExoLaunchingConfigurationConstants.ATTR_EXO_SHUTDOWN, false);

    return shutdown;
  }

  public static IExoTomcatLocation getLocation(ILaunchConfiguration pConfig) throws CoreException
  {
    // verify Tomcat Installation directory
    String tomcatDir = pConfig.getAttribute(IExoLaunchingConfigurationConstants.ATTR_TOMCAT_INSTALLATION_DIR, (String) null);

    if (tomcatDir == null)
    {
      throw new CoreException(
          ExoCorePlugin.createErrorStatus("Tomcat installation directory is not specified. Please specify one.", null));
    }

    IPath tomcatDirPath = new Path(tomcatDir);
    if (!tomcatDirPath.toFile().exists())
    {
      throw new CoreException(ExoCorePlugin.createErrorStatus("Tomcat installation directory ["
          + tomcatDirPath + "] does not exist. Please specify another one.", null));
    }

    // verify Tomcat source directory
    String tomcatSrcDir = pConfig.getAttribute(IExoLaunchingConfigurationConstants.ATTR_TOMCAT_SOURCE_DIR, (String) null);

    if ((tomcatSrcDir != null) && (tomcatSrcDir.length() > 0))
    {
      IPath tomcatSrcDirPath = new Path(tomcatSrcDir);
      if (!tomcatSrcDirPath.toFile().exists())
      {
        throw new CoreException(ExoCorePlugin.createErrorStatus("Tomcat source directory ["
            + tomcatSrcDirPath + "] does not exist. Please specify another one.", null));
      }
    }

    // verify eXo Platform source directory
    String exoSrcDir = pConfig.getAttribute(IExoLaunchingConfigurationConstants.ATTR_EXO_SOURCE_DIR, (String) null);
    if ((exoSrcDir != null) && (exoSrcDir.length() > 0))
    {
      IPath exoSrcDirPath = new Path(exoSrcDir);
      if (!exoSrcDirPath.toFile().exists())
      {
        throw new CoreException(ExoCorePlugin.createErrorStatus("eXo Platform source directory ["
            + exoSrcDirPath + "] does not exist. Please specify another one.", null));
      }
    }

    // ok, all is ok, put them in one object
    IExoTomcatLocation exoTomcatLocation = new ExoTomcatLocation();
    exoTomcatLocation.setTomcatHome(tomcatDir);
    exoTomcatLocation.setTomcatSrc(tomcatSrcDir);
    exoTomcatLocation.setExoSrc(exoSrcDir);

    return exoTomcatLocation;
  }

  public static IExoTomcatRuntimeConfigurationProvider getConfigurationProvider(ILaunchConfiguration pConfig)
      throws CoreException
  {
    // now lets see if the configuration has a provider that was perviously specified,
    // otherwise throw an execption
    String providerId = pConfig.getAttribute(IExoLaunchingConfigurationConstants.ATTR_EXO_TOMCAT_RUNTIME_CONFIGURATION_PROVIDER_ID, (String) null);

    if (providerId != null)
    {
      IExoTomcatRuntimeConfigurationProvider provider = ExoLaunching.getExoTomcatRuntimeConfigurationProvider(providerId);
      if (provider != null)
      {
        return provider;
      }
      else
      {
        throw new CoreException(
            ExoCorePlugin.createErrorStatus("Can not find the specified eXo Platform version. Please choose another one.", null));
      }
    }
    else
    {
      throw new CoreException(
          ExoCorePlugin.createErrorStatus("No eXo Platform version was specified. Please specify a version.", null));
    }
  }

  private static String[] resolveRuntimeClasspathEntries(IRuntimeClasspathEntry[] pEntries,
      ILaunchConfiguration pConfig) throws CoreException
  {
    IRuntimeClasspathEntry[] resolvedEntries = JavaRuntime.resolveRuntimeClasspath(pEntries, pConfig);
    List stringEntries = new ArrayList(resolvedEntries.length);
    for (int i = 0; i < resolvedEntries.length; i++)
    {
      String location = resolvedEntries[i].getLocation();
      if (location != null)
      {
        stringEntries.add(location);
      }
    }

    return (String[]) stringEntries.toArray(new String[stringEntries.size()]);
  }

  private static String getString(String[] pStrings, String pSeparator)
  {
    StringBuffer sb = new StringBuffer();

    if (pStrings != null)
    {
      for (int i = 0, sizei = pStrings.length; i < sizei; ++i)
      {
        sb.append(pStrings[i]);
        if (i < (sizei - 1))
        {
          sb.append(pSeparator);
        }
      }
    }

    return sb.toString();
  }
}