/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.launching.provider;


import groovy.lang.Binding;
import groovy.lang.GroovyShell;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jdt.launching.IRuntimeClasspathEntry;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.JavaRuntime;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.launching.ITomcatLocation;
import org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration;
import org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfigurationProvider;
import org.exoplatform.eclipse.core.launching.VMArgument;
import org.exoplatform.eclipse.core.launching.VMArgumentType;
import org.exoplatform.eclipse.core.launching.VMArgumentsContainer;
import org.osgi.framework.Bundle;

/**
 * DOCUMENT ME!
 * 
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class GenericTomcatRuntimeConfigurationProvider implements ITomcatRuntimeConfigurationProvider,
    IExecutableExtension
{
  public static final String CLASS_VERSION = "$Id: GenericTomcatRuntimeConfigurationProvider.java,v 1.1 2004/09/20 01:39:14 hatimk Exp $"; //$NON-NLS-1$
  private String mConfigurationId = null;
  private String mConfigurationName = null;
  private URL mGroovyScriptLocation = null;

  //DO NOT FORGET everytime you add a new instance variable make sure you list it in the
  // reset method.
  private ITomcatLocation mTomcatLocation = null;
  private IVMInstall mVMInstall = null;
  private String mMainTypeName = null;
  private List mStartupProgramArguments = null;
  private List mShutdownProgramArguments = null;
  private String mWorkingDirectory = null;
  private List mBinLibraries = null;
  private List mServerLibraries = null;
  private List mCommonLibraries = null;
  private List mSharedLibraries = null;
  private List mServerWebappsLibraries = null;
  private List mDefaultWebappsLibraries = null;
  private List mClasspathLibraries = null;
  private List mEndorsedLibraries = null;
  private List mVMArguments = null;

  /**
   * @see org.eclipse.core.runtime.IExecutableExtension#setInitializationData(org.eclipse.core.runtime.IConfigurationElement,
   *      java.lang.String, java.lang.Object)
   */
  public void setInitializationData(IConfigurationElement pConfig, String pPropertyName, Object pData)
      throws CoreException
  {
    mConfigurationId = pConfig.getAttribute("configuration-id");
    mConfigurationName = pConfig.getAttribute("configuration-name");

    String groovylocationStr = pConfig.getAttribute("tomcat-groovy-script");
    String bundleName = pConfig.getDeclaringExtension().getNamespace();
    Bundle bundle = Platform.getBundle(bundleName);
    URL groovyLocationURL = bundle.getEntry(groovylocationStr);
    URL groovyLocation = null;
    try
    {
      groovyLocation = Platform.asLocalURL(groovyLocationURL);
    }
    catch (Exception e)
    {
      ExoCorePlugin.logError("Encountered an error while trying to determine the location of the groovy script file to figure out Tomcat configuration.", e);
    }

    if (groovyLocation != null)
    {
      mGroovyScriptLocation = groovyLocation;
    }
  }

  /**
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfigurationProvider#getConfiguration(java.lang.String,
   *      org.eclipse.jdt.launching.IVMInstall)
   */
  public synchronized ITomcatRuntimeConfiguration getConfiguration(ITomcatLocation pTomcatLocation,
      IVMInstall pVMInstall) throws CoreException
  {
    prepareLibraries(pTomcatLocation, pVMInstall);

    String[] startupArguments = convertToStringArray(mStartupProgramArguments);
    String[] shutdownArguments = convertToStringArray(mShutdownProgramArguments);
    IPath workingDirectory = new Path(mWorkingDirectory);
    IRuntimeClasspathEntry[] binLibraries = convertToRuntimeClasspathEntries(mBinLibraries);
    IRuntimeClasspathEntry[] serverLibraries = convertToRuntimeClasspathEntries(mServerLibraries);
    IRuntimeClasspathEntry[] commonLibraries = convertToRuntimeClasspathEntries(mCommonLibraries);
    IRuntimeClasspathEntry[] sharedLibraries = convertToRuntimeClasspathEntries(mSharedLibraries);
    IRuntimeClasspathEntry[] serverWebappsLibraries = convertToRuntimeClasspathEntries(mServerWebappsLibraries);
    IRuntimeClasspathEntry[] defaultWebappsLibraries = convertToRuntimeClasspathEntries(mDefaultWebappsLibraries);
    IRuntimeClasspathEntry[] classpathLibraries = convertToRuntimeClasspathEntries(mClasspathLibraries);
    IRuntimeClasspathEntry[] endorsedLibraries = convertToRuntimeClasspathEntries(mEndorsedLibraries);
    VMArgument[] VMArguments = convertToVMArguments(mVMArguments);

    ITomcatRuntimeConfiguration config = new TomcatRuntimeConfiguration(mConfigurationId, mConfigurationName,
        mMainTypeName, startupArguments, shutdownArguments, workingDirectory, binLibraries, serverLibraries,
        commonLibraries, sharedLibraries, serverWebappsLibraries, defaultWebappsLibraries,
        classpathLibraries, endorsedLibraries, VMArguments);

    return config;
  }

  private void prepareLibraries(ITomcatLocation pTomcatLocation, IVMInstall pVMInstall) throws CoreException
  {
    // lets determine if we need to recompute everything.
    if ((mTomcatLocation == null) || (mVMInstall == null) || (!mTomcatLocation.equals(pTomcatLocation))
        || (mVMInstall != pVMInstall))
    {
      // yes we need to recompute
      reset();
      initializeConfiguration(pTomcatLocation, pVMInstall);
      mTomcatLocation = pTomcatLocation;
      mVMInstall = pVMInstall;
    }
  }

  private void reset()
  {
    mTomcatLocation = null;
    mVMInstall = null;
    mMainTypeName = null;
    mStartupProgramArguments = null;
    mShutdownProgramArguments = null;
    mWorkingDirectory = null;
    mBinLibraries = null;
    mServerLibraries = null;
    mCommonLibraries = null;
    mSharedLibraries = null;
    mServerWebappsLibraries = null;
    mDefaultWebappsLibraries = null;
    mClasspathLibraries = null;
    mEndorsedLibraries = null;
    mVMArguments = null;
  }

  /**
   * This method runs the given groovy script using the given binding. The scirpt will be
   * run with the context class loader set to match this class classloader.
   * 
   * @param pScriptLocation URL of the groovy script
   * @param pBinding the binding will all the neccessary variables already set
   * 
   * @return the result of evaluating the given groovy script
   * 
   * @throws CoreException when unable to process or locate the script
   */
  protected Object runScript(URL pScriptLocation, Binding pBinding) throws CoreException
  {
    ClassLoader previous = Thread.currentThread().getContextClassLoader();

    try
    {
      //Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());

      try
      {
        String script = readScript(pScriptLocation);

        GroovyShell shell = new GroovyShell(this.getClass().getClassLoader(),pBinding);
        Object scriptvalue = shell.evaluate(script);

        return scriptvalue;
      }
      catch (IOException ioe)
      {
        throw new CoreException(
            ExoCorePlugin.createErrorStatus("Encountered a problem while trying to read groovy script file ["
                + pScriptLocation + "].", ioe));
      }
      catch (Exception e)
      {
        throw new CoreException(
            ExoCorePlugin.createErrorStatus("Encountered a problem while processing groovy script file ["
                + pScriptLocation + "].", e));
      }
    }
    finally
    {
      if (previous != null)
      {
        //Thread.currentThread().setContextClassLoader(previous);
      }
    }
  }

  private void initializeConfiguration(ITomcatLocation pTomcatLocation, IVMInstall pVMInstall)
      throws CoreException
  {
    Binding binding = new Binding();
    binding.setVariable("TOMCATHOME", pTomcatLocation.getTomcatHome());
    binding.setVariable("TOMCATSRC", pTomcatLocation.getTomcatSrc());
    binding.setVariable("VMInstall", pVMInstall);

    Object scriptvalue = runScript(mGroovyScriptLocation, binding);

    mMainTypeName = (String) binding.getVariable("mainTypeName");
    mStartupProgramArguments = (List) binding.getVariable("startupArguments");
    mShutdownProgramArguments = (List) binding.getVariable("shutdownArguments");
    mWorkingDirectory = (String) binding.getVariable("suggestedWorkingDirectory");
    mBinLibraries = (List) binding.getVariable("binLibraries");
    mServerLibraries = (List) binding.getVariable("serverLibraries");
    mCommonLibraries = (List) binding.getVariable("commonLibraries");
    mSharedLibraries = (List) binding.getVariable("sharedLibraries");
    mServerWebappsLibraries = (List) binding.getVariable("serverWebappsLibraries");
    mDefaultWebappsLibraries = (List) binding.getVariable("defaultWebappsLibraries");
    mClasspathLibraries = (List) binding.getVariable("classpathLibraries");
    mEndorsedLibraries = (List) binding.getVariable("endorsedLibraries");
    mVMArguments = (List) binding.getVariable("VMArguments");
  }

  protected static String[] convertToStringArray(List pList)
  {
    if (pList == null)
    {
      return new String[0];
    }

    String[] resultarray = new String[pList.size()];

    return (String[]) pList.toArray(resultarray);
  }

  protected static IRuntimeClasspathEntry[] convertToRuntimeClasspathEntries(List pList)
  {
    List result = new ArrayList();

    if (pList != null)
    {
      Iterator iterator = pList.iterator();
      while (iterator.hasNext())
      {
        Map map = (Map) iterator.next();
        String classesStr = (String) map.get("classes");
        String srcStr = (String) map.get("src");
        String srcrootStr = (String) map.get("src-root");

        IPath classesPath = (classesStr == null) ? Path.EMPTY : new Path(classesStr);
        IPath srcPath = (srcStr == null) ? null : new Path(srcStr);
        IPath srcrootPath = (srcrootStr == null) ? null : new Path(srcrootStr);

        IRuntimeClasspathEntry entry = JavaRuntime.newArchiveRuntimeClasspathEntry(classesPath);
        if (srcPath != null)
        {
          entry.setSourceAttachmentPath(srcPath);
        }
        if (srcrootPath != null)
        {
          entry.setSourceAttachmentRootPath(srcrootPath);
        }

        result.add(entry);
      }
    }

    IRuntimeClasspathEntry[] resultarray = new IRuntimeClasspathEntry[result.size()];

    return (IRuntimeClasspathEntry[]) result.toArray(resultarray);
  }

  protected static VMArgument[] convertToVMArguments(List pList)
  {
    VMArgumentsContainer container = new VMArgumentsContainer();

    if (pList != null)
    {
      Iterator iterator = pList.iterator();
      while (iterator.hasNext())
      {
        Map map = (Map) iterator.next();
        String key = (String) map.get("key");
        String value = (String) map.get("value");
        String type = (String) map.get("type");

        VMArgument entry = new VMArgument(key, VMArgumentType.parse(type));
        entry.appendValue(value);
        container.appendVMArgumentValues(entry);
      }
    }

    return container.getVMArguments();
  }

  protected static String readScript(URL pURL) throws IOException
  {
    StringBuffer sb = new StringBuffer();
    URLConnection filestream = pURL.openConnection();
    InputStream input = filestream.getInputStream();
    BufferedReader reader = new BufferedReader(new InputStreamReader(input));

    int c;
    while ((c = reader.read()) != -1)
    {
      sb.append((char) c);
    }

    return sb.toString();
  }

  /**
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getConfigurationId()
   */
  public String getConfigurationId()
  {
    return mConfigurationId;
  }

  /**
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getConfigurationName()
   */
  public String getConfigurationName()
  {
    return mConfigurationName;
  }

}