/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation;


import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.exoplatform.eclipse.core.IPreferencesConstant;
import org.exoplatform.eclipse.core.common.IOverwriteQuery;
import org.exoplatform.eclipse.core.filesprovider.ProjectFilesProvider;
import org.exoplatform.eclipse.core.operation.helper.ProjectSettings;
import org.exoplatform.eclipse.core.operation.helper.WebappProjectSettings;

import java.lang.reflect.InvocationTargetException;

/**
 * Use this operation when you want to mount a web content folder to an existing folder.
 * This operation takes care of the following:
 * <UL>
 * <LI>It will create all the default subfolders and files for a standard portlet project
 * (given via an extension point), such as portlet.xml, portlet.tld, etc.
 * <LI>If indicated, it will make WEB-INF/classes the default binary folder, and it will
 * make it a drived resource
 * <LI>If indicated, it will search all jars in WEB-INF/lib and add them to the
 * classpath.
 * <LI>It will update .exo file accordingly.
 * </UL>
 * 
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ChangeWebFolderOperation extends WorkspaceModifyOperation
{
  public static final String CLASS_VERSION = "$Id: ChangeWebFolderOperation.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$
  private IProject mProject;
  private WebappProjectSettings mSettings;
  private IOverwriteQuery mQuery;
  private boolean mMakeDefaultBinary;
  private boolean mAddJarsToClasspath;

  /**
   * The constructor method.
   * 
   * @param pProject the project in question
   * @param pSettings the settings (most likely got generated by SampleWebappInitializer
   * @param pMakeDefaultBinary if true then the WEB-INF/classes will be made the default
   *          binary folder
   * @param pAddJarsToClasspath if true then all jars in WEB-INF/lib will be added to the
   *          classpath
   * @param pQuery do you want to overwrite files or not
   */
  public ChangeWebFolderOperation(IProject pProject, WebappProjectSettings pSettings,
      boolean pMakeDefaultBinary, boolean pAddJarsToClasspath, IOverwriteQuery pQuery)
  {
    mProject = pProject;
    mSettings = pSettings;
    mQuery = pQuery;
    mMakeDefaultBinary = pMakeDefaultBinary;
    mAddJarsToClasspath = pAddJarsToClasspath;
  }

  /**
   * @see org.exoplatform.eclipse.core.operation.WorkspaceModifyOperation#execute(org.eclipse.core.runtime.IProgressMonitor)
   */
  protected void execute(IProgressMonitor pMonitor) throws CoreException, InvocationTargetException,
      InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      pMonitor.beginTask("Modifying web content settings : ", 4000);
      pMonitor.subTask("importing files");
      importFiles(mSettings, mProject, mQuery, new SubProgressMonitor(pMonitor, 1000));

      // deal with the default binary folder
      if (mMakeDefaultBinary)
      {
        assignBinaryDefault(new SubProgressMonitor(pMonitor, 1000));
      }
      else
      {
        pMonitor.worked(1000);
      }

      // deal with adding the jar files to the class path
      if (mAddJarsToClasspath)
      {
        addJarsToClasspath(new SubProgressMonitor(pMonitor, 1000));
      }
      else
      {
        pMonitor.worked(1000);
      }

      // update the project scope settings file
      updatePortletStore(new SubProgressMonitor(pMonitor, 1000));
    }
    finally
    {
      pMonitor.done();
    }
  }

  private void updatePortletStore(IProgressMonitor pMonitor) throws CoreException, InvocationTargetException,
      InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }
    try
    {
      pMonitor.beginTask("update .settings", 1000);

      IPath webpath = mSettings.getParentFolder();

      if (webpath == null)
      {
        webpath = mSettings.getWebFolder();
      }
      else
      {
        webpath = webpath.append(mSettings.getWebFolder());
      }

      // now add the web folder to the portlet project xml store
      SimplePortletProjectStoreOperation storeOp = new SimplePortletProjectStoreOperation(mProject);
      storeOp.addPropertyToModify(IPreferencesConstant.PORTLET_PROJECT_STORE_WEBFOLDER, webpath.toString());
      storeOp.run(new SubProgressMonitor(pMonitor, 1000));
    }
    finally
    {
      pMonitor.done();
    }
  }

  private void addJarsToClasspath(IProgressMonitor pMonitor) throws CoreException, InvocationTargetException,
      InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }
    try
    {
      pMonitor.beginTask("adding all jars to the classpath", 1000);

      IJavaProject javaProject = JavaCore.create(mProject);
      IPath jarpath = mSettings.getParentFolder();
      if (jarpath == null)
      {
        jarpath = mSettings.getLibraryFolder();
      }
      else
      {
        jarpath = jarpath.append(mSettings.getLibraryFolder());
      }

      IFolder libFolder = createFolder(mProject, jarpath, new SubProgressMonitor(pMonitor, 500));
      if (libFolder != null)
      {
        AddFolderToClasspathOperation addOp = new AddFolderToClasspathOperation(javaProject, libFolder, true);
        addOp.run(new SubProgressMonitor(pMonitor, 500));
      }
    }
    finally
    {
      pMonitor.done();
    }
  }

  private void assignBinaryDefault(IProgressMonitor pMonitor) throws CoreException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }
    try
    {
      pMonitor.beginTask("changing the default binary folder", 1000);

      IJavaProject javaProject = JavaCore.create(mProject);
      IPath webpath = mSettings.getParentFolder();
      if (webpath == null)
      {
        webpath = mSettings.getBinaryFolder();
      }
      else
      {
        webpath = webpath.append(mSettings.getBinaryFolder());
      }

      IFolder defaultBinaryFolder = createFolder(mProject, webpath, new SubProgressMonitor(pMonitor, 500));
      if (defaultBinaryFolder != null)
      {
        pMonitor.subTask("make the java project aware of the default binary folder");
        javaProject.setOutputLocation(defaultBinaryFolder.getFullPath(),
            new SubProgressMonitor(pMonitor, 500));
        defaultBinaryFolder.setDerived(true);
      }
    }
    finally
    {
      pMonitor.done();
    }
  }

  protected void importFiles(ProjectSettings pSettings, IProject pProject, IOverwriteQuery pOverwriteQuery,
      IProgressMonitor pMonitor) throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      ProjectFilesProvider[] providers = pSettings.getProjectFilesProviders();

      if ((providers == null) || (providers.length < 1))
      {
        pMonitor.beginTask("", 1); //$NON-NLS-1$

        return;
      }

      pMonitor.beginTask("", providers.length * 100); //$NON-NLS-1$
      for (int i = 0, sizei = providers.length; i < sizei; ++i)
      {
        importFiles(providers[i], pProject, pSettings.getParentFolder(), pOverwriteQuery,
            new SubProgressMonitor(pMonitor, 100));
        if (pMonitor.isCanceled())
        {
          throw new InterruptedException();
        }
      }
    }
    finally
    {
      pMonitor.done();
    }
  }

  protected void importFiles(ProjectFilesProvider pProvider, IProject pProject, IPath pParentFolder,
      IOverwriteQuery pOverwriteQuery, IProgressMonitor pMonitor) throws CoreException,
      InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      pMonitor.beginTask("", IProgressMonitor.UNKNOWN); //$NON-NLS-1$

      IPath parentPath;
      if (pParentFolder == null)
      {
        parentPath = pProject.getFullPath();
      }
      else
      {
        // recursivlely detect if the parent folder already exists, if not then create it
        IFolder parentFolder = createFolder(pProject, pParentFolder, new SubProgressMonitor(pMonitor, 1));
        parentPath = parentFolder.getFullPath();
      }

      ImportOperation op = new ImportOperation(parentPath, pProvider.getSource(), pProvider
          .getStructureProvider(), pOverwriteQuery);
      op.run(new SubProgressMonitor(pMonitor, IProgressMonitor.UNKNOWN));
    }
    finally
    {
      pMonitor.done();
    }
  }

  /**
   * Given a project and a folder (a path relative to the project), recursively creates
   * the folder if it does not exist. The folder can be of any depth. If the folder is
   * null or empty, null is returned.
   * 
   * @param pProject the project to create the folder in
   * @param pFolder a path (relative to the project)
   * @param pMonitor to report progress
   * 
   * @return the folder that was created (if need be) by this method
   * 
   * @throws CoreException if the folder creation has a problem
   */
  protected IFolder createFolder(IProject pProject, IPath pFolder, IProgressMonitor pMonitor)
      throws CoreException
  {
    if ((pFolder == null) || (pFolder.segmentCount() < 1))
    {
      return null;
    }
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      pMonitor.beginTask("", IProgressMonitor.UNKNOWN); //$NON-NLS-1$

      // recursively detect if the folder already exists, if not then create it
      IFolder parentFolder = pProject.getFolder(pFolder.segment(0));

      if (!parentFolder.exists())
      {
        pMonitor.subTask("create " + parentFolder.toString());
        parentFolder.create(false, true, new SubProgressMonitor(pMonitor, 1));
      }

      for (int i = 1; i < pFolder.segmentCount(); i++)
      {
        parentFolder = parentFolder.getFolder(new Path(pFolder.segment(i)));
        if (!parentFolder.exists())
        {
          pMonitor.subTask("create " + parentFolder.toString());
          parentFolder.create(false, true, new SubProgressMonitor(pMonitor, 1));
        }
      }

      return parentFolder;
    }
    finally
    {
      pMonitor.done();
    }
  }
}