/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation;


import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.common.IOverwriteQuery;
import org.exoplatform.eclipse.core.operation.helper.JavaProjectConfiguration;
import org.exoplatform.eclipse.core.operation.helper.JavaProjectSettings;

import java.lang.reflect.InvocationTargetException;

/**
 * An operation which creates a new java project and populates it according to
 * the given java project configuration. This includes setting up the
 * libraries, the source folders, and the binary folders.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class JavaProjectCreateOperation extends ProjectCreateOperation
{
  public static final String CLASS_VERSION =
    "$Id: JavaProjectCreateOperation.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$
  private JavaProjectConfiguration mProjectConfig;
  private IJavaProject mJavaProject;

  /**
   * The constructor method.
   *
   * @param pConfig A configuration that consists of the project name, the
   *        project path, and one or more project settings.
   * @param pProjectName the project name (can contain spaces)
   * @param pProjectPath the project path. If the path is null, then it means
   *        that the project name is to be used as the path (relative to the
   *        workspace). For example, if the given path is null, and the
   *        project name is 'test', then it should be interpreted  as a
   *        project 'test' in the workspace.
   * @param pOverwriteQuery the implementation of this interface should answer
   *        one of the prescribed return codes when asked whether to overwrite
   *        a certain path string (which could represent a resource path, a
   *        file system path, etc). The above comment was copied from the
   *        IOverwriteQuery interface comment.
   */
  public JavaProjectCreateOperation(JavaProjectConfiguration pConfig,
    String pProjectName, IPath pProjectPath, IOverwriteQuery pOverwriteQuery)
  {
    super(pConfig, pProjectName, pProjectPath, pOverwriteQuery);
    mProjectConfig = pConfig;
  }

  /**   
   * Returns a handler to the newly created and configured java project. The
   * handler is only available after the run() method has been called once.
   *
   * @return a handler to the newly created and configured java project
   */
  public IJavaProject getJavaProject()
  {
    return mJavaProject;
  }

  /**   
   * @see org.eclipse.ui.actions.WorkspaceModifyOperation#execute(org.eclipse.core.runtime.IProgressMonitor)
   */
  protected void execute(IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    pMonitor.beginTask("creating a java project : ", 3000);
    pMonitor.subTask("create the standard project itself");
    super.execute(new SubProgressMonitor(pMonitor, 100));
    pMonitor.subTask("upgrade to java project");
    mJavaProject =
      upgradeToJavaProject(getProject(), new SubProgressMonitor(pMonitor, 1900));
    pMonitor.subTask("assign jar files to the classpath");
    addLibraryFolders(mProjectConfig, new SubProgressMonitor(pMonitor, 500));
    pMonitor.subTask("assign source folders and binary folders");
    addSourceAndBinary(mProjectConfig, new SubProgressMonitor(pMonitor, 500));
    pMonitor.done();
  }

  protected IJavaProject upgradeToJavaProject(IProject pProject,
    IProgressMonitor pMonitor) throws CoreException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    pMonitor.beginTask("converting the project to java project", 120);
    pMonitor.subTask("process the project description and adds the java nature");

    IProjectDescription description = pProject.getDescription();
    description.setNatureIds(new String[] { JavaCore.NATURE_ID });
    pProject.setDescription(description, new SubProgressMonitor(pMonitor, 50));

    if (pMonitor.isCanceled())
    {
      throw new InterruptedException();
    }

    pMonitor.subTask("create the java project by calling the java core plugin");

    IJavaProject javaProject = JavaCore.create(pProject);
    javaProject.setRawClasspath(mProjectConfig.getProjectLibraries(),
      new SubProgressMonitor(pMonitor, 50));

    // deal with the default binary folder if it exists
    IPath defaultBinary = mProjectConfig.getDefaultBinaryFolder();
    pMonitor.subTask("create the default binary folder");

    IFolder defaultBinaryFolder =
      createFolder(getProject(), defaultBinary,
        new SubProgressMonitor(pMonitor, 10));
    if (defaultBinaryFolder != null)
    {
      pMonitor.subTask(
        "make the java project aware of the default binary folder");
      javaProject.setOutputLocation(defaultBinaryFolder.getFullPath(),
        new SubProgressMonitor(pMonitor, 10));
      defaultBinaryFolder.setDerived(true);
    }

    pMonitor.done();

    return javaProject;
  }

  protected void addSourceAndBinary(JavaProjectConfiguration pConfig,
    IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      if ((pConfig == null) || (pConfig.getProjectSettings().length < 1))
      {
        pMonitor.beginTask("", 1); //$NON-NLS-1$

        return;
      }

      JavaProjectSettings[] settings = pConfig.getJavaProjectSettings();
      pMonitor.beginTask("", settings.length * 1000); //$NON-NLS-1$
      for (int i = 0, sizei = settings.length; i < sizei; ++i)
      {
        addSourceAndBinary(settings[i], new SubProgressMonitor(pMonitor, 1000));
        if (pMonitor.isCanceled())
        {
          throw new InterruptedException();
        }
      }
    }
    finally
    {
      pMonitor.done();
    }
  }

  protected void addSourceAndBinary(JavaProjectSettings pSettings,
    IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      IPath sourcePath = pSettings.getSourceFolder();
      IPath binaryPath = pSettings.getBinaryFolder();
      IPath parentPath = pSettings.getParentFolder();
      IPath projectPath = getProject().getFullPath();
      IPath fullSourcePath =
        getPathRelativeToProject(projectPath, parentPath, sourcePath);
      IPath fullBinaryPath =
        getPathRelativeToProject(projectPath, parentPath, binaryPath);

      // simply without a source folder we can't proceed
      if (fullSourcePath == null)
      {
        pMonitor.beginTask("", 1); //$NON-NLS-1$

        return;
      }

      pMonitor.beginTask("", 2); //$NON-NLS-1$

      // create the source and binary folder if they don't exist
      pMonitor.subTask("create the source folder");
      createFolder(getProject(), fullSourcePath.removeFirstSegments(1),
        new SubProgressMonitor(pMonitor, 1));
      pMonitor.subTask("create the binary folder");
      createFolder(getProject(), fullBinaryPath.removeFirstSegments(1),
        new SubProgressMonitor(pMonitor, 1));

      IClasspathEntry src;
      if ((fullBinaryPath == null) || (!ExoCorePlugin.ALLOW_MULTIPLE_OUTPUT))
      {
        src = JavaCore.newSourceEntry(fullSourcePath);
      }
      else
      {
        src =
          JavaCore.newSourceEntry(fullSourcePath, new IPath[0], fullBinaryPath);
      }

      addToClasspath(src, getJavaProject());
    }
    finally
    {
      pMonitor.done();
    }
  }

  protected void addLibraryFolders(JavaProjectConfiguration pConfig,
    IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      if ((pConfig == null) || (pConfig.getProjectSettings().length < 1))
      {
        pMonitor.beginTask("", 1); //$NON-NLS-1$

        return;
      }

      JavaProjectSettings[] settings = pConfig.getJavaProjectSettings();
      pMonitor.beginTask("", settings.length * 1000); //$NON-NLS-1$
      for (int i = 0, sizei = settings.length; i < sizei; ++i)
      {
        addLibraryFolder(settings[i], new SubProgressMonitor(pMonitor, 1000));
        if (pMonitor.isCanceled())
        {
          throw new InterruptedException();
        }
      }
    }
    finally
    {
      pMonitor.done();
    }
  }

  protected void addLibraryFolder(JavaProjectSettings pSettings,
    IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      IPath libraryPath = pSettings.getLibraryFolder();
      IPath parentPath = pSettings.getParentFolder();
      IPath projectPath = getProject().getFullPath();
      IPath fullLibraryPath =
        getPathRelativeToProject(projectPath, parentPath, libraryPath);

      // simply without a library folder we can't proceed
      if (fullLibraryPath == null)
      {
        pMonitor.beginTask("", 1); //$NON-NLS-1$

        return;
      }

      pMonitor.beginTask("", 100); //$NON-NLS-1$

      // create the library folder if they don't exist
      pMonitor.subTask("create the library folder");

      IFolder libraryFolder =
        createFolder(getProject(), fullLibraryPath.removeFirstSegments(1),
          new SubProgressMonitor(pMonitor, 1));
      pMonitor.subTask(
        "assign all jar files in the library folder to the classpath");

      AddFolderToClasspathOperation addOp =
        new AddFolderToClasspathOperation(getJavaProject(), libraryFolder, true);
      addOp.run(new SubProgressMonitor(pMonitor, 99));
    }
    finally
    {
      pMonitor.done();
    }
  }

  /**   
   * Given a project path and a parent path and a folder path, concatenate all
   * of them. However if the folder path is null then return null.
   *
   * @param pProject a project path
   * @param pParent a parent path
   * @param pFolder a folder path
   *
   * @return concatenated path, null if folder is null or empty
   */
  protected IPath getPathRelativeToProject(IPath pProject, IPath pParent,
    IPath pFolder)
  {
    IPath result = pProject;
    if ((pParent != null) && (pParent.segmentCount() > 0))
    {
      result = result.append(pParent);
    }

    if ((pFolder != null) && (pFolder.segmentCount() > 0))
    {
      result = result.append(pFolder);
    }
    else
    {
      result = null;
    }

    return result;
  }

  protected void addToClasspath(IClasspathEntry pEntry, IJavaProject pProject)
    throws JavaModelException
  {
    IClasspathEntry[] oldEntries = pProject.getRawClasspath();
    IClasspathEntry[] newEntries = new IClasspathEntry[oldEntries.length + 1];
    System.arraycopy(oldEntries, 0, newEntries, 0, oldEntries.length);
    newEntries[oldEntries.length] = pEntry;
    pProject.setRawClasspath(newEntries, new NullProgressMonitor());
  }
}
