/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation;


import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.exoplatform.eclipse.core.common.IOverwriteQuery;
import org.exoplatform.eclipse.core.filesprovider.ProjectFilesProvider;
import org.exoplatform.eclipse.core.operation.helper.ProjectConfiguration;
import org.exoplatform.eclipse.core.operation.helper.ProjectSettings;

import java.lang.reflect.InvocationTargetException;

/**
 * An operation which creates a new project and populates it according to the
 * given project configuration.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ProjectCreateOperation extends WorkspaceModifyOperation
{
  public static final String CLASS_VERSION =
    "$Id: ProjectCreateOperation.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$
  private String mProjectName;
  private IPath mProjectPath;
  private ProjectConfiguration mProjectConfig;
  private IOverwriteQuery mOverwriteQuery;
  private IProject mProjectHandle;

  /**
   * The constructor method.
   *
   * @param pConfig A configuration that consists of the project name, the
   *        project path, and one or more project settings.
   * @param pProjectName the project name (can contain spaces)
   * @param pProjectPath the project path. If the path is null, then it means
   *        that the project name is to be used as the path (relative to the
   *        workspace). For example, if the given path is null, and the
   *        project name is 'test', then it should be interpreted  as a
   *        project 'test' in the workspace.
   * @param pOverwriteQuery the implementation of this interface should answer
   *        one of the prescribed return codes when asked whether to overwrite
   *        a certain path string (which could represent a resource path, a
   *        file system path, etc). The above comment was copied from the
   *        IOverwriteQuery interface comment.
   */
  public ProjectCreateOperation(ProjectConfiguration pConfig,
    String pProjectName, IPath pProjectPath, IOverwriteQuery pOverwriteQuery)
  {
    mProjectConfig = pConfig;
    mProjectName = pProjectName;
    mProjectPath = pProjectPath;
    mOverwriteQuery = pOverwriteQuery;
  }

  /**   
   * Returns a handler to the newly created and configured project. The handler
   * is only available after the run() method has been called once.
   *
   * @return a handler to the newly created and configured project
   */
  public IProject getProject()
  {
    return mProjectHandle;
  }

  /**   
   * Returns the project name that this class represents. The project name can
   * contain spaces.
   *
   * @return the project name that this class represents. The project name can
   *         contain spaces.
   */
  public String getProjectName()
  {
    return mProjectName;
  }

  /**   
   * Returns the project path.
   *
   * @return the project path. If the path is null, then it means that the
   *         project name is to be used as the path (relative to the
   *         workspace). For example, if the path is null, and the project
   *         name is 'test', then it should be interpreted  as a project
   *         'test' in the workspace.
   */
  public IPath getProjectPath()
  {
    return mProjectPath;
  }

  /**   
   * @see org.eclipse.ui.actions.WorkspaceModifyOperation#execute(org.eclipse.core.runtime.IProgressMonitor)
   */
  protected void execute(IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    pMonitor.beginTask("Creating a project : ", 2000);
    pMonitor.subTask("create the project itself");
    mProjectHandle = createProject(new SubProgressMonitor(pMonitor, 100));
    pMonitor.subTask("import files to the project");
    populateProject(mProjectConfig, new SubProgressMonitor(pMonitor, 1900));
    pMonitor.done();
  }

  /**   
   * Creates a project.
   *
   * @param pMonitor the progress monitor to report progress and to detect
   *        cancellation
   *
   * @return a workspace resource of type  project which groups resources into
   *         buildable, reusable units
   *
   * @throws CoreException if the operation fails due to a CoreException
   * @throws InterruptedException this method throws this exception if it
   *         detects that pMonitor.isCanceled() occured.
   */
  protected IProject createProject(IProgressMonitor pMonitor)
    throws CoreException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      pMonitor.beginTask("", 2000); //$NON-NLS-1$
      pMonitor.subTask("create .project file");

      IProject projectHandle =
        ResourcesPlugin.getWorkspace().getRoot().getProject(getProjectName());

      IWorkspace workspace = ResourcesPlugin.getWorkspace();
      final IProjectDescription description =
        workspace.newProjectDescription(projectHandle.getName());

      description.setLocation(getProjectPath());

      if (pMonitor.isCanceled())
      {
        throw new InterruptedException();
      }
      projectHandle.create(description, new SubProgressMonitor(pMonitor, 1000));

      if (pMonitor.isCanceled())
      {
        throw new InterruptedException();
      }
      projectHandle.open(new SubProgressMonitor(pMonitor, 1000));

      return projectHandle;
    }
    finally
    {
      pMonitor.done();
    }
  }

  protected void populateProject(ProjectConfiguration pConfig,
    IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      if ((pConfig == null) || (pConfig.getProjectSettings().length < 1))
      {
        pMonitor.beginTask("", 1); //$NON-NLS-1$

        return;
      }

      ProjectSettings[] settings = pConfig.getProjectSettings();
      pMonitor.beginTask("", settings.length * 1000); //$NON-NLS-1$
      for (int i = 0, sizei = settings.length; i < sizei; ++i)
      {
        importFiles(settings[i], new SubProgressMonitor(pMonitor, 1000));
        if (pMonitor.isCanceled())
        {
          throw new InterruptedException();
        }
      }
    }
    finally
    {
      pMonitor.done();
    }
  }

  protected void importFiles(ProjectSettings pSettings,
    IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      ProjectFilesProvider[] providers = pSettings.getProjectFilesProviders();

      if ((providers == null) || (providers.length < 1))
      {
        pMonitor.beginTask("", 1); //$NON-NLS-1$

        return;
      }

      pMonitor.beginTask("", providers.length * 100); //$NON-NLS-1$
      for (int i = 0, sizei = providers.length; i < sizei; ++i)
      {
        importFiles(providers[i], pSettings.getParentFolder(), mOverwriteQuery,
          new SubProgressMonitor(pMonitor, 100));
        if (pMonitor.isCanceled())
        {
          throw new InterruptedException();
        }
      }
    }
    finally
    {
      pMonitor.done();
    }
  }

  protected void importFiles(ProjectFilesProvider pProvider,
    IPath pParentFolder, IOverwriteQuery pOverwriteQuery,
    IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      pMonitor.beginTask("", IProgressMonitor.UNKNOWN); //$NON-NLS-1$

      IPath parentPath;
      if (pParentFolder == null)
      {
        parentPath = mProjectHandle.getFullPath();
      }
      else
      {
        // recursivlely detect if the parent folder already exists, if not then create it
        IFolder parentFolder =
          createFolder(mProjectHandle, pParentFolder,
            new SubProgressMonitor(pMonitor, 1));
        parentPath = parentFolder.getFullPath();
      }

      ImportOperation op =
        new ImportOperation(parentPath, pProvider.getSource(),
          pProvider.getStructureProvider(), pOverwriteQuery);
      op.run(new SubProgressMonitor(pMonitor, IProgressMonitor.UNKNOWN));
    }
    finally
    {
      pMonitor.done();
    }
  }

  /**   
   * Given a project and a folder (a path relative to the project), recursively
   * creates the folder if it does not exist. The folder can be of any depth.
   * If the folder is null or empty, null is returned.
   *
   * @param pProject the project to create the folder in
   * @param pFolder a path (relative to the project)
   * @param pMonitor to report progress
   *
   * @return the folder that was created (if need be) by this method
   *
   * @throws CoreException if the folder creation has a problem
   */
  protected IFolder createFolder(IProject pProject, IPath pFolder,
    IProgressMonitor pMonitor) throws CoreException
  {
    if ((pFolder == null) || (pFolder.segmentCount() < 1))
    {
      return null;
    }
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      pMonitor.beginTask("", IProgressMonitor.UNKNOWN); //$NON-NLS-1$

      // recursively  detect if the folder already exists, if not then create it
      IFolder parentFolder = mProjectHandle.getFolder(pFolder.segment(0));

      if (!parentFolder.exists())
      {
        pMonitor.subTask("create " + parentFolder.toString());
        parentFolder.create(false, true, new SubProgressMonitor(pMonitor, 1));
      }

      for (int i = 1; i < pFolder.segmentCount(); i++)
      {
        parentFolder = parentFolder.getFolder(new Path(pFolder.segment(i)));
        if (!parentFolder.exists())
        {
          pMonitor.subTask("create " + parentFolder.toString());
          parentFolder.create(false, true, new SubProgressMonitor(pMonitor, 1));
        }
      }

      return parentFolder;
    }
    finally
    {
      pMonitor.done();
    }
  }

  protected IOverwriteQuery getOverwriteQuery()
  {
    return mOverwriteQuery;
  }
}
