/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation;


import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.osgi.service.prefs.Preferences;

import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * Use this operation if you want to update one or more attributes in the project-scoped
 * preferences for the portlet project settings. The preferences contain information such
 * as the webfolder and the context root. The creating of the preferences under the
 * project is done by the resource plugin that comes with Eclipse 3.0. This feature was
 * not available before 3.0
 * 
 * @author Hatim Khan
 */
public class SimplePortletProjectStoreOperation extends WorkspaceModifyOperation
{
  private IProject mProject;
  private Map mPropertyMap = new HashMap();

  /**
   * The constructor method.
   * 
   * @param pProject the project to update its xml store
   */
  public SimplePortletProjectStoreOperation(IProject pProject)
  {
    mProject = pProject;
  }

  /**
   * Use this method to indicate which attirbutes in the xml store you want to update.
   * This will only update attribute values that are direct elements of the root.
   * 
   * @param pProperty the name of the element tag
   * @param pValue its body content
   */
  public void addPropertyToModify(String pProperty, String pValue)
  {
    mPropertyMap.put(pProperty, pValue);
  }

  /**
   * @see org.exoplatform.eclipse.core.operation.WorkspaceModifyOperation#execute(org.eclipse.core.runtime.IProgressMonitor)
   */
  protected void execute(IProgressMonitor pMonitor) throws CoreException, InvocationTargetException,
      InterruptedException
  {
    if (pMonitor == null) pMonitor = new NullProgressMonitor();

    if ((mProject == null) || (mPropertyMap.isEmpty())) return;

    Preferences projectPreferences = ExoCorePlugin.getProjectScopePreferences(mProject);
    if (projectPreferences == null) return;

    Set keyset = mPropertyMap.keySet();
    pMonitor.beginTask("", (keyset.size() * 100));

    Iterator ikeyset = keyset.iterator();
    while (ikeyset.hasNext())
    {
      String key = (String) ikeyset.next();
      String value = (String) mPropertyMap.get(key);
      pMonitor
          .subTask("updating project-scoped preferences with key=[" + key + "] and value=[" + value + "]");

      if (pMonitor.isCanceled())
      {
        throw new InterruptedException();
      }

      setStringProperty(projectPreferences, key, value);
      pMonitor.worked(100);
    }

    if (pMonitor.isCanceled())
    {
      throw new InterruptedException();
    }

    saveProjectPreferences(projectPreferences);
    mPropertyMap.clear();
    pMonitor.done();
  }

  private void saveProjectPreferences(Preferences pProjectPreferences)
  {
    try
    {
      pProjectPreferences.flush();
      pProjectPreferences.sync();
    }
    catch (Exception e)
    {
      addError("could not save the project-scoped preferences", e);
      ExoCorePlugin.logError(e);
    }
  }

  private void setStringProperty(Preferences pProjectNode, String pProperty, String pValue)
  {
    try
    {
      if ((pValue == null) || (pValue.length() < 1))
      {
        pProjectNode.remove(pProperty);
      }
      else
      {
        pProjectNode.put(pProperty, pValue);
      }
    }
    catch (Exception e)
    {
      addError("could not set a property on the project-scoped preferences", e);
      ExoCorePlugin.logError(e);
    }
  }
}