/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.IPortletProject;
import org.exoplatform.eclipse.core.operation.helper.IWebXMLModifyStrategy;
import org.exoplatform.eclipse.core.runtime.IExoProjectRuntimeProperties;
import org.exoplatform.tools.text.TextDocument;
import org.exoplatform.tools.xml.ModificationTaskException;
import org.exoplatform.tools.xml.XMLModificationTask;
import org.exoplatform.tools.xml.XMLUtils;
import org.exoplatform.tools.xml.webapp.v23.ModifyWebXMLOperation;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * Modifies the web application deployment descriptor "web.xml" to satisfy eXo platfrom
 * requirement.
 * 
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class WebXMLModifyOperation extends WorkspaceModifyOperation
{
  public static final String CLASS_VERSION = "$Id: WebXMLModifyOperation.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$
  private IPortletProject mProject;
  private IWebXMLModifyStrategy mStrategy;
  private IFile mWebXML;
  private int mStartOffset;
  private int mEndOffset;

  /**
   * The constructor method.
   * 
   * @param pProject
   * @param pStrategy
   */
  public WebXMLModifyOperation(IPortletProject pProject, IWebXMLModifyStrategy pStrategy)
  {
    mProject = pProject;
    mStrategy = pStrategy;
  }

  /**
   * @see org.exoplatform.eclipse.core.operation.WorkspaceModifyOperation#execute(org.eclipse.core.runtime.IProgressMonitor)
   */
  protected void execute(IProgressMonitor pMonitor) throws CoreException, InvocationTargetException,
      InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    ClassLoader previous = Thread.currentThread().getContextClassLoader();

    try
    {
      // Since jdk 1.4 comes with xml-api.jar, the implementation of the XMLReaderFactory
      // will eventually try to use the thread class loader to locate the xerces
      // classes. However, since each plugin has its own class loader, we need to
      // associate the context class loader of the thread with the plugin class
      // loader. This way the xerces files will be available.
      Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
      runWithDifferentClassLoader(pMonitor);
    }
    finally
    {
      if (previous != null)
      {
        Thread.currentThread().setContextClassLoader(previous);
      }
    }
  }

  private void runWithDifferentClassLoader(IProgressMonitor pMonitor) throws InterruptedException,
      CoreException
  {
    if (mProject == null)
    {
      return;
    }

    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    if (!getDetectWebXMLOption())
    {
      return;
    }

    mWebXML = getWebXML();
    if (mWebXML == null)
    {
      return;
    }

    pMonitor.beginTask("Inspecting web.xml", 700);

    mWebXML.refreshLocal(IResource.DEPTH_ZERO, new SubProgressMonitor(pMonitor, 100));

    String contextRoot = getContextRoot();
    ModifyWebXMLOperation op = new ModifyWebXMLOperation(contextRoot);
    XMLModificationTask[] modificationList = op.getModificationTasks();

    pMonitor.subTask("reading web.xml");

    Document doc = loadDOMDocument(mWebXML.getContents());
    String encoding = getEncoding(doc);
    pMonitor.worked(100);

    pMonitor.subTask("analyzing web.xml");

    boolean needModification = modifyDocument(modificationList, doc, new SubProgressMonitor(pMonitor, 400));

    // give a chance to the strategy to tell us if we should continue or not
    // a UI caller, might want to prompt the user for such information
    if (needModification)
    {
      if (!mStrategy.isOkToModify(mWebXML))
      {
        pMonitor.done();

        return;
      }
    }
    else
    {
      pMonitor.done();

      return;
    }

    // since we are forcing to save we will not alert the user to save web.xml
    // if they have the editor open
    saveToFile(doc, encoding, mWebXML, new SubProgressMonitor(pMonitor, 100));

    pMonitor.done();
  }

  private boolean modifyDocument(XMLModificationTask[] pModificationList, Document pDoc,
      IProgressMonitor pMonitor) throws CoreException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    pMonitor.beginTask("analyzing web.xml", pModificationList.length * 100);
    pMonitor.subTask("analyzing web.xml");
    try
    {
      boolean result = false;
      for (int i = 0, sizei = pModificationList.length; i < sizei; ++i)
      {
        boolean opResult = pModificationList[i].modify(pDoc);
        result = result || opResult;
        pMonitor.worked(100);
        if (pMonitor.isCanceled())
        {
          throw new InterruptedException();
        }

      }

      return result;
    }
    catch (ModificationTaskException ioe)
    {
      String message = "Could not read or access the document required to perform the xml parsing: ";
      throw new CoreException(ExoCorePlugin.createErrorStatus(message + ioe.toString(), ioe));
    }
    finally
    {
      pMonitor.done();
    }

  }

  private boolean getDetectWebXMLOption()
  {
    IProject project = mProject.getProject();
    boolean result = IExoProjectRuntimeProperties.DETECT_WEBXML_FORMODIFICATION_OPTION_DEFAULT;
    String resultString = null;
    try
    {
      resultString = project.getPersistentProperty(IExoProjectRuntimeProperties.DETECT_WEBXML_FORMODIFICATION_OPTION);
      if ((resultString == null) || (resultString.length() < 1))
      {
        project.setPersistentProperty(IExoProjectRuntimeProperties.DETECT_WEBXML_FORMODIFICATION_OPTION, String.valueOf(result));
      }
      else
      {
        result = Boolean.valueOf(resultString).booleanValue();
      }
    }
    catch (Exception e)
    {
    }

    return result;
  }

  private IFile getWebXML()
  {
    IPath webpath = mProject.getWebFolder();
    if (webpath == null)
    {
      return null;
    }

    IFile webxml = mProject.getProject().getFile(webpath.append("WEB-INF/web.xml"));

    return (webxml.exists()) ? webxml : null;
  }

  private String getContextRoot()
  {
    IPath contextRootPath = mProject.getContextRoot();
    if (contextRootPath == null)
    {
      return null;
    }

    String contextRoot = contextRootPath.removeTrailingSeparator().toString();
    if (contextRoot.startsWith("/") || contextRoot.startsWith("\\"))
    {
      contextRoot = contextRoot.substring(1);
    }
    if (contextRoot.startsWith("/") || contextRoot.startsWith("\\"))
    {
      contextRoot = contextRoot.substring(1);
    }

    return contextRoot;
  }

  private Document loadDOMDocument(InputStream pInputStream) throws InterruptedException, CoreException
  {
    try
    {
      return XMLUtils.loadDOMDocument(pInputStream);
    }
    catch (IOException ioe)
    {
      String message = "Could not read or access the document required to perform the xml parsing";
      throw new CoreException(ExoCorePlugin.createErrorStatus(message, ioe));
    }
    catch (SAXException se)
    {
      String message = "Encountered a problem while trying to parse the xml\n";
      Exception current = se;
      if (se.getException() != null)
      {
        current = se.getException();
        if (current instanceof InterruptedException)
        {
          throw (InterruptedException) current;
        }
      }
      throw new CoreException(
          ExoCorePlugin.createErrorStatus(message + current.getLocalizedMessage(), current));
    }
  }

  private String getEncoding(Document pDoc)
  {
    return XMLUtils.getEncoding(pDoc);
  }

  private TextDocument loadFile(IFile pFile, String pEncoding) throws CoreException
  {
    try
    {
      return XMLUtils.loadFile(pFile.getContents(true), pEncoding);
    }
    catch (IOException io)
    {
      IStatus status = ExoCorePlugin.createErrorStatus("Unable to open or decode web.xml", io);
      throw new CoreException(status);
    }
  }

  private void saveToFile(Document pDoc, String pEncoding, IFile pFile, IProgressMonitor pMonitor)
      throws CoreException
  {
    try
    {
      byte[] output = XMLUtils.getFormattedDoc(pDoc, pEncoding, pFile.getContents(true));

      InputStream inputStream = new BufferedInputStream(new ByteArrayInputStream(output));
      pFile.setContents(inputStream, true, true, pMonitor);
    }
    catch (UnsupportedEncodingException e)
    {
      IStatus status = ExoCorePlugin.createErrorStatus("The web.xml is written in an encoding that is not supported", e);
      throw new CoreException(status);
    }
    catch (IOException e)
    {
      IStatus status = ExoCorePlugin.createErrorStatus("Could not finish writing information to web.xml", e);
      throw new CoreException(status);
    }
  }
}