/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation;


import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;
import org.exoplatform.eclipse.core.ExoCorePlugin;

import java.lang.reflect.InvocationTargetException;

/**
 * An operation which potentially makes changes to the workspace. All resource
 * modification should be performed using this operation. The primary
 * consequence of using this operation is that events which typically occur as
 * a result of workspace changes (such as the firing of resource deltas,
 * performance of autobuilds, etc.) are deferred until the outermost operation
 * has successfully completed.
 * 
 * <p>
 * Subclasses must implement <code>execute</code> to do the work of the
 * operation.
 * </p>
 * 
 * <p>
 * I took this class from org.eclipse.ui.actions.WorkspaceModifyOperation.
 * However, I removed any UI dependency from this class.
 * </p>
 */
public abstract class WorkspaceModifyOperation implements IExoOperation
{
  public static final String CLASS_VERSION = "$Id: WorkspaceModifyOperation.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$
  private MessageMultiStatus mStatus;

  /**
   * Creates a new operation.
   */
  protected WorkspaceModifyOperation()
  {
    mStatus =
      new MessageMultiStatus(ExoCorePlugin.PLUGIN_ID, IStatus.OK, "", null); //$NON-NLS-1$        
  }

  /**   
   * Performs the steps that are to be treated as a single logical workspace
   * change.
   * 
   * <p>
   * Subclasses must implement this method.
   * </p>
   *
   * @param pMonitor the progress monitor to use to display progress and field
   *        user requests to cancel
   *
   * @exception CoreException if the operation fails due to a CoreException
   * @exception InvocationTargetException if the operation fails due to an
   *            exception other than CoreException
   * @exception InterruptedException if the operation detects a request to
   *            cancel,  using <code>IProgressMonitor.isCanceled()</code>, it
   *            should exit by throwing  <code>InterruptedException</code>. It
   *            is also possible to throw
   *            <code>OperationCanceledException</code>, which gets mapped to
   *            <code>InterruptedException</code> by the <code>run</code>
   *            method.
   */
  protected abstract void execute(IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException;

  /**   
   * The <code>WorkspaceModifyOperation</code> implementation of this
   * <code>IRunnableWithProgress</code> method initiates a batch of changes by
   * invoking the <code>execute</code> method as a workspace runnable
   * (<code>IWorkspaceRunnable</code>).
   *
   * @param pMonitor
   *
   * @throws CoreException
   * @throws InvocationTargetException
   * @throws InterruptedException
   * @throws OperationCanceledException
   */
  public synchronized final void run(IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    final InvocationTargetException[] iteHolder =
      new InvocationTargetException[1];
    try
    {
      IWorkspaceRunnable workspaceRunnable =
        new IWorkspaceRunnable()
        {
          public void run(IProgressMonitor pm) throws CoreException
          {
            try
            {
              execute(pm);
            }
            catch (InvocationTargetException e)
            {
              // Pass it outside the workspace runnable
              iteHolder[0] = e;
            }
            catch (InterruptedException e)
            {
              // Re-throw as OperationCanceledException, which will be
              // caught and re-thrown as InterruptedException below.
              throw new OperationCanceledException(e.getMessage());
            }

            // CoreException and OperationCanceledException are propagated
          }
        };
      ResourcesPlugin.getWorkspace().run(workspaceRunnable, pMonitor);
    }
    catch (CoreException e)
    {
      throw new InvocationTargetException(e);
    }
    catch (OperationCanceledException e)
    {
      throw new InterruptedException(e.getMessage());
    }

    // Re-throw the InvocationTargetException, if any occurred
    if (iteHolder[0] != null)
    {
      throw iteHolder[0];
    }
  }

  /**   
   * Returns the current status of this operation. The result is a status
   * object which may contain individual nested status objects.
   *
   * @return the status of this operation
   */
  public IStatus getStatus()
  {
    return mStatus;
  }

  protected void addToStatus(CoreException pException)
  {
    IStatus status = pException.getStatus();
    String message = pException.getLocalizedMessage();
    if ((message == null) || (message.length() < 1))
    {
      message = getDefaultStatusErrorMessage();
      status =
        new Status(status.getSeverity(), status.getPlugin(), status.getCode(),
          message, pException);
    }
    mStatus.add(status);

    if ((mStatus.getMessage() == null) || (mStatus.getMessage().length() < 1))
    {
      mStatus.setMessage(message);
    }
  }

  protected void addToStatus(IStatus pStatus)
  {
    mStatus.add(pStatus);

    if ((mStatus.getMessage() == null) || (mStatus.getMessage().length() < 1))
    {
      String message = pStatus.getMessage();
      if ((message != null) && (message.length() > 0))
      {
        mStatus.setMessage(message);
      }
    }
  }

  protected String getDefaultStatusErrorMessage()
  {
    return "Unexpected core error during the operation.";
  }

  /**   
   * Adds a new warning to the list with the passed information. Normally the
   * export operation continues after a warning. (copied from
   * JarFileExportOperation)
   *
   * @param pMessage the message
   * @param pError the throwable that caused the warning, or<code>null</code>
   */
  protected void addWarning(String pMessage, Throwable pError)
  {
    mStatus.add(new Status(IStatus.WARNING, ExoCorePlugin.PLUGIN_ID, 0,
        pMessage, pError));

    if ((mStatus.getMessage() == null) || (mStatus.getMessage().length() < 1))
    {
      mStatus.setMessage(pMessage);
    }
  }

  /**   
   * Adds a new error to the list with the passed information. Normally an
   * error terminates the export operation.
   *
   * @param pMessage the message
   * @param pException the throwable that caused the error, or
   *        <code>null</code>
   */
  protected void addError(String pMessage, Throwable pException)
  {
    mStatus.add(new Status(IStatus.ERROR, ExoCorePlugin.PLUGIN_ID, 0, pMessage,
        pException));
    if ((mStatus.getMessage() == null) || (mStatus.getMessage().length() < 1))
    {
      mStatus.setMessage(pMessage);
    }
  }

  private static class MessageMultiStatus extends MultiStatus
  {
    MessageMultiStatus(String pPluginId, int pCode, String pMessage,
      Throwable exception)
    {
      super(pPluginId, pCode, pMessage, exception);
    }

    /*
     * allows to change the message
     */
    protected void setMessage(String pMessage)
    {
      super.setMessage(pMessage);
    }
  }
}
