/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation.helper;


import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.IClasspathEntry;

import java.util.ArrayList;
import java.util.List;

/**
 * This class represents a java project configuration. Usually, a java project
 * configuration is required by a java project create operation. A java
 * project configuration consists of  libraries entries, the default binary
 * folder and one or more java project settings.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class JavaProjectConfiguration extends ProjectConfiguration
{
  public static final String CLASS_VERSION =
    "$Id: JavaProjectConfiguration.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$
  private IClasspathEntry[] mProjectLibraries;
  private List mJavaProjectSettings;
  private IPath mDefaultBinaryFolder;

  /**   
   * Adds the given java project settings to the collection of the settings
   * that this class represents. Also, in addition of being exposed in the
   * getJavaProjectSettings(), the given settings will be exposed in the
   * getProjectSettings() too.
   *
   * @param pSettings provides access to multiple java project files providers
   *        and to the mounting point (path relative to the project) that they
   *        should exist under.
   */
  public void addJavaProjectSettings(JavaProjectSettings pSettings)
  {
    if (mJavaProjectSettings == null)
    {
      mJavaProjectSettings = new ArrayList();
    }

    mJavaProjectSettings.add(pSettings);
    addProjectSettings(pSettings);
  }

  /**   
   * Returns the java project settings that this class represents.
   *
   * @return the java project settings that this class represents. Each
   *         settings provides access to multiple project files providers and
   *         to the mounting point (path relative to the project) that they
   *         should exist under.
   */
  public JavaProjectSettings[] getJavaProjectSettings()
  {
    if (mJavaProjectSettings == null)
    {
      return new JavaProjectSettings[0];
    }

    JavaProjectSettings[] result =
      new JavaProjectSettings[mJavaProjectSettings.size()];
    result = (JavaProjectSettings[]) mJavaProjectSettings.toArray(result);

    return result;
  }

  /**   
   * Allows you to set all java project settings that this class represents.
   * This method removes any existing settings that this class represents and
   * replaces them with the ones given.  Also, in addition of being exposed in
   * the getJavaProjectSettings(), the given settings will be exposed in the
   * getProjectSettings() too.
   *
   * @param pSetting An array of java project settings
   */
  public void setJavaProjectSettings(JavaProjectSettings[] pSetting)
  {
    if (mJavaProjectSettings == null)
    {
      mJavaProjectSettings = new ArrayList();
    }
    else
    {
      mJavaProjectSettings.clear();
    }

    if ((pSetting == null) || (pSetting.length < 1))
    {
      return;
    }

    for (int i = 0, sizei = pSetting.length; i < sizei; ++i)
    {
      mJavaProjectSettings.add(pSetting[i]);
      addProjectSettings(pSetting[i]);
    }
  }

  /**   
   * Returns all the libraries (classpaths) assoicated with this java project.
   *
   * @return all the libraries (classpaths) assoicated with this java project
   */
  public IClasspathEntry[] getProjectLibraries()
  {
    return (mProjectLibraries == null) ? new IClasspathEntry[0]
                                       : mProjectLibraries;
  }

  /**   
   * Allows you to set all the libraries (classpaths) assoicated with this java
   * project
   *
   * @param pEntries all the libraries (classpaths) assoicated with this java
   *        project
   */
  public void setProjectLibraries(IClasspathEntry[] pEntries)
  {
    mProjectLibraries = pEntries;
  }

  /**   
   * Returns the folder name to use as a default binary folder
   *
   * @return the default binary folder (can be null)
   */
  public IPath getDefaultBinaryFolder()
  {
    return mDefaultBinaryFolder;
  }

  /**   
   * Allows you to set the default binary folder
   *
   * @param pPath a path relative to the project that should be treated as the
   *        the default binary folder
   */
  public void setDefaultBinaryFolder(IPath pPath)
  {
    mDefaultBinaryFolder = pPath;
  }
}
