/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.portal;

import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.common.Utils;
import org.exoplatform.eclipse.core.exoservices.Configuration;
import org.exoplatform.eclipse.core.exoservices.IExoPlatformInstallationInfo;
import org.exoplatform.eclipse.core.exoservices.IImportConfigurationURLResolver;
import org.exoplatform.eclipse.core.exoservices.ServicesInspector;
import org.exoplatform.eclipse.core.exoservices.ServicesMerger;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

/**
 * This class contains a collection of static methods to help with
 * reading/writing/scanning the PortalSettings class.
 * 
 * @author Hatim Khan
 */
public class PortalSettingsBuilder
{
  /**
   * This method scans an exo platform installation and returns a PortalSettings instance
   * that represents the information found during the scan.
   * 
   * @param pInstallationInfo an implementation of the IExoPlatformInstallationInfo
   * @param pResolver to resolve any import url found in the configuration files
   * @return a portal settings instance
   * @throws CoreException when things go wrong
   */
  public static PortalSettings scanPlatform(IExoPlatformInstallationInfo pInstallationInfo,
      IImportConfigurationURLResolver pResolver, IProgressMonitor pMonitor) throws CoreException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    ServicesInspector inspector = new ServicesInspector(pInstallationInfo, pResolver);
    PortalSettings settings = new PortalSettings();

    try
    {
      pMonitor.beginTask("", 5 * 10);
      settings.setRootConfiguration(PortalSettingsBuilder.getRootConfiguration(inspector, pMonitor, 10));
      settings.setDefaultPortalConfiguration(PortalSettingsBuilder.getCommonPortalServices(inspector, pMonitor, 10));
      settings.setPortalSessionConfiguration(PortalSettingsBuilder.getCommonPortalSessionServices(inspector, pMonitor, 10));

      ServicesMerger merger = new ServicesMerger();
      merger.add(PortalSettingsBuilder.getCommonPortalServices(inspector, pMonitor, 10));
      merger.add(PortalSettingsBuilder.getPortalServices(inspector, pMonitor, 10));
      settings.setCustomPortalConfiguration(merger.getMergedServices());
    }
    catch (IOException e)
    {
      pMonitor.done();
      IStatus status = ExoCorePlugin.createErrorStatus("Could not scan the eXo Platform installation directory for available services. Please make sure that you have the correct path specified.", e);
      throw new CoreException(status);
    }

    pMonitor.done();
    return settings;
  }

  public static void writeTo(PortalSettings pSettings, IFile pFile) throws CoreException
  {
    XStream xstream = new XStream(new DomDriver());
    String content = xstream.toXML(pSettings);
    Utils.setFileContent(pFile, content, null);
  }

  public static PortalSettings readFrom(IFile pFile) throws CoreException
  {
    try
    {
      String content = Utils.getFileContent(pFile);
      XStream xstream = new XStream(new DomDriver());
      return (PortalSettings) xstream.fromXML(content);
    }
    catch (IOException e)
    {
      IStatus status = ExoCorePlugin.createErrorStatus("unable to read the portal settings", e);
      throw new CoreException(status);
    }
  }

  private static Configuration getRootConfiguration(ServicesInspector pInspector, IProgressMonitor pMonitor,
      int pWorkUnits) throws IOException
  {
    Configuration config = pInspector.getRootServices(new SubProgressMonitor(pMonitor, pWorkUnits,
        SubProgressMonitor.PREPEND_MAIN_LABEL_TO_SUBTASK));
    config.setReadOnly(true);
    return config;
  }

  private static Configuration getCommonPortalServices(ServicesInspector pInspector,
      IProgressMonitor pMonitor, int pWorkUnits) throws IOException
  {
    Configuration config = pInspector.getCommonPortalServices(new SubProgressMonitor(pMonitor, pWorkUnits,
        SubProgressMonitor.PREPEND_MAIN_LABEL_TO_SUBTASK));
    config.markGlobalParamsSticky();
    return config;
  }

  private static Configuration getCommonPortalSessionServices(ServicesInspector pInspector,
      IProgressMonitor pMonitor, int pWorkUnits) throws IOException
  {
    Configuration config = pInspector.getCommonPortalSessionServices(new SubProgressMonitor(pMonitor,
        pWorkUnits, SubProgressMonitor.PREPEND_MAIN_LABEL_TO_SUBTASK));
    config.setReadOnly(true);
    return config;
  }

  private static Configuration getPortalServices(ServicesInspector pInspector, IProgressMonitor pMonitor,
      int pWorkUnits) throws IOException
  {
    Configuration config = pInspector.getPortalServices(new SubProgressMonitor(pMonitor, pWorkUnits,
        SubProgressMonitor.PREPEND_MAIN_LABEL_TO_SUBTASK));
    return config;
  }

}