/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.warpackager;


import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.OperationCanceledException;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.common.IOverwriteQuery;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import java.util.jar.JarEntry;
import java.util.jar.JarOutputStream;
import java.util.jar.Manifest;
import java.util.zip.CRC32;

/**
 * Creates a WAR file for the given WAR package data. he idea and the code came
 * from org.eclipse.jdt.ui.jarpackager.JarWriter
 */
public class WarWriter
{
  private JarOutputStream mWarOutputStream;
  private WarPackageData mWarPackage;

  /**
   * Creates an instance which is used to create a WAR based on the given
   * WarPackage.
   *
   * @param pWarPackage the WAR specification
   * @param pOverwriteQuery the shell used to display question dialogs,
   *        or<code>null</code> if "false/no/cancel" is the answer and no
   *        dialog should be shown
   *
   * @throws CoreException to signal any other unusal termination. This can
   *         also be used to return information in the status object.
   * @throws OperationCanceledException
   */
  public WarWriter(WarPackageData pWarPackage, IOverwriteQuery pOverwriteQuery)
    throws CoreException
  {
    mWarPackage = pWarPackage;
    if (!mWarPackage.isValid())
    {
      throw WarPackagerUtil.createCoreException("The WAR package specification is invalid",
        null);
    }
    if (!canCreateWar(pOverwriteQuery))
    {
      throw new OperationCanceledException();
    }

    try
    {
      if (mWarPackage.usesManifest())
      {
        Manifest manifest =
          mWarPackage.getManifestProvider().create(mWarPackage);
        mWarOutputStream =
          new JarOutputStream(new FileOutputStream(mWarPackage.getWarLocation()
                                                              .toOSString()),
            manifest);
      }
      else
      {
        mWarOutputStream =
          new JarOutputStream(new FileOutputStream(mWarPackage.getWarLocation()
                                                              .toOSString()));
      }

      String comment = mWarPackage.getComment();
      if (comment != null)
      {
        mWarOutputStream.setComment(comment);
      }
    }
    catch (IOException ex)
    {
      throw WarPackagerUtil.createCoreException(ex.getLocalizedMessage(), ex);
    }
  }

  /**   
   * Closes the archive and does all required cleanup.
   *
   * @throws CoreException to signal any other unusal termination. This can
   *         also be used to return information in the status object.
   */
  public void close() throws CoreException
  {
    if (mWarOutputStream != null)
    {
      try
      {
        mWarOutputStream.close();
        registerInWorkspaceIfNeeded();
      }
      catch (IOException ex)
      {
        throw WarPackagerUtil.createCoreException(ex.getLocalizedMessage(), ex);
      }
    }
  }

  /**   
   * Writes the passed resource to the current archive.
   *
   * @param pResource the file to be written
   * @param pDestinationPath the path for the file inside the archive
   *
   * @throws CoreException to signal any other unusal termination. This can
   *         also be used to return information in the status object.
   */
  public void write(IFile pResource, IPath pDestinationPath)
    throws CoreException
  {
    ByteArrayOutputStream output = null;
    BufferedInputStream contentStream = null;

    try
    {
      output = new ByteArrayOutputStream();
      if (!pResource.isLocal(IResource.DEPTH_ZERO))
      {
        String message =
          "Resource not found or not accessible: " + pResource.getFullPath();
        throw WarPackagerUtil.createCoreException(message, null);
      }
      contentStream = new BufferedInputStream(pResource.getContents(false));

      int chunkSize = 4096;
      byte[] readBuffer = new byte[chunkSize];
      int count;
      while ((count = contentStream.read(readBuffer, 0, chunkSize)) != -1)
      {
        output.write(readBuffer, 0, count);
      }
    }
    catch (IOException ex)
    {
      throw WarPackagerUtil.createCoreException(ex.getLocalizedMessage(), ex);
    }
    finally
    {
      try
      {
        if (output != null)
        {
          output.close();
        }
        if (contentStream != null)
        {
          contentStream.close();
        }
      }
      catch (IOException ex)
      {
        throw WarPackagerUtil.createCoreException(ex.getLocalizedMessage(), ex);
      }
    }
    try
    {
      IPath fileLocation = pResource.getLocation();
      long lastModified = System.currentTimeMillis();
      if (fileLocation != null)
      {
        File file = new File(fileLocation.toOSString());
        if (file.exists())
        {
          lastModified = file.lastModified();
        }
      }
      write(pDestinationPath, output.toByteArray(), lastModified);
    }
    catch (IOException ex)
    {
      // Ensure full path is visible
      String message = null;
      if (ex.getLocalizedMessage() != null)
      {
        message =
          "Problem writing " + pResource.getFullPath() + " to WAR: "
          + ex.getLocalizedMessage();
      }
      else
      {
        message = "Problem writing " + pResource.getFullPath() + " to WAR";
      }
      throw WarPackagerUtil.createCoreException(message, ex);
    }
  }

  /**   
   * Creates a new JarEntry with the passed path and contents, and writes it to
   * the current archive.
   *
   * @param pPath the path inside the archive
   * @param pContents the bytes to write
   * @param pLastModified a long which represents the last modification date
   *
   * @throws IOException if an I/O error has occurred
   */
  protected void write(IPath pPath, byte[] pContents, long pLastModified)
    throws IOException
  {
    JarEntry newEntry =
      new JarEntry(pPath.toString().replace(File.separatorChar, '/'));
    if (mWarPackage.isCompressed())
    {
      newEntry.setMethod(JarEntry.DEFLATED);
    }

    // Entry is filled automatically.
    else
    {
      newEntry.setMethod(JarEntry.STORED);
      newEntry.setSize(pContents.length);

      CRC32 checksumCalculator = new CRC32();
      checksumCalculator.update(pContents);
      newEntry.setCrc(checksumCalculator.getValue());
    }

    // Set modification time
    newEntry.setTime(pLastModified);

    try
    {
      mWarOutputStream.putNextEntry(newEntry);
      mWarOutputStream.write(pContents);
    }
    finally
    {
      /*
       * Commented out because some JREs throw an NPE if a stream
       * is closed twice. This works because
       * a) putNextEntry closes the previous entry
       * b) closing the stream closes the last entry
       */

      // fJarOutputStream.closeEntry();
    }
  }

  /**   
   * Checks if the WAR file can be overwritten. If the WAR package setting does
   * not allow to overwrite the WAR then a dialog will ask the user again.
   *
   * @param pOverwriteQuery
   *
   * @return <code>true</code> if it is OK to create the WAR
   */
  protected boolean canCreateWar(IOverwriteQuery pOverwriteQuery)
  {
    File file = mWarPackage.getWarLocation().toFile();
    if (file.exists())
    {
      if (!file.canWrite())
      {
        return false;
      }

      return WarPackagerUtil.askForOverwritePermission(pOverwriteQuery,
        mWarPackage.getWarLocation().toOSString());
    }

    // Test if directory exists
    String path = file.getAbsolutePath();
    int separatorIndex = path.lastIndexOf(File.separator);
    if (separatorIndex == -1) // ie.- default dir, which is fine
    {
      return true;
    }

    File directory = new File(path.substring(0, separatorIndex));
    if (directory.exists())
    {
      return true;
    }
    else
    {
      return false;
    }
  }

  private void registerInWorkspaceIfNeeded()
  {
    IPath jarPath = mWarPackage.getWarLocation();
    IProject[] projects =
      ResourcesPlugin.getWorkspace().getRoot().getProjects();
    for (int i = 0; i < projects.length; i++)
    {
      IProject project = projects[i];
      IPath projectLocation = project.getLocation();
      boolean isInWorkspace =
        (projectLocation != null) && projectLocation.isPrefixOf(jarPath);
      if (isInWorkspace)
      {
        try
        {
          jarPath = jarPath.removeFirstSegments(projectLocation.segmentCount());
          jarPath = jarPath.removeLastSegments(1);

          IResource containingFolder = project.findMember(jarPath);
          if ((containingFolder != null) && containingFolder.isAccessible())
          {
            containingFolder.refreshLocal(IResource.DEPTH_ONE, null);
          }
        }
        catch (CoreException ex)
        {
          // don't refresh the folder but log the problem
          ExoCorePlugin.logError(ex);
        }
        break;
      }
    }
  }
}
