/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.xmlvalidator;


import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.xmlvalidator.helper.ExoSAXHandler;
import org.osgi.framework.Bundle;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * This class uses Xerces SAX parser to validate the portlet.xml based on a given schema.
 * 
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ExoSchemaBasedPortletValidator implements IPortletDescriptorValidator, IExecutableExtension
{
  public static final String CLASS_VERSION = "$Id: ExoSchemaBasedPortletValidator.java,v 1.1 2004/04/19 03:45:50 hatimk Exp $"; //$NON-NLS-1$  
  private static final String EXTENSION_NAME_SCHEMAENTRY = "schema-entry"; //$NON-NLS-1$
  private static final String EXTENSION_ATTRIBUTE_NAMESPACEURI = "namespace-uri"; //$NON-NLS-1$
  private static final String EXTENSION_ATTRIBUTE_SCHEMALOCATION = "schema-location"; //$NON-NLS-1$
  private static final String SPACE = " "; //$NON-NLS-1$
  private String mSupportedSpecVersion = null;
  private String mSupportedExoVersion = null;
  private boolean mDefault = false;
  private int mOrder;
  private String mSchemaLocations = null;

  /**
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#validate()
   */
  public void validate(PortletValidationProblemReporter pReporter, IProgressMonitor pMonitor)
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    validateWithDifferentClassLoader(pReporter, pMonitor);
  }

  private void validateWithDifferentClassLoader(PortletValidationProblemReporter pReporter,
      IProgressMonitor pMonitor)
  {
    if (pMonitor.isCanceled())
    {
      return;
    }

    ClassLoader previous = Thread.currentThread().getContextClassLoader();

    try
    {
      // Since jdk 1.4 comes with xml-api.jar, the implementation of the XMLReaderFactory
      // will eventually try to use the thread class loader to locate the xerces
      // classes. However, since each plugin has its own class loader, we need to
      // associate the context class loader of the thread with the plugin class
      // loader. This way the xerces files will be available.
      Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
      runValidation(pReporter, pMonitor);
    }
    finally
    {
      if (previous != null)
      {
        Thread.currentThread().setContextClassLoader(previous);
      }
    }
  }

  private void runValidation(PortletValidationProblemReporter pReporter, IProgressMonitor pMonitor)
  {
    InputStream inStream = null;
    try
    {
      IFile portletXML = (IFile) pReporter.getResource();
      XMLReader parser = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");

      parser.setFeature("http://xml.org/sax/features/namespaces", true);
      parser.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
      parser.setFeature("http://xml.org/sax/features/validation", true);
      parser.setFeature("http://apache.org/xml/features/validation/schema", true);
      parser.setFeature("http://apache.org/xml/features/validation/schema-full-checking", true);

      parser.setProperty("http://java.sun.com/xml/jaxp/properties/schemaLanguage", "http://www.w3.org/2001/XMLSchema");
      if (mSchemaLocations != null)
      {
        parser.setProperty("http://apache.org/xml/properties/schema/external-schemaLocation", mSchemaLocations);
      }

      inStream = portletXML.getContents();

      pMonitor.beginTask("preparing Xerces parser", 200);
      pMonitor.subTask("Xerces parser is running");

      ExoSAXHandler errorHandler = new ExoSAXHandler(pReporter, new SubProgressMonitor(pMonitor, 200));
      parser.setErrorHandler(errorHandler);
      parser.setContentHandler(errorHandler);

      InputSource input = new InputSource(inStream);

      if (pMonitor.isCanceled())
      {
        return;
      }
      parser.parse(input);
    }
    catch (IOException ioe)
    {
      String message = "Could not read or access all the documents required to perform the portlet validation";
      pReporter.addOperationError(message);
      ExoCorePlugin.logError(message, ioe);
    }
    catch (SAXException se)
    {
      // in this case, this should be reported as a fatal error. This
      // error might not be because of validation, so we need to make a
      // decision whether to report this as a problem marker.
      if (!(se instanceof SAXParseException))
      {
        pReporter.addOperationError("Encountered a problem with the parser while trying to perform the portlet validation. The message from the parser is :"
            + se.getMessage());
      }
    }
    catch (CoreException e)
    {
      String message = "Encountered unknown problem while trying to perform the portlet validation. The description of the problem is :"
          + e.getMessage();
      pReporter.addOperationError(message);
      ExoCorePlugin.logError(message, e);
    }
    finally
    {
      if (inStream != null)
      {
        try
        {
          inStream.close();
        }
        catch (IOException e)
        {
        }
      }
      pMonitor.done();
    }
  }

  /**
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#getSupportedSpecVersion()
   */
  public String getSupportedSpecVersion()
  {
    return mSupportedSpecVersion;
  }

  /**
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#setSupportedSpecVersion(java.lang.String)
   */
  public void setSupportedSpecVersion(String pVersion)
  {
    mSupportedSpecVersion = pVersion;
  }

  /**
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#getSupportedExoVersion()
   */
  public String getSupportedExoVersion()
  {
    return mSupportedExoVersion;
  }

  /**
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#setSupportedExoVersion(java.lang.String)
   */
  public void setSupportedExoVersion(String pVersion)
  {
    mSupportedExoVersion = pVersion;
  }

  /**
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#isDefault()
   */
  public boolean isDefault()
  {
    return mDefault;
  }

  /**
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#setDefault(boolean)
   */
  public void setDefault(boolean pDefault)
  {
    mDefault = pDefault;
  }

  /**
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#getOrder()
   */
  public int getOrder()
  {
    return mOrder;
  }

  /**
   * @see org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator#setOrder(int)
   */
  public void setOrder(int pOrder)
  {
    mOrder = pOrder;
  }

  /**
   * @see org.eclipse.core.runtime.IExecutableExtension#setInitializationData(org.eclipse.core.runtime.IConfigurationElement,
   *      java.lang.String, java.lang.Object)
   */
  public void setInitializationData(IConfigurationElement pConfig, String pPropertyName, Object pData)
      throws CoreException
  {
    StringBuffer sb = new StringBuffer();

    try
    {
      String namespaceURI;
      String schemaLocation;
      URL relativeLocation;
      IConfigurationElement[] schemaEntries = pConfig.getChildren(EXTENSION_NAME_SCHEMAENTRY);
      for (int i = 0, sizei = schemaEntries.length; i < sizei; ++i)
      {
        namespaceURI = schemaEntries[i].getAttribute(EXTENSION_ATTRIBUTE_NAMESPACEURI);
        if (namespaceURI == null)
        {
          continue;
        }
        schemaLocation = schemaEntries[i].getAttribute(EXTENSION_ATTRIBUTE_SCHEMALOCATION);
        if (schemaLocation == null)
        {
          continue;
        }

        String bundleName = pConfig.getDeclaringExtension().getNamespace();
        Bundle bundle = Platform.getBundle(bundleName);
        relativeLocation = bundle.getEntry(new Path(schemaLocation).toString());
        schemaLocation = Platform.asLocalURL(relativeLocation).toExternalForm();

        // now add the found mapping to the string buffer
        sb.append(namespaceURI);
        sb.append(SPACE);
        sb.append(schemaLocation);
        sb.append(SPACE);
      }
    }
    catch (MalformedURLException e)
    {
      ExoCorePlugin.logError("Unable to parse the schema location in the extension point", e);
    }
    catch (IOException e)
    {
      ExoCorePlugin.logError("Unable to parse the schema location in the extension point", e);
    }

    mSchemaLocations = sb.toString();
  }
}