/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.xmlvalidator;


import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.common.IExoMarker;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * A class that knows how to create portlet validation problem markers.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class PortletValidationProblemReporter
{
  public static final String CLASS_VERSION = "$Id: PortletValidationProblemReporter.java,v 1.1 2004/04/19 03:45:50 hatimk Exp $"; //$NON-NLS-1$
  private IResource mResource;
  private List mProblems = new ArrayList();

  /**
   * The constructor method.
   *
   * @param pResource
   */
  public PortletValidationProblemReporter(IResource pResource)
  {
    mResource = pResource;
  }

  /**   
   * Use this method to add errors to report them later.
   *
   * @param pMessage the message
   * @param pLineNumber the line number
   * @param pColumnNumber the column number
   */
  public void addError(String pMessage, int pLineNumber, int pColumnNumber)
  {
    mProblems.add(new ProblemEntry(IMarker.SEVERITY_ERROR, pMessage,
        pLineNumber, pColumnNumber));
  }

  /**   
   * Use this method to add warnings to report them later.
   *
   * @param pMessage the message
   * @param pLineNumber the line number
   * @param pColumnNumber the column number
   */
  public void addWarning(String pMessage, int pLineNumber, int pColumnNumber)
  {
    mProblems.add(new ProblemEntry(IMarker.SEVERITY_WARNING, pMessage,
        pLineNumber, pColumnNumber));
  }

  /**   
   * Use this method to add fatal errors to report them later.
   *
   * @param pMessage the message
   * @param pLineNumber the line number
   * @param pColumnNumber the column number
   */
  public void addFatalErrors(String pMessage, int pLineNumber, int pColumnNumber)
  {
    mProblems.add(new ProblemEntry(IMarker.SEVERITY_ERROR, pMessage,
        pLineNumber, pColumnNumber));
  }

  /**
   * Use this method when you want to report issues related to problems
   * with conducting the validation operation itself, rather than validation
   * errors in the document.
   * 
   * @param pMessage the message to display to the user  
   */
  public void addOperationError(String pMessage)
  {
    mProblems.add(new ProblemEntry(IMarker.SEVERITY_ERROR, pMessage, -1, -1));
  }

  /**   
   * This method helps avoid sending many resource change events (as the result
   * of creating and populating markers) by using the workspace run operation.
   */
  public void reportProblems()
  {
    try
    {
      IWorkspaceRunnable runnable =
        new IWorkspaceRunnable()
        {
          public void run(IProgressMonitor pMonitor) throws CoreException
          {
            reportProblemEntries();
          }
        };

      ResourcesPlugin.getWorkspace().run(runnable, null);
    }
    catch (CoreException e)
    {
      ExoCorePlugin.logError("Could not create portlet problem markers", e);
    }
  }

  /**   
   * Deletes all the portlet validation problem markers. This method uses the
   * workspace run method to avoid sending too many resource change events.
   */
  public void deleteAllProblems()
  {
    if (mResource == null)
    {
      return;
    }
    try
    {
      IWorkspaceRunnable runnable =
        new IWorkspaceRunnable()
        {
          public void run(IProgressMonitor pMonitor) throws CoreException
          {
            mResource.deleteMarkers(IExoMarker.PORTLET_PROBLEM, false,
              IResource.DEPTH_ZERO);
          }
        };

      mResource.getWorkspace().run(runnable, null);
    }
    catch (CoreException e)
    {
      ExoCorePlugin.logError("Could not delete all portlet problem markers", e);
    }
  }

  /**   
   * The resource this reporter is associated with.
   *
   * @return the resource this reporter is associated with
   */
  public IResource getResource()
  {
    return mResource;
  }

  private void reportProblemEntries()
  {
    if (mProblems == null)
    {
      return;
    }

    Iterator iterator = mProblems.iterator();
    ProblemEntry problem;
    while (iterator.hasNext())
    {
      problem = (ProblemEntry) iterator.next();
      createPortletMarker(problem.getSeverity(), problem.getMessage(),
        problem.getLineNumber(), problem.getColumnNumber());
    }
  }

  private void createPortletMarker(int pSeverity, String pMessage,
    int pLineNumber, int pColumnNumber)
  {
    if (mResource == null)
    {
      return;
    }

    IMarker marker;
    try
    {
      marker = mResource.createMarker(IExoMarker.PORTLET_PROBLEM);

      Map map = new HashMap();
      map.put(IMarker.MESSAGE, pMessage);
      map.put(IMarker.SEVERITY, new Integer(pSeverity));
      if (pLineNumber >= 0)
      {
        map.put(IMarker.LINE_NUMBER, new Integer(pLineNumber));
      }
      if (pColumnNumber >= 0)
      {
        map.put(IExoMarker.COLUMN, new Integer(pColumnNumber));
      }
      marker.setAttributes(map);
    }
    catch (CoreException e)
    {
      ExoCorePlugin.logError("Could not create a marker to indicate a portlet problem",
        e);
    }
  }

  class ProblemEntry
  {
    private String mMessage;
    private int mLineNumber;
    private int mColumnNumber;
    private int mSeverity;

    public ProblemEntry(int pSeverity, String pMessage, int pLineNumber,
      int pColumnNumber)
    {
      mSeverity = pSeverity;
      mMessage = pMessage;
      mLineNumber = pLineNumber;
      mColumnNumber = pColumnNumber;
    }

    /**   
     * Returns the column number associated with this problem
     *
     * @return the column number associated with this problem
     */
    public int getColumnNumber()
    {
      return mColumnNumber;
    }

    /**   
     * Returns the line number associated with this problem
     *
     * @return the line number associated with this problem
     */
    public int getLineNumber()
    {
      return mLineNumber;
    }

    /**   
     * Returns the message associated with this problem
     *
     * @return the message associated with this problem
     */
    public String getMessage()
    {
      return mMessage;
    }

    public int getSeverity()
    {
      return mSeverity;
    }
  }
}
