/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.core;


import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.core.runtime.preferences.IEclipsePreferences.PreferenceChangeEvent;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.IPortletProject;
import org.exoplatform.eclipse.core.IPreferencesConstant;
import org.exoplatform.eclipse.core.xmlvalidator.helper.PortletDeploymentDescriptorInspector;
import org.osgi.service.prefs.Preferences;

/**
 * An implementation of the IPortletProject interface that wraps around a resource
 * project.
 * 
 * @author Hatim Khan
 */
public class PortletProject implements IPortletProject, IResourceChangeListener,
    IEclipsePreferences.IPreferenceChangeListener
{
  private static final String EMPTY_STRING = ""; //$NON-NLS-1$
  private IProject mProject;
  private String mWebFolder = null;
  private String mContextRoot = null;
  private String mPortletVersion = null;
  private boolean mValidPortletVersionCache = false;
  private boolean mValidPortletProjectSettingsCache = false;

  /**
   * The constructor method.
   * 
   * @param pProject the resource project to wrap
   */
  public PortletProject(IProject pProject)
  {
    mProject = pProject;
    // since this class provides information about the portlet version, the web folder,
    // etc, we need to make sure that if these values change that this class is aware of
    // such change. we accomplish this via resouce change events, and preference change
    // events
    ResourcesPlugin.getWorkspace().addResourceChangeListener(this, IResourceChangeEvent.POST_CHANGE
        | IResourceChangeEvent.PRE_CLOSE | IResourceChangeEvent.PRE_DELETE);

    IEclipsePreferences projectPreferences = ExoCorePlugin.getProjectScopePreferences(mProject);
    if (projectPreferences != null)
    {
      projectPreferences.addPreferenceChangeListener(this);
    }
  }

  /**
   * @see org.exoplatform.eclipse.core.IPortletProject#getWebFolder()
   */
  public IPath getWebFolder()
  {
    if (!mValidPortletProjectSettingsCache)
    {
      loadPortletProjectSettings();
    }

    return (mWebFolder == null) ? null : new Path(mWebFolder);
  }

  /**
   * @see org.exoplatform.eclipse.core.IPortletProject#getPortletXML()
   */
  public IPath getPortletXML()
  {
    IPath webfolder = getWebFolder();
    if (webfolder == null)
    {
      return null;
    }

    return webfolder.append("WEB-INF/portlet.xml");
  }

  /**
   * @see org.exoplatform.eclipse.core.IPortletProject#getContextRoot()
   */
  public IPath getContextRoot()
  {
    if (!mValidPortletProjectSettingsCache)
    {
      loadPortletProjectSettings();
    }

    return (mContextRoot == null) ? null : new Path(mContextRoot);
  }

  /**
   * @see org.exoplatform.eclipse.core.IPortletProject#getPortleXMLVersion()
   */
  public String getPortletXMLVersion()
  {
    if (!mValidPortletVersionCache)
    {
      loadPortletXMLVersion();
    }

    return mPortletVersion;
  }

  /**
   * @see org.exoplatform.eclipse.core.IPortletProject#getProject()
   */
  public IProject getProject()
  {
    return mProject;
  }

  /**
   * @see org.eclipse.core.runtime.preferences.IEclipsePreferences.IPreferenceChangeListener#preferenceChange(org.eclipse.core.runtime.preferences.IEclipsePreferences.PreferenceChangeEvent)
   */
  public void preferenceChange(PreferenceChangeEvent pEvent)
  {
    //TODO make sure that the event is fired only if the project scope preferences node
    // keys are modifyied as opposed to firing the event when any node keys are modified
    firePortletProjectSettingsChange();
  }

  /**
   * @see org.eclipse.core.resources.IResourceChangeListener#resourceChanged(org.eclipse.core.resources.IResourceChangeEvent)
   */
  public void resourceChanged(IResourceChangeEvent pEvent)
  {
    // check if the project is closed or deleted
    if ((pEvent.getType() == IResourceChangeEvent.PRE_CLOSE)
        || (pEvent.getType() == IResourceChangeEvent.PRE_DELETE))
    {
      IResource resource = pEvent.getResource();

      if ((resource != null) && (resource == mProject))
      {
        ResourcesPlugin.getWorkspace().removeResourceChangeListener(this);
        IEclipsePreferences projectPreferences = ExoCorePlugin.getProjectScopePreferences(mProject);
        if (projectPreferences != null)
        {
          projectPreferences.removePreferenceChangeListener(this);
        }
      }

      return;
    }

    //  we are only interested in POST_CHANGE events from this point on
    if (pEvent.getType() != IResourceChangeEvent.POST_CHANGE)
    {
      return;
    }

    IResourceDelta rootDelta = pEvent.getDelta();

    if (rootDelta == null)
    {
      // if rootDelta is null, it means that there are too many changes
      // for the workspace to report, so, we assume that the .exo file changed
      // too.
      //TODO verify that if rootDelta is null, then too many changes occured
      firePortletXMLVersionChange();

      return;
    }

    // see if we got any changes related to our project, if not return fast.
    IPath projectPath = mProject.getFullPath();
    IResourceDelta projectDelta = rootDelta.findMember(projectPath);
    if (projectDelta == null)
    {
      return;
    }

    IPath portletxmlPath = getPortletXML();
    if (portletxmlPath != null)
    {
      IResourceDelta portletDelta = projectDelta.findMember(portletxmlPath);
      if (portletDelta != null)
      {
        // we are interested in any change to the portlet.xml, it does not matter
        // if it is a delete, move, rename, or content change
        firePortletXMLVersionChange();
      }
    }
  }

  /**
   * Finds the value of the version attribute in the portlet.xml
   */
  private void loadPortletXMLVersion()
  {
    IPath portletXML = getPortletXML();
    if (portletXML == null)
    {
      mPortletVersion = null;

      return;
    }

    IFile portletXMLResource = mProject.getFile(portletXML);
    if (!portletXMLResource.exists())
    {
      mPortletVersion = null;

      return;
    }

    mPortletVersion = PortletDeploymentDescriptorInspector.getVersion(portletXMLResource);
    mValidPortletVersionCache = true;

    return;
  }

  /**
   * Loads the portlet project settings from the xml store file.
   */
  private void loadPortletProjectSettings()
  {
    mWebFolder = null;
    mContextRoot = null;
    Preferences projectPreferences = ExoCorePlugin.getProjectScopePreferences(mProject);
    if (projectPreferences != null)
    {
      mWebFolder = getNullIfEmptyString(projectPreferences.get(IPreferencesConstant.PORTLET_PROJECT_STORE_WEBFOLDER, null));
      mContextRoot = getNullIfEmptyString(projectPreferences.get(IPreferencesConstant.PORTLET_PROJECT_STORE_CONTEXTROOT, null));
    }

    mValidPortletProjectSettingsCache = true;
  }

  private String getNullIfEmptyString(String pString)
  {
    String result = null;
    if ((pString != null) && (pString.length() > 0))
    {
      result = pString;
    }

    return result;
  }

  private void firePortletProjectSettingsChange()
  {
    mValidPortletProjectSettingsCache = false;
  }

  private void firePortletXMLVersionChange()
  {
    mValidPortletVersionCache = false;
  }

}