/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.core;


import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.runtime.CoreException;
import org.exoplatform.eclipse.core.ExoCorePlugin;

/**
 * Provides the portlet nature to a given project.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class PortletProjectNature implements IProjectNature
{
  public static final String CLASS_VERSION = "$Id: PortletProjectNature.java,v 1.1 2004/04/19 17:48:24 hatimk Exp $"; //$NON-NLS-1$
  private IProject mProject;

  /**   
   * @see org.eclipse.core.resources.IProjectNature#configure()
   */
  public void configure() throws CoreException
  {
    // register portlet builder
    addToBuildSpec(ExoCorePlugin.PORTLET_BUILDER_ID);
  }

  /**   
   * @see org.eclipse.core.resources.IProjectNature#deconfigure()
   */
  public void deconfigure() throws CoreException
  {
    // deregister portlet builder
    removeFromBuildSpec(ExoCorePlugin.PORTLET_BUILDER_ID);
  }

  /**   
   * @see org.eclipse.core.resources.IProjectNature#getProject()
   */
  public IProject getProject()
  {
    return mProject;
  }

  /**   
   * @see org.eclipse.core.resources.IProjectNature#setProject(org.eclipse.core.resources.IProject)
   */
  public void setProject(IProject pProject)
  {
    mProject = pProject;
  }

  /**   
   * Adds a builder to the build spec for the given project.
   *
   * @param pBuilderID the full id of the portlet builder
   *
   * @throws CoreException
   */
  protected void addToBuildSpec(String pBuilderID) throws CoreException
  {
    IProjectDescription description = getProject().getDescription();
    ICommand javaCommand = getPortletCommand(description);

    if (javaCommand == null)
    {
      // Add a Java command to the build spec
      ICommand command = description.newCommand();
      command.setBuilderName(pBuilderID);
      setPortletCommand(description, command);
    }
  }

  /**   
   * Removes the given builder from the build spec for the given project.
   *
   * @param pBuilderID the full id of the portlet builder
   *
   * @throws CoreException
   */
  protected void removeFromBuildSpec(String pBuilderID)
    throws CoreException
  {
    IProjectDescription description = getProject().getDescription();
    ICommand[] commands = description.getBuildSpec();
    for (int i = 0; i < commands.length; ++i)
    {
      if (commands[i].getBuilderName().equals(pBuilderID))
      {
        ICommand[] newCommands = new ICommand[commands.length - 1];
        System.arraycopy(commands, 0, newCommands, 0, i);
        System.arraycopy(commands, i + 1, newCommands, i,
          commands.length - i - 1);
        description.setBuildSpec(newCommands);
        getProject().setDescription(description, null);

        return;
      }
    }
  }

  /**   
   * Find the specific portlet command amongst the build spec of a given
   * description
   *
   * @param pDescription the project description that contains a list of the
   *        build spec
   *
   * @return the command that represents the portlet builder
   *
   * @throws CoreException if things go wrong
   */
  private ICommand getPortletCommand(IProjectDescription pDescription)
    throws CoreException
  {
    ICommand[] commands = pDescription.getBuildSpec();
    for (int i = 0; i < commands.length; ++i)
    {
      if (commands[i].getBuilderName().equals(ExoCorePlugin.PORTLET_BUILDER_ID))
      {
        return commands[i];
      }
    }

    return null;
  }

  /**   
   * Update the portlet command in the build spec (replace existing one if
   * present, add one first if none).
   *
   * @param pDescription
   * @param pNewCommand
   *
   * @throws CoreException
   */
  private void setPortletCommand(IProjectDescription pDescription,
    ICommand pNewCommand) throws CoreException
  {
    ICommand[] oldCommands = pDescription.getBuildSpec();
    ICommand oldJavaCommand = getPortletCommand(pDescription);
    ICommand[] newCommands;

    if (oldJavaCommand == null)
    {
      // Add a portlet build spec before other builders
      newCommands = new ICommand[oldCommands.length + 1];
      System.arraycopy(oldCommands, 0, newCommands, 1, oldCommands.length);
      newCommands[0] = pNewCommand;
    }
    else
    {
      for (int i = 0, max = oldCommands.length; i < max; i++)
      {
        if (oldCommands[i] == oldJavaCommand)
        {
          oldCommands[i] = pNewCommand;

          break;
        }
      }
      newCommands = oldCommands;
    }

    // Commit the spec change into the project
    pDescription.setBuildSpec(newCommands);
    getProject().setDescription(pDescription, null);
  }

  /**   
   * A convenient method to check if the given project has the portlet nature.
   * If an exception occurs, then the method returns false
   *
   * @param pProject the project to check its nature
   *
   * @return true if it is a portlet project, false otherwise
   */
  public static boolean isPortletProject(IProject pProject)
  {
    if (pProject == null)
    {
      return false;
    }
    try
    {
      return pProject.hasNature(ExoCorePlugin.PORTLET_NATURE_ID);
    }
    catch (CoreException e)
    {
      ExoCorePlugin.logError(e);
    }

    return false;
  }
}
