/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.core.builder;


import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.IPortletProject;
import org.exoplatform.eclipse.core.IPreferencesConstant;
import org.exoplatform.eclipse.core.operation.ValidatePortletDeploymentDescriptorOperation;
import org.exoplatform.eclipse.core.xmlvalidator.PortletValidationProblemReporter;
import org.exoplatform.eclipse.internal.core.PortletProjectNature;

import java.lang.reflect.InvocationTargetException;
import java.util.Map;

/**
 * This is a custom builder that delegates the actual validation of portlet.xml
 * to the appropriate validator according to the spec version that the
 * portlet.xml is using.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class PortletDeploymentDescriptorValidator
  extends IncrementalProjectBuilder
{
  public static final String CLASS_VERSION =
    "$Id: PortletDeploymentDescriptorValidator.java,v 1.1 2004/04/19 17:48:25 hatimk Exp $"; //$NON-NLS-1$

  /**   
   * @see org.eclipse.core.internal.events.InternalBuilder#build(int,
   *      java.util.Map, org.eclipse.core.runtime.IProgressMonitor)
   */
  protected IProject[] build(int pKind, Map pArgs, IProgressMonitor pMonitor)
    throws CoreException
  {
    IProject project = getProject();

    // ignore if the project is not of a portlet nature
    if (!PortletProjectNature.isPortletProject(project))
    {
      return null;
    }

    IPortletProject portletProject =
      ExoCorePlugin.createPortletProject(project);

    IPath portletXMLPath = portletProject.getPortletXML();
    if (portletXMLPath == null)
    {
      return null;
    }

    IResource portletXMLResource = null;

    IResourceDelta delta = null;
    if (pKind != FULL_BUILD)
    {
      delta = getDelta(project);
    }

    if ((delta == null) || (pKind == FULL_BUILD))
    {
      portletXMLResource = project.getFile(portletXMLPath);
    }
    else
    {
      delta = delta.findMember(portletXMLPath);
      if ((delta != null) && (delta.getKind() != IResourceDelta.REMOVED))
      {
        portletXMLResource = delta.getResource();
      }
    }

    // at this stage, we need to verify that the portletXMLResource is valid
    if ((portletXMLResource == null) || (!portletXMLResource.exists())
        || (portletXMLResource.getType() != IResource.FILE))
    {
      return null;
    }

    if (!isAutomaticPortletValidation())
    {
      PortletValidationProblemReporter reporter =
        new PortletValidationProblemReporter(portletXMLResource);
      reporter.deleteAllProblems();

      return null;
    }

    handleValidation(project, pMonitor);

    return null;
  }

  private void handleValidation(IProject pProject, IProgressMonitor pMonitor)
    throws CoreException
  {
    ValidatePortletDeploymentDescriptorOperation op = null;
    try
    {
      op = new ValidatePortletDeploymentDescriptorOperation(pProject);
      op.run(pMonitor);

      IStatus status = op.getStatus();
      if (!status.isOK())
      {
        ExoCorePlugin.log(status);
      }
    }
    catch (InvocationTargetException target)
    {
      Throwable cause = target.getTargetException();
      if (cause instanceof CoreException)
      {
        IStatus status = ((CoreException) cause).getStatus();
        ExoCorePlugin.log(status);
      }
      else
      {
        ExoCorePlugin.logError(cause);
      }
    }
    catch (InterruptedException e)
    {
      // no action is required if user requests cancelation
    }
  }

  protected boolean isAutomaticPortletValidation()
  {
    return ExoCorePlugin.getDefault().getPluginPreferences().getBoolean(IPreferencesConstant.PORTLET_VALIDATION_ENABLE);
  }
}
