/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.IProgressService;
import org.exoplatform.eclipse.core.exoservices.GlobalParam;
import org.exoplatform.eclipse.core.exoservices.Parameter;
import org.exoplatform.eclipse.core.exoservices.Property;
import org.exoplatform.eclipse.core.exoservices.ServiceConfiguration;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.core.portal.model.INode;
import org.exoplatform.eclipse.core.portal.model.ValueHolder;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * This wizard page allows the user to edit the name and description of a parameter.
 * However, this class is an abstract class; implementors must implement
 * createNewParameter() and addValuesContent() to provide their own values content. The
 * createNewParameter() will be called once during the creation time but only and only if
 * the mode is 'add'. Generally, a typical implementation will also override
 * updateFromModel(), attachListeners(), validateInput(), and finish(). Notice that the
 * finish() method takes care of updating the model and firing the model change events,
 * implementors should just update the mDirty flag if they want to update the model.
 * 
 * @author Hatim Khan
 */
public abstract class EditServiceParameterPage extends DidYouKnowPage
{
  public static final int ADD_MODE = 0;
  public static final int EDIT_MODE = 1;
  public static final String TITLE_EDIT = "Edit an eXo Platform service configuration parameter";
  public static final String TITLE_ADD = "Add an eXo Platform service configuration parameter";
  public static final Class[] EDIT_ACCEPT_CLASSES = new Class[] { ValueHolder.class, Property.class };

  protected int mMode = EDIT_MODE;
  private PortalSettings mPortalSettings;
  protected Object mParameterParent;
  protected Parameter mParameter;
  private String mOriginalDescription;
  private String mOriginalName;
  private List mParametersKeys;
  protected boolean mDirty;

  private Text mDescriptionText;
  private Text mNameText;

  public EditServiceParameterPage(String pPageName, String pDidYouKnowMessage, int pMode,
      PortalSettings pPortalSettings, Object pParameterParent, Parameter pParameter)
  {
    super(pPageName, pDidYouKnowMessage);
    setTitle((pMode == ADD_MODE) ? TITLE_ADD : TITLE_EDIT);
    mDirty = false;

    mMode = pMode;
    mPortalSettings = pPortalSettings;
    mParameterParent = pParameterParent;
    if ((pParameter == null) || (pMode == ADD_MODE))
    {
      mParameter = createNewParameter();
    }
    else
    {
      mParameter = pParameter;
    }
  }

  protected abstract Parameter createNewParameter();

  public void createActualContent(Composite pParent)
  {
    Composite container = UICompositeUtil.createFillBothComposite(pParent, 2, 1);
    GridLayout gl = (GridLayout) container.getLayout();
    gl.makeColumnsEqualWidth = false;
    Dialog.applyDialogFont(container);

    createDescription(container);
    createNameContent(container);
    addValuesContent(container);

    updateFromModel();
    attachListeners();

    setPageComplete((mMode == EDIT_MODE));
  }

  protected abstract void addValuesContent(Composite pClient);

  protected void createNameContent(Composite pParent)
  {
    UICompositeUtil.addLabel(pParent, "Name : ", 1);

    mNameText = UICompositeUtil.addText(pParent, "", 1);
    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    mNameText.setLayoutData(gd);
  }

  protected void createDescription(Composite pParent)
  {
    Label label = new Label(pParent, SWT.NONE);
    label.setText("Description : ");

    GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING | GridData.VERTICAL_ALIGN_BEGINNING);
    gridData.horizontalSpan = 1;
    label.setLayoutData(gridData);

    mDescriptionText = new Text(pParent, SWT.MULTI | SWT.WRAP | SWT.LEFT | SWT.BORDER);
    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    mDescriptionText.setLayoutData(gd);
    UICompositeUtil.setHeightHint(mDescriptionText, 20);
    mDescriptionText.setText("");
  }

  protected void updateFromModel()
  {
    String name = "";
    String desc = "";
    if (mParameter != null)
    {
      name = UICompositeUtil.emptyIfNull(mParameter.getName()).trim();
      desc = UICompositeUtil.emptyIfNull(mParameter.getDescription()).trim();
    }

    mOriginalName = name;
    mOriginalDescription = desc;

    mParametersKeys = new ArrayList();

    if (mParameterParent instanceof ServiceConfiguration)
    {
      ServiceConfiguration service = (ServiceConfiguration) mParameterParent;
      mParametersKeys = service.getParametersKeys();
      // remember service returns a List, but changes to the list order will not affect
      // the service internal list
    }
    else if (mParameterParent instanceof GlobalParam)
    {
      GlobalParam global = (GlobalParam) mParameterParent;
      mParametersKeys = global.getParametersKeys();
    }

    // remove the original key from the list so we don't confuse it with the other keys
    if (name.length() > 0) mParametersKeys.remove(name);

    mNameText.setText(name);
    mDescriptionText.setText(desc);
    mDirty = false;

    // if the parameter is sticky then we can not edit its name
    if ((mParameter != null) && (mParameter.isSticky()))
    {
      mNameText.setEditable(false);
    }
  }

  protected boolean validateInput()
  {
    String name = UICompositeUtil.emptyIfNull(mNameText.getText()).trim();

    // check if both are empty, if so then display an error
    if ((name.length() < 1))
    {
      setErrorMessage("The name of the parameter can NOT be empty.");
      return false;
    }

    boolean existingName = mParametersKeys.contains(name);
    // check if the proposed name matches an existing name (except the original parameter)
    if (existingName)
    {
      setErrorMessage("There is an existing parameter with the same name.");
      return false;
    }

    // if we get to this point then validation is complete and we don't have any errors
    // so update the error message
    setErrorMessage(null);
    return true;
  }

  protected void attachListeners()
  {
    mNameText.addListener(SWT.Modify, new Listener()
    {
      public void handleEvent(Event pEvent)
      {
        setPageComplete(validateInput());
      }
    });
    mDescriptionText.addListener(SWT.Modify, new Listener()
    {
      public void handleEvent(Event pEvent)
      {
        setPageComplete(validateInput());
      }
    });
  }

  public void finish()
  {
    // first lets detect if the user changed either the name and/or the description, if
    // so, then we need to make the model dirty and fire an event
    String name = UICompositeUtil.emptyIfNull(mNameText.getText()).trim();
    String desc = UICompositeUtil.emptyIfNull(mDescriptionText.getText()).trim();
    if ((!name.equals(mOriginalName)) || (!desc.equals(mOriginalDescription)))
    {
      if (mParameter != null)
      {
        mParameter.setName(name);
        mParameter.setDescription(desc);
        mDirty = true;
      }
    }

    if (mDirty)
    {
      if (mPortalSettings != null) mPortalSettings.markDirty();
      if (mMode == EDIT_MODE)
      {
        if (mPortalSettings != null) mPortalSettings.fireConfigurationElementUpdated(mParameter);
      }
      else if (mMode == ADD_MODE)
      {
        if (mParameterParent instanceof INode)
        {
          INode parentNode = (INode) mParameterParent;
          parentNode.addChild(mParameter);
          if (mPortalSettings != null) mPortalSettings.fireConfigurationElementAdded(mParameter);
        }
      }
    }
  }

  protected void showBusyWhile(final Runnable pRunnable)
  {
    IProgressService progressService = PlatformUI.getWorkbench().getProgressService();
    try
    {
      progressService.busyCursorWhile(new IRunnableWithProgress()
      {
        public void run(IProgressMonitor pMonitor)
        {
          getShell().getDisplay().asyncExec(pRunnable);
        }
      });
    }
    catch (InvocationTargetException e)
    {
    }
    catch (InterruptedException e)
    {
    }
  }

  protected boolean isSameStringList(List p1, List p2)
  {
    if ((p1 == null) && (p2 == null)) return true;
    if (p1 == null) return false;
    if (p2 == null) return false;

    if (p1.size() != p2.size()) return false;
    for (int i = 0, sizei = p1.size(); i < sizei; ++i)
    {
      String v1 = (String) p1.get(i);
      String v2 = (String) p2.get(i);
      if (!(v1.equals(v2))) return false;
    }

    return true;
  }

  /**
   * @return Returns the dirty.
   */
  public boolean isDirty()
  {
    return mDirty;
  }
}