/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.wizard;


import org.eclipse.core.runtime.IStatus;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.eclipse.ui.help.WorkbenchHelp;
import org.exoplatform.eclipse.ui.IExoHelpContextIds;
import org.exoplatform.eclipse.ui.validator.InputTextValidator;

/**
 * This page allows the user to specify the project name, project path, the
 * source folder name, the web content folder name and the context root.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class NewPortletsProjectSettingPage extends WizardNewProjectCreationPage
{
  public static final String CLASS_VERSION = "$Id: NewPortletsProjectSettingPage.java,v 1.1 2004/04/19 03:37:24 hatimk Exp $"; //$NON-NLS-1$
  public static final String DEFAULT_SRC = "src"; //$NON-NLS-1$
  public static final String DEFAULT_WEB = "web-content"; //$NON-NLS-1$
  private Text mSourceFolderText;
  private Text mWebFolderText;
  private Text mContextRootText;

  /**
   * Creates a wizard page.
   *
   * @param pPageName the name of this page
   */
  public NewPortletsProjectSettingPage(String pPageName)
  {
    super(pPageName);
    setPageComplete(false);
  }

  /**   
   * @see org.eclipse.ui.dialogs.WizardNewProjectCreationPage#createControl(org.eclipse.swt.widgets.Composite)
   */
  public void createControl(Composite pParent)
  {
    Composite mainComposite = createFillBothComposite(pParent, 1);
    super.createControl(mainComposite);

    // because we want to overwrite the super help context we need to
    // reassign it here
    Control composite = getControl();
    WorkbenchHelp.setHelp(composite, IExoHelpContextIds.CREATE_PORTLET_PROJECT);

    createSourceFolderGroup(mainComposite);
    createWebFolderGroup(mainComposite);
    createContextRootGroup(mainComposite);
    setPageComplete(validatePage());

    // Show description on opening instead of error messages
    setErrorMessage(null);
    setMessage(null);
    WorkbenchHelp.setHelp(mainComposite,
      IExoHelpContextIds.CREATE_PORTLET_PROJECT);
    setControl(mainComposite);
  }

  /**   
   * Returns the value of the source folder field with leading and trailing
   * spaces removed.
   *
   * @return the source folder
   */
  public String getSourceFolder()
  {
    if (mSourceFolderText == null)
    {
      return ""; //$NON-NLS-1$
    }
    else
    {
      return mSourceFolderText.getText().trim();
    }
  }

  /**   
   * Returns the value of the web folder field with leading and trailing spaces
   * removed.
   *
   * @return the source folder
   */
  public String getWebFolder()
  {
    if (mWebFolderText == null)
    {
      return ""; //$NON-NLS-1$
    }
    else
    {
      return mWebFolderText.getText().trim();
    }
  }

  /**   
   * Returns the value of the context root field with leading and trailing
   * spaces removed.
   *
   * @return the source folder
   */
  public String getContextRoot()
  {
    if (mContextRootText == null)
    {
      return ""; //$NON-NLS-1$
    }
    else
    {
      return mContextRootText.getText().trim();
    }
  }

  /**   
   * @see org.eclipse.ui.dialogs.WizardNewProjectCreationPage#validatePage()
   */
  protected boolean validatePage()
  {
    boolean parentResult = super.validatePage();
    if (!parentResult)
    {
      return parentResult;
    }

    // validate source folder
    String sourceFolder = getSourceFolder();
    IStatus validationStatus =
      InputTextValidator.validateSourceFolderName(sourceFolder);
    if (validationStatus.getSeverity() == IStatus.INFO)
    {
      setErrorMessage(null);
      setMessage(validationStatus.getMessage());

      return false;
    }
    else if (validationStatus.getSeverity() != IStatus.OK)
    {
      setMessage(null);
      setErrorMessage(validationStatus.getMessage());

      return false;
    }

    // validate web folder
    String webFolder = getWebFolder();
    validationStatus = InputTextValidator.validateWebFolderName(webFolder);
    if (validationStatus.getSeverity() == IStatus.INFO)
    {
      setErrorMessage(null);
      setMessage(validationStatus.getMessage());

      return false;
    }
    else if (validationStatus.getSeverity() != IStatus.OK)
    {
      setMessage(null);
      setErrorMessage(validationStatus.getMessage());

      return false;
    }

    // validate context root
    String contextRoot = getContextRoot();
    validationStatus = InputTextValidator.validateContextRootName(contextRoot);
    if (validationStatus.getSeverity() == IStatus.INFO)
    {
      setErrorMessage(null);
      setMessage(validationStatus.getMessage());

      return false;
    }
    else if (validationStatus.getSeverity() != IStatus.OK)
    {
      setMessage(null);
      setErrorMessage(validationStatus.getMessage());

      return false;
    }
    setErrorMessage(null);
    setMessage(null);

    return true;
  }

  private void createSourceFolderGroup(Composite pParent)
  {
    Group group = createFillBothGroup(pParent, "Source folder setting", 2);
    addLabel(group,
      "Please provide the name of the folder that will contain \nthe source code once the project is created : ",
      2);
    mSourceFolderText = addText(group, DEFAULT_SRC);
    mSourceFolderText.addListener(SWT.Modify, TextModifyListener);
  }

  private void createWebFolderGroup(Composite pParent)
  {
    Group group = createFillBothGroup(pParent, "Web content folder setting", 2);

    addLabel(group,
      "Please provide the name of the folder that will contain \nthe web content (jsp pages, WEB-INF directory, etc.) : ",
      2);
    mWebFolderText = addText(group, DEFAULT_WEB);
    mWebFolderText.addListener(SWT.Modify, TextModifyListener);
  }

  private void createContextRootGroup(Composite pParent)
  {
    Group group = createFillBothGroup(pParent, "Context root setting", 2);

    addLabel(group,
      "Please provide the context root to use for this web application during deployment. \n"
      + "Note: the context root is not used during the project creation. It will be used \n"
      + "later when you deploy the application. Therefore, you can leave it empty if you \n"
      + "want, and supply a value later in the property page of this project. You can \n"
      + "access the property page by selecting the 'Project -> Properties' menu option", 2);
    mContextRootText = addText(group, "");
    mContextRootText.addListener(SWT.Modify, TextModifyListener);
  }

  private Composite createFillBothComposite(Composite pParent, int pNumColumns)
  {
    Composite composite = new Composite(pParent, SWT.NONE);
    GridLayout layout = new GridLayout();
    layout.marginHeight = 5;
    layout.marginWidth = 5;
    layout.numColumns = pNumColumns;

    GridData data = new GridData();
    data.horizontalAlignment = GridData.FILL;
    data.verticalAlignment = GridData.FILL;
    data.grabExcessHorizontalSpace = true;
    data.grabExcessVerticalSpace = true;
    composite.setLayout(layout);
    composite.setLayoutData(data);
    composite.setFont(pParent.getFont());

    return composite;
  }

  private Group createFillBothGroup(Composite pParent, String pTitle,
    int pNumColumns)
  {
    Group group = new Group(pParent, SWT.NONE);
    group.setText(pTitle);

    GridLayout layout = new GridLayout();
    layout.marginHeight = 5;
    layout.marginWidth = 5;
    layout.numColumns = pNumColumns;

    GridData data = new GridData();
    data.horizontalAlignment = GridData.FILL;
    data.verticalAlignment = GridData.FILL;
    data.grabExcessHorizontalSpace = true;
    data.grabExcessVerticalSpace = true;
    group.setLayout(layout);
    group.setLayoutData(data);
    group.setFont(pParent.getFont());

    return group;
  }

  private Text addText(Composite pComp, String pTextDefaultValue)
  {
    Text result = new Text(pComp, SWT.SINGLE | SWT.BORDER);
    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    result.setLayoutData(gridData);
    result.setText(pTextDefaultValue);

    return result;
  }

  private void addLabel(Composite pComp, String pText, int pHorizontalSpan)
  {
    Label label = new Label(pComp, SWT.NONE);

    label.setText(pText);

    GridData gd = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
    gd.horizontalSpan = pHorizontalSpan;
    label.setLayoutData(gd);
  }

  /**   
   * Create some empty space.
   *
   * @param pComp the composite to add the space to
   * @param pColSpan the number of column to span
   */
  private void createVerticalSpacer(Composite pComp, int pColSpan)
  {
    Label label = new Label(pComp, SWT.NONE);
    GridData gd = new GridData();
    gd.horizontalSpan = pColSpan;
    label.setLayoutData(gd);
  }

  private Listener TextModifyListener =
    new Listener()
    {
      public void handleEvent(Event e)
      {
        setPageComplete(validatePage());
      }
    };
}
