/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.action;


import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.actions.BuildAction;
import org.eclipse.ui.dialogs.PropertyDialogAction;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.IPortletProject;
import org.exoplatform.eclipse.core.operation.DeployWebappOperation;
import org.exoplatform.eclipse.core.runtime.IExoProjectRuntimeProperties;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.common.DeploymentOverwriteQuery;
import org.exoplatform.eclipse.ui.dialog.ProblemDialog;

import java.io.File;

import java.lang.reflect.InvocationTargetException;

/**
 * Deploys a portlet web application. It will build the project before it
 * deploys it.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class DeployPortletappActionDelegate
  implements IWorkbenchWindowActionDelegate
{
  public static final String CLASS_VERSION =
    "$Id: DeployPortletappActionDelegate.java,v 1.1 2004/04/19 03:37:26 hatimk Exp $"; //$NON-NLS-1$
  private IWorkbenchWindow mWindow;
  private ISelection mSelection;
  private String mWarDeploymentTitle = "WAR Deployment";
  private String mWillOpenPropertyDialogMessage =
    "Click OK to open the properites page of the selected project. "
    + "You can then specify a different value "
    + "by selecting eXo Portlet Settings. In general, you can access the properties page "
    + "of any project, by highlighting the project and selecting the 'Project -> Properties' menu option.";

  /**   
   * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
   */
  public void dispose()
  {
  }

  /**   
   * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(org.eclipse.ui.IWorkbenchWindow)
   */
  public void init(IWorkbenchWindow pWindow)
  {
    mWindow = pWindow;
  }

  /**   
   * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
   */
  public void run(IAction action)
  {
    IProject project;
    IPath webFolder;
    int deploymentOption;
    IPath deployDir;
    IPath contextRoot;

    try
    {
      project = getValidatedProject();
      if (project == null)
      {
        return;
      }
      webFolder = getValidatedWebFolder(project);
      if (webFolder == null)
      {
        return;
      }
      deployDir = getValidatedDeploymentDir(project);
      if (deployDir == null)
      {
        return;
      }

      contextRoot = getValidatedContextRoot(project);
      if (contextRoot == null)
      {
        return;
      }

      deploymentOption = getDeploymentOption(project);
      
      File deploymentFile = deployDir.append(contextRoot).toFile();
      modifyWebXML(project);
      buildProject(project);
      performDeployAction(project, webFolder, deploymentFile, deploymentOption);
    }
    catch (Exception e)
    {
      reportError("Encountered a problem while trying to deploy \n"
        + e.toString());
      ExoUIPlugin.logError("Encountered a problem while trying to deploy", e);
    }
  }

  /**   
   * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
   *      org.eclipse.jface.viewers.ISelection)
   */
  public void selectionChanged(IAction pAction, ISelection pSelection)
  {
    mSelection = pSelection;
  }

  protected void performDeployAction(IProject pProject, IPath pWebFolder,
    File pDeploymentDir, int pDeploymentOption)
  {
    DeployWebappOperation op = null;
    try
    {
      op =
        new DeployWebappOperation(pProject, pWebFolder, pDeploymentDir,
          pDeploymentOption,
          new DeploymentOverwriteQuery(pProject, mWindow.getShell()));
      new ProgressMonitorDialog(mWindow.getShell()).run(true, true,
        new ExoRunnableAdapter(op));

      IStatus status = op.getStatus();
      if (!status.isOK())
      {
        ProblemDialog.open(mWindow.getShell(), mWarDeploymentTitle, null, status); //$NON-NLS-1$
      }
    }
    catch (InvocationTargetException target)
    {
      Throwable cause = target.getTargetException();
      if (cause instanceof CoreException)
      {
        IStatus status = ((CoreException) cause).getStatus();
        ProblemDialog.open(mWindow.getShell(), mWarDeploymentTitle, null, status); //$NON-NLS-1$        
        ExoUIPlugin.log(status);
      }
      else
      {
        String message =
          (cause == null) ? target.getLocalizedMessage() : cause.toString();
        MessageDialog.openError(mWindow.getShell(), mWarDeploymentTitle,
          "Encountered a problem while trying to deploy \n" + message);
        ExoUIPlugin.logError(cause);
      }
    }
    catch (InterruptedException e)
    {
      // no action is required if user requests cancelation
    }
  }

  protected void modifyWebXML(IProject pProject)
  {
    WebXMLModifyAction action =
      new WebXMLModifyAction(mWindow.getShell(), pProject);
    action.run();
  }

  protected void buildProject(IProject pProject)
  {
    BuildAction action =
      new BuildAction(mWindow.getShell(),
        IncrementalProjectBuilder.INCREMENTAL_BUILD);
    action.selectionChanged(new StructuredSelection(pProject));
    action.run();
  }
  
  protected IProject getValidatedProject() throws CoreException
  {
    IProject project = null;
    project = getSelectedProject();

    // check if the project is not empty 
    if (project == null)
    {
      reportError(
        "Unable to find a java web application project to deploy. The operation can not continue. Please select a java project and try again.");

      return null;
    }

    // check if the project is a portlet project 
    if (!project.hasNature(ExoCorePlugin.PORTLET_NATURE_ID))
    {
      reportError(
        "Trying to deploy a project that is not a portlet project. The operation can not continue. Please select a portlet project and try again. Or upgrade the project to a portlet project and try again.");

      return null;
    }

    // check if the project is a java project
    //    IJavaProject javaProject = JavaCore.create(project);
    //    if ((javaProject == null) || (!javaProject.exists()))
    //    {
    //      reportError(
    //        "Trying to deploy a project that is not a java project. The operation can not continue. Please select a java project and try again.");
    //
    //      return null;
    //    }
    return project;
  }

  protected IPath getValidatedWebFolder(IProject pProject)
  {
    String errorMessage =
      "Either the web content folder is invalid or it is not specified. "
      + mWillOpenPropertyDialogMessage;

    String errorUnknown =
      "Unable to locate the web content folder. "
      + mWillOpenPropertyDialogMessage;

    IFolder webFolder = null;
    try
    {
      IPath path = getWebFolder(pProject);
      if ((path == null) || (!path.isValidPath(path.toString())))
      {
        reportError(errorMessage);
        showPropertyDialog(pProject);

        return null;
      }

      webFolder = ResourcesPlugin.getWorkspace().getRoot().getFolder(path);
      if (!webFolder.exists())
      {
        reportError(errorMessage);
        showPropertyDialog(pProject);

        return null;
      }
    }
    catch (Exception exception)
    {
      reportError(errorUnknown);
      ExoUIPlugin.logError("Encountered a problem while trying to validate the web content folder",
        exception);
      showPropertyDialog(pProject);

      return null;
    }

    return (webFolder == null) ? null : webFolder.getFullPath();
  }

  protected IPath getValidatedDeploymentDir(IProject pProject)
  {
    String errorMessage =
      "Either the deployment directory is invalid or it is not specified. "
      + mWillOpenPropertyDialogMessage;

    String errorUnknown =
      "Unable to locate the deployment directory. "
      + mWillOpenPropertyDialogMessage;

    IPath deploymentDir = null;
    try
    {
      deploymentDir = getDeploymentDir(pProject);
      if ((deploymentDir == null)
          || (!deploymentDir.isValidPath(deploymentDir.toString())))
      {
        reportError(errorMessage);
        showPropertyDialog(pProject);

        return null;
      }

      if (!(deploymentDir.toFile().exists()))
      {
        reportError(errorMessage);
        showPropertyDialog(pProject);

        return null;
      }
    }
    catch (Exception exception)
    {
      reportError(errorUnknown);
      ExoUIPlugin.logError("Encountered a problem while trying to validate the deployment directory",
        exception);
      showPropertyDialog(pProject);

      return null;
    }

    return deploymentDir;
  }

  protected IPath getValidatedContextRoot(IProject pProject)
  {
    String errorMessage =
      "Either the context root is invalid or it is not specified. "
      + mWillOpenPropertyDialogMessage;

    String rootNotAllowedMessage =
      "The context root can not be the root (/). "
      + mWillOpenPropertyDialogMessage;

    String errorUnknown =
      "Unable to obtain the context root for this project. "
      + mWillOpenPropertyDialogMessage;

    IPath contextRoot = null;
    try
    {
      contextRoot = getContextRoot(pProject);
      if ((contextRoot == null)
          || (!contextRoot.isValidPath(contextRoot.toString())))
      {
        reportError(errorMessage);
        showPropertyDialog(pProject);

        return null;
      }
      if (contextRoot.isRoot())
      {
        reportError(rootNotAllowedMessage);
        showPropertyDialog(pProject);

        return null;
      }
      if (contextRoot.segmentCount() > 1)
      {
        String childNotAllowedMessage =
          "The context root (" + contextRoot.toString()
          + ") must be the direct child of the root. "
          + mWillOpenPropertyDialogMessage;

        reportError(childNotAllowedMessage);
        showPropertyDialog(pProject);

        return null;
      }
    }
    catch (Exception exception)
    {
      reportError(errorUnknown);
      ExoUIPlugin.logError("Encountered a problem while trying to validate the context root",
        exception);
      showPropertyDialog(pProject);

      return null;
    }

    return contextRoot;
  }

  /**   
   * Returns the current project that include the current selection. For
   * example if the user opens a file, and then invoke this action, then the
   * projec that contains this file will be returned.
   *
   * @return the project that contains the current selection
   */
  protected IProject getSelectedProject()
  {
    IResource resource = null;

    if ((mSelection != null) && (!mSelection.isEmpty()))
    {
      if (mSelection instanceof IStructuredSelection)
      {
        Object selection =
          ((IStructuredSelection) mSelection).getFirstElement();
        if (selection instanceof IResource)
        {
          resource = (IResource) selection;
        }
        else if (selection instanceof IAdaptable)
        {
          resource =
            (IResource) ((IAdaptable) selection).getAdapter(IResource.class);
        }
      }
      else
      {
        org.eclipse.ui.IEditorPart activeEditor =
          mWindow.getActivePage().getActiveEditor();

        if (activeEditor != null)
        {
          IEditorInput input = activeEditor.getEditorInput();
          if (input instanceof IFileEditorInput)
          {
            resource = ((IFileEditorInput) input).getFile();
          }
        }
      }
    }

    return (resource == null) ? null : resource.getProject();
  }

  protected IPath getWebFolder(IProject pProject) throws CoreException
  {
    IPortletProject portletProject =
      ExoCorePlugin.createPortletProject(pProject);
    IPath path = portletProject.getWebFolder();

    if (path == null)
    {
      return null;
    }

    return pProject.getFullPath().append(path);
  }

  protected IPath getDeploymentDir(IProject pProject) throws CoreException
  {
    String deployDirString =
      pProject.getPersistentProperty(IExoProjectRuntimeProperties.DEPLOYMENT_DIR);
    if (deployDirString == null)
    {
      return null;
    }

    IPath deployDir = new Path(deployDirString);

    return deployDir;
  }

  protected IPath getContextRoot(IProject pProject) throws CoreException
  {
    IPortletProject portletProject =
      ExoCorePlugin.createPortletProject(pProject);

    return portletProject.getContextRoot();
  }

  protected int getDeploymentOption(IProject pProject)
    throws CoreException
  {
    String optionString =
      pProject.getPersistentProperty(IExoProjectRuntimeProperties.DEPLOYMENT_OPTION);
    int option = IExoProjectRuntimeProperties.DEPLOYMENT_OPTION_DEFAULT;
    try
    {
      option = Integer.parseInt(optionString);
    }
    catch (NumberFormatException e)
    {
      // not a valid deployment option so stick with the default
    }

    return option;
  }

  protected void reportError(String pMessage)
  {
    MessageDialog.openError(mWindow.getShell(), mWarDeploymentTitle, pMessage);
  }

  protected void showPropertyDialog(IProject pProject)
  {
    Action propertyAction =
      new PropertyDialogAction(mWindow.getShell(),
        new SimpleSelectionProvider(pProject));
    propertyAction.run();

    return;
  }
}
