/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.action;

import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.ILaunchesListener;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.events.MenuAdapter;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowPulldownDelegate2;
import org.exoplatform.eclipse.ui.launching.ExoUILaunching;


/**
 * An action delegate that acts as a drop down menu. 
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class StopExoServerMenuActionDelegate implements IWorkbenchWindowPulldownDelegate2,
  ILaunchesListener
{
  public static final String CLASS_VERSION = "$Id: StopExoServerMenuActionDelegate.java,v 1.1 2004/09/20 01:42:55 hatimk Exp $"; //$NON-NLS-1$
  private Menu mMenu;
  private IWorkbenchWindow mWindow;
  private IAction mAction;
  private boolean mRecreateMenu = false;

  /**   
   * @see org.eclipse.jface.action.IMenuCreator#getMenu(org.eclipse.swt.widgets.Menu)
   */
  public Menu getMenu(Menu pParent)
  {
    setMenu(new Menu(pParent));
    fillMenu(mMenu);
    initMenu(mMenu);

    return mMenu;
  }

  /**   
   * @see org.eclipse.ui.IWorkbenchWindowPulldownDelegate#getMenu(org.eclipse.swt.widgets.Control)
   */
  public Menu getMenu(Control pParent)
  {
    setMenu(new Menu(pParent));
    fillMenu(mMenu);
    initMenu(mMenu);

    return mMenu;
  }

  /**   
   * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
   */
  public void dispose()
  {
    setMenu(null);
    getLaunchManager().removeLaunchListener(this);
  }

  /**   
   * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(org.eclipse.ui.IWorkbenchWindow)
   */
  public void init(IWorkbenchWindow pWindow)
  {
    mWindow = pWindow;
  }

  /**   
   * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
   */
  public void run(IAction pAction)
  {
  	ILaunch launch = getLastLaunch();
  	if (launch == null)
  	{
			MessageDialog.openError(mWindow.getShell(), "Stopping eXo Platform Error",
								"There are no eXo Platform instances running.");  		  		
  	}
  	else
  	{
			IAction action = new StopExoServerAction(launch, mWindow);
			action.run();  		
  	}
  }

  /**   
   * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
   *      org.eclipse.jface.viewers.ISelection)
   */
  public void selectionChanged(IAction pAction, ISelection pSelection)
  {
    if (mAction == null)
    {
      initialize(pAction);
    }
  }

  /**   
   * @see org.eclipse.debug.core.ILaunchesListener#launchesRemoved(org.eclipse.debug.core.ILaunch[])
   */
  public void launchesRemoved(ILaunch[] pLaunches)
  {
    mRecreateMenu = true;
    updateEnablement();
    updateTooltip();
  }

  /**   
   * @see org.eclipse.debug.core.ILaunchesListener#launchesAdded(org.eclipse.debug.core.ILaunch[])
   */
  public void launchesAdded(ILaunch[] pLaunches)
  {
    mRecreateMenu = true;
    updateEnablement();
    updateTooltip();
  }

  /**   
   * @see org.eclipse.debug.core.ILaunchesListener#launchesChanged(org.eclipse.debug.core.ILaunch[])
   */
  public void launchesChanged(ILaunch[] pLaunches)
  {
    mRecreateMenu = true;
  }

  private void fillMenu(Menu pMenu)
  {
    ILaunch[] launches = ExoUILaunching.getExoLaunches();
    for (int i = 0, sizei = launches.length; i < sizei; ++i)
    {
      IAction action = new StopExoServerAction(launches[i], mWindow);
      StringBuffer label = new StringBuffer();
      if ((i >= 0) && (i < 9))
      {
        //add the numerical accelerator
        label.append('&');
        label.append(i + 1);
        label.append(' ');
      }
			label.append("Stop ");      
      label.append(action.getText());
      action.setText(label.toString());

      ActionContributionItem item = new ActionContributionItem(action);
      item.fill(pMenu, -1);
    }

    // now if we have no items to display then disable this action
    if (launches.length < 1)
    {
      IAction action = getAction();
      if (action != null)
      {
        action.setEnabled(false);
      }
    }
  }

  /**   
   * Creates the menu for the action
   *
   * @param pMenu the menu to listen to its event
   */
  private void initMenu(Menu pMenu)
  {
    // Add listener to repopulate the menu each time
    // it is shown because of dynamic eXo platform launch
    pMenu.addMenuListener(new MenuAdapter()
      {
        public void menuShown(MenuEvent e)
        {
          if (mRecreateMenu)
          {
            Menu m = (Menu) e.widget;
            MenuItem[] items = m.getItems();
            for (int i = 0; i < items.length; i++)
            {
              items[i].dispose();
            }
            fillMenu(m);
            mRecreateMenu = false;
          }
        }
      });
  }

  /**   
   * Initialize this action so that it can dynamically set its tooltip.  Also set the enabled state
   * of the underlying action based on whether there are any running eXo Platform launches.
   *
   * @param pAction
   */
  private void initialize(IAction pAction)
  {
    getLaunchManager().addLaunchListener(this);
    setAction(pAction);
    updateTooltip();
    updateEnablement();
  }

  /**   
   * Return whether there are any launch instances of the eXo Platform.
   *
   * @return whether there are any launch instances of the eXo Platform
   */
  private boolean anyExoPlatformLaunches()
  {
    ILaunch[] launches = ExoUILaunching.getExoLaunches();

    if ((launches == null) || (launches.length < 1))
    {
      return false;
    }
    else
    {
      return true;
    }
  }

  /**   
   * Enable/disable this action if there is one or more eXo platform running.
   */
  private void updateEnablement()
  {
    IAction action = getAction();
    if (action != null)
    {
      action.setEnabled(anyExoPlatformLaunches());
    }
  }

  /**   
   * Updates this action's tooltip to correspond to the most recent launch of the eXo platform.
   */
  private void updateTooltip()
  {
  	String text = "Stop eXo Platform";
  	ILaunch launch = getLastLaunch();
  	if (launch != null)
  	{
  		ILaunchConfiguration config = launch.getLaunchConfiguration();
  		if (config != null)
  		{
  			text = "Stop " + config.getName();
  		}
  	}
  	
  	IAction action = getAction();
  	if (action != null)
  	{
			action.setToolTipText(text);  		
  	}
  }

	private ILaunch getLastLaunch()
	{
		ILaunch[] launches = ExoUILaunching.getExoLaunches();

		if ((launches == null) || (launches.length < 1))
		{
			return null;
		}
		else
		{
			return launches[launches.length-1];
		}		
	}
	
  /**   
   * Sets the action used to render this delegate.
   *
   * @param pAction the action used to render this delegate
   */
  private void setAction(IAction pAction)
  {
    mAction = pAction;
  }

  /**   
   * Returns the action used to render this delegate.
   *
   * @return the action used to render this delegate
   */
  private IAction getAction()
  {
    return mAction;
  }

  /**   
   * Sets this action's drop-down menu, disposing the previous menu.
   *
   * @param pMenu the new menu
   */
  private void setMenu(Menu pMenu)
  {
    if (mMenu != null)
    {
      mMenu.dispose();
    }
    mMenu = pMenu;
  }

  /**   
   * Returns the launch manager.
   *
   * @return launch  manager
   */
  private ILaunchManager getLaunchManager()
  {
    return DebugPlugin.getDefault().getLaunchManager();
  }
}
