/**
 * Copyright 2001-2004 The eXo platform SARL All rights reserved.
 * (c) Copyright 2002, Hewlett-Packard Development Company, LP
 * Please look at license.txt and license_hp.txt in info directory for more license detail.
 **/
package org.exoplatform.services.rdf.model;

import com.hp.hpl.jena.db.IDBConnection;
import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.ontology.daml.DAMLModel;
import com.hp.hpl.jena.rdf.model.InfModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelMaker;
import com.hp.hpl.jena.rdf.model.ModelSpec;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.reasoner.Reasoner;
import com.hp.hpl.jena.shared.PrefixMapping;
import com.hp.hpl.jena.shared.ReificationStyle;

/**
 * Created by the Exo Development team.
 * 
 * @author Ove Ranheim (oranheim@users.sourceforge.net)
 * @since 07.apr.2004 18:39:36
 */
public interface RDFModelService {

    public String guessDBURL();

    public String guessDBUser();

    public String guessDBPassword();

    public String guessDBType();

    public String guessDBDriver();

    /**
     * Guess the database type as the string between the first and second colons
     * of the URL. This method is public so that it may be invoked from test
     * packages.
     * 
     * @param dbURL a string of the form nocolons:somename:somestuff
     * @return somename
     */

    public String extractType(String dbURL);

    /**
     * The standard reification style; quadlets contribute to reified
     * statements, and are visible to listStatements().
     */
    public ReificationStyle finalStandard();

    /**
     * The convenient reification style; quadlets contribute to reified
     * statements, but are invisible to listStatements().
     */
    public ReificationStyle finalConvenient();

    /**
     * The minimal reification style; quadlets do not contribute to reified
     * statements, and are visible to listStatements().
     */
    public ReificationStyle finalMinimal();

    /**
     * Each Model created by ModelFactory has a default set of prefix mappings.
     * 
     * These mappings are copied from a (static) default PrefixMapping which is
     * set by setDefaultModelPrefixes. It is the reference to a PrefixMapping
     * that is retained, not a copy of it, so a user may set the defaults with
     * this method and continue to modify it; the modifications will appear in
     * the next model to be created.
     * 
     * <p>
     * 
     * When a Model is created from an existing Graph, the prefixes of that
     * Graph are not disturbed; only ones not present in the Graph are added.
     * 
     * @param pm the default prefixes to use
     * @return the previous default prefix mapping
     */
    public PrefixMapping setDefaultModelPrefixes(PrefixMapping pm);

    /**
     * Answer the current default model prefixes PrefixMapping object.
     */
    public PrefixMapping getDefaultModelPrefixes();

    /**
     * Answer a ModelSpec which can create models to the specifications in the
     * RDF description. The root of the description is the unique resource of
     * type ModelSpec.
     */
    public ModelSpec createSpec(Model desc);

    /**
     * Answer a ModelSpec which can create models to the specifcations in the
     * RDF description rooted at the given root.
     */
    public ModelSpec createSpec(Resource root, Model desc);

    /**
     * Answer a fresh Model created according to the ModelSpec argument.
     */
    public Model createModel(ModelSpec desc);

    /**
     * Answer a fresh Model created according to the given specifictaion and
     * based on any underlying model with the given name.
     * 
     * @param desc the ModelSpec which describes the kind of model to create
     * @param name the name of the base model in the underlying ModelMaker
     * @return a fresh model based over the named model
     */
    public Model createModelOver(ModelSpec desc, String name);

    /**
     * Answer a fresh Model with the default specification and Standard
     * reification style
     * 
     * [reification triples contribute to ReifiedStatements, and are visible to
     * listStatements, etc].
     */
    public Model createDefaultModel();

    /**
     * Answer a new memory-based model with the given reification style
     */
    public Model createDefaultModel(ReificationStyle style);

    /**
     * Answer a read-only Model with all the statements of this Model and any
     * statements "hidden" by reification. That model is dynamic, ie any changes
     * this model will be reflected that one.
     */
    public Model withHiddenStatements(Model m);

    /**
     * construct a new memory-based model that does not capture reification
     * triples (but still handles reifyAs() and .as(ReifiedStatement).
     */
    public Model createNonreifyingModel();

    /**
     * Answer a model that encapsulates the given graph. Existing prefixes are
     * undisturbed.
     * 
     * @param g A graph structure
     * @return A model presenting an API view of graph g
     */
    public Model createModelForGraph(Graph g);

    /**
     * Answer a ModelMaker that constructs memory-based Models that are backed
     * by files in the root directory. The Model is loaded from the file when it
     * is opened, and when the Model is closed it is written back. The model is
     * given the Standard reification style.
     * 
     * @param root the name of the directory in which the backing files are held
     * @return a ModelMaker linked to the files in the root
     */
    public ModelMaker createFileModelMaker(String root);

    /**
     * Answer a ModelMaker that constructs memory-based Models that are backed
     * by files in the root directory. The Model is loaded from the file when it
     * is opened, and when the Model is closed it is written back.
     * 
     * @param root the name of the directory in which the backing files are held
     * @param style the desired reification style
     * @return a ModelMaker linked to the files in the root
     */
    public ModelMaker createFileModelMaker(String root, ReificationStyle style);

    /**
     * Answer a ModelMaker that constructs memory-based Models that do not
     * persist past JVM termination. The model has the Standard reification
     * style.
     * 
     * @return a ModelMaker that constructs memory-based models
     */
    public ModelMaker createMemModelMaker();

    /**
     * Answer a ModelMaker that constructs memory-based Models that do not
     * persist past JVM termination, with the given reification style.
     * 
     * @param style the reification style for the model
     * @return a ModelMaker that constructs memory-based models
     */
    public ModelMaker createMemModelMaker(ReificationStyle style);

    /**
     * Answer a ModelMaker that accesses database-backed Models on the database
     * at the other end of the connection c with the usual Standard reification
     * style.
     * 
     * @param c a connection to the database holding the models
     * @return a ModelMaker whose Models are held in the database at c
     */
    public ModelMaker createModelRDBMaker(IDBConnection c);

    /**
     * Answer a ModelMaker that accesses database-backed Models on the database
     * at the other end of the connection c with the given reification style.
     * 
     * @param c a connection to the database holding the models
     * @param style the desired reification style
     * @return a ModelMaker whose Models are held in the database at c
     */
    public ModelMaker createModelRDBMaker(IDBConnection c, ReificationStyle style);

    /*
     * static class ModelRDBMaker extends ModelMakerImpl implements ModelMaker {
     * 
     * public ModelRDBMaker( GraphRDBMaker gm ) { super( gm ); }
     * 
     * 
     * 
     * public Model makeModel( Graph graphRDB ) { return new ModelRDB(
     * (GraphRDB) graphRDB ); } }
     */

    /**
     * Answer a plain IDBConnection to a database with the given URL, with the
     * given user having the given password. For more complex ways of forming a
     * connection, see the DBConnection documentation.
     * 
     * @param url the URL of the database
     * @param user the user name to use to access the database
     * @param password the password to use. WARNING: open text.
     * @param dbType the databate type: currently, "Oracle" or "MySQL".
     * @return the connection
     * @exception quite possibly
     */
    public IDBConnection createSimpleRDBConnection(String url, String user, String password, String dbType);

    /**
     * Answer a plain IDBConnection to a database, with the arguments implicitly
     * supplied by system properties:
     * 
     * <p>
     * 
     * The database URL - jena.db.url <br>
     * The user - jena.db.user, or fails back to "test" <br>
     * The password - jena.db.password, or fails back to ""<br>
     * The db type - jena.db.type, or guessed from the URL
     */
    public IDBConnection createSimpleRDBConnection();

    /**
     * Return a Model through which all the RDFS entailments derivable from the
     * given model are accessible. Some work is done when the inferenced model
     * is created but each query will also trigger some additional inference
     * work.
     * 
     * @param model the Model containing both instance data and schema
     *            assertions to be inferenced over
     */
    public InfModel createRDFSModel(Model model);

    /**
     * Return a Model through which all the RDFS entailments derivable from the
     * given data and schema models are accessible.
     * 
     * There is no strict requirement to separate schema and instance data
     * between the two arguments.
     * 
     * @param model a Model containing instance data assertions
     * @param schema a Model containing RDFS schema data
     */
    public InfModel createRDFSModel(Model schema, Model model);

    /**
     * Build an inferred model by attaching the given RDF model to the given
     * reasoner.
     * 
     * @param reasoner the reasoner to use to process the data
     * @param model the Model containing both instance data and schema
     *            assertions to be inferenced over
     */
    public InfModel createInfModel(Reasoner reasoner, Model model);

    /**
     * Build an inferred model by attaching the given RDF model to the given
     * reasoner.
     * 
     * @param reasoner the reasoner to use to process the data
     * @param model a Model containing instance data assertions
     * @param schema a Model containing RDFS schema data
     */
    public InfModel createInfModel(Reasoner reasoner, Model schema, Model model);

    /**
     * Answer a new ontology model which will process in-memory models of
     * ontologies expressed the default ontology language (OWL). The default
     * document manager will be used to load the ontology's included documents.
     * </p>
     * 
     * @return A new ontology model
     * @see OntModelSpec#getDefaultSpec
     */
    public OntModel createOntologyModel();

    /**
     * Answer a new ontology model which will process in-memory models of
     * ontologies in the given language.
     * 
     * The default document manager will be used to load the ontology's included
     * documents.
     * </p>
     * 
     * @param languageURI The URI specifying the ontology language we want to
     *            process
     * @return A new ontology model
     * @see OntModelSpec#getDefaultSpec
     */
    public OntModel createOntologyModel(String languageURI);

    /**
     * Answer a new ontology model which will process in-memory models of
     * ontologies expressed the default ontology language (OWL).
     * 
     * The default document manager will be used to load the ontology's included
     * documents.
     * </p>
     * 
     * @param spec An ontology model specification that defines the language and
     *            reasoner to use
     * @param maker A model maker that is used to get the initial store for the
     *            ontology (unless the base model is given), and create
     *            addtional stores for the models in the imports closure
     * @param base The base model, which contains the contents of the ontology
     *            to be processed
     * @return A new ontology model
     * @see OntModelSpec
     */
    public OntModel createOntologyModel(OntModelSpec spec, ModelMaker maker, Model base);

    /**
     * Answer a new ontology model, constructed according to the given ontology
     * model specification, and starting with the ontology data in the given
     * model.
     * </p>
     * 
     * @param spec An ontology model specification object, that will be used to
     *            construct the ontology model with different options of
     *            ontology language, reasoner, document manager and storage
     *            model
     * @param base An existing model to treat as an ontology model, or null.
     * @return A new ontology model
     * @see OntModelSpec
     */
    public OntModel createOntologyModel(OntModelSpec spec, Model base);

    /**
     * Answer a model for processing DAML+OIL, using the legacy Jena1 DAML API.
     * Users are encouraged to switch from the DAML-specific API to the new
     * generic ontology API
     * 
     * (see {@link #createOntologyModel(OntModelSpec, Model)}). The
     * continuation of the DAML-specific
     * 
     * API in Jena is not assured beyond Jena version 2.
     * </p>
     * 
     * @return A model for in-memory processing of DAML objects.
     */
    public DAMLModel createDAMLModel();
}

