/**
 **************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.nodetype;

import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.MalformedInputException;
import java.nio.charset.UnmappableCharacterException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFactory;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.nodetype.PropertyDefinition;

import org.exoplatform.commons.utils.ISO8601;
import org.exoplatform.services.jcr.core.nodetype.NodeDefinitionValue;
import org.exoplatform.services.jcr.core.nodetype.NodeTypeValue;
import org.exoplatform.services.jcr.core.nodetype.PropertyDefinitionValue;
import org.exoplatform.services.jcr.impl.core.LocationFactory;
import org.exoplatform.services.jcr.impl.core.NamespaceRegistryImpl;
import org.exoplatform.services.jcr.impl.core.value.BaseValue;
import org.exoplatform.services.jcr.impl.core.value.ValueFactoryImpl;
import org.exoplatform.services.log.impl.ExoLog;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: NodeTypeImpl.java 3510 2005-12-29 17:32:57Z geaz $
 */
public class NodeTypeImpl implements NodeType {

	protected static final Value[] NULL_VALUES = { null };
	
	protected static final String[] EMPTY_CONSTRAINTS = new String[0];

	protected ExoLog log;

	protected String name;

	protected boolean mixin;

	protected boolean orderableChild;

	protected String primaryItemName;

	protected NodeType[] declaredSupertypes;

	protected PropertyDefinition[] declaredPropertyDefinitions;

	protected NodeDefinition[] declaredChildNodeDefinitions;

	protected NodeTypeManager manager;

	protected NodeTypeImpl(NodeTypeManager manager) {
		this.manager = manager;
		// this.valueFactory = valueFactory;
		// log = LogUtil.getLog("org.exoplatform.services.jcr");
		// this.log = (ExoLog) logService.getLog(this.getClass());
	}

	NodeTypeImpl(NodeTypeManager manager, ValueFactory valueFactory,
			NodeTypeValue value) throws NoSuchNodeTypeException,
			ConstraintViolationException, RepositoryException {
		this(manager);

		this.name = value.getName();
		this.mixin = value.isMixin();
		this.orderableChild = value.isOrderableChild();
		this.primaryItemName = value.getPrimaryItemName();

		List supertypesList = value.getDeclaredSupertypeNames();
		this.declaredSupertypes = new NodeType[supertypesList.size()];
		for (int i = 0; i < supertypesList.size(); i++)
			declaredSupertypes[i] = manager.getNodeType((String) supertypesList
					.get(i));

		List declaredPropertyDefList = value
				.getDeclaredPropertyDefinitionValues();
		/*
		 * log.info("NodeTypeImpl: declaredPropertyDefList: " +
		 * declaredPropertyDefList + ", size: " +
		 * declaredPropertyDefList.size());
		 */

		this.declaredPropertyDefinitions = new PropertyDefinition[declaredPropertyDefList
				.size()];
		for (int i = 0; i < declaredPropertyDefList.size(); i++) {
			PropertyDefinitionValue p = (PropertyDefinitionValue) declaredPropertyDefList
					.get(i);

			/*
			 * According the specification constraints can be null, indicationg
			 * that value constraint information is unavailable
			 */
			String[] constraints = null;
			if (p.getValueConstraints() != null) {
				/*
				 * Or an array interpreted as disjunctive set of constraints
				 * Empty array means that constraints is available and that no
				 * constaraints are placed
				 */
				constraints = new String[p.getValueConstraints().size()];
				for (int j = 0; j < p.getValueConstraints().size(); j++) {
					constraints[j] = (String) p.getValueConstraints().get(j); // Fixed
					// from i to
					// j,
					// tippo^)
				}
			}

			/*
			 * According the specification defValues can be null, indicationg
			 * that has no fixed default value for property definition
			 */
			Value[] defValues = null;
			if (p.getDefaultValueStrings() != null) {
				/* Or an array of values objects */
				defValues = new Value[p.getDefaultValueStrings().size()];
				for (int j = 0; j < p.getDefaultValueStrings().size(); j++) {
					defValues[j] = valueFactory.createValue((String) p
							.getDefaultValueStrings().get(j), p
							.getRequiredType());
				}
			}

			declaredPropertyDefinitions[i] = new PropertyDefinitionImpl(p
					.getName(), this, p.getRequiredType(), constraints,
					defValues, p.isAutoCreate(), p.isMandatory(), p
							.getOnVersion(), p.isReadOnly(), p.isMultiple());
		}

		List declaredChildNodeDefList = value
				.getDeclaredChildNodeDefinitionValues();
		this.declaredChildNodeDefinitions = new NodeDefinition[declaredChildNodeDefList
				.size()];
		for (int i = 0; i < declaredChildNodeDefList.size(); i++) {
			NodeDefinitionValue n = (NodeDefinitionValue) declaredChildNodeDefList
					.get(i);

			NodeType[] requiredTypes = new NodeType[n
					.getRequiredNodeTypeNames().size()];
			for (int j = 0; j < n.getRequiredNodeTypeNames().size(); j++) {
				requiredTypes[j] = manager.getNodeType((String) n
						.getRequiredNodeTypeNames().get(j)); // Fixed from i
				// to j, tippo^)
			}

			/*
			 * According the specification DefaultPrimaryType can be null,
			 * indicationg that DefaultPrimaryType is not specified
			 */
			NodeType nt = null;
			if (n.getDefaultNodeTypeName() != null) {
				nt = manager.getNodeType(n.getDefaultNodeTypeName());
			}
			declaredChildNodeDefinitions[i] = new NodeDefinitionImpl(n
					.getName(), this, requiredTypes, nt, n.isAutoCreate(), n
					.isMandatory(), n.getOnVersion(), n.isReadOnly(), n
					.isSameNameSiblings());
		}
	}

	/** @see javax.jcr.nodetype.NodeType#getName */
	public String getName() {
		return name;
	}

	/** @see javax.jcr.nodetype.NodeType#isMixin */
	public boolean isMixin() {
		return mixin;
	}

	/** @see javax.jcr.nodetype.NodeType#hasOrderableChildNodes */
	public boolean hasOrderableChildNodes() {
		return orderableChild;
	}

	/** @see javax.jcr.nodetype.NodeType#getPrimaryItemName */
	public String getPrimaryItemName() {
		return primaryItemName;
	}

	/** @see javax.jcr.nodetype.NodeType#getSupertypes */
	public NodeType[] getSupertypes() {
		ArrayList stypesList = new ArrayList();
		fillSupertypes(stypesList, this);
		if (stypesList.size() > 0) {
			NodeType[] supertypes = new NodeType[stypesList.size()];
			for (int i = 0; i < stypesList.size(); i++) {
				supertypes[i] = (NodeType) stypesList.get(i);
			}
			return supertypes;
		}
		return new NodeType[0];
	}

	/** @see javax.jcr.nodetype.NodeType#getDeclaredSupertypes */
	public NodeType[] getDeclaredSupertypes() {
		if (declaredSupertypes != null)
			return declaredSupertypes;
		else
			return new NodeType[0];

	}

	/** @see javax.jcr.nodetype.NodeType#getDeclaredSupertypes */
	public boolean isNodeType(String nodeTypeName) {
		NodeType superType = null;
		try {
			superType = manager.getNodeType(nodeTypeName);
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(" NodeTypeImpl.isNodeType (" + superType
					+ ") failed Reason:" + e);
		}
		return isSameOrSubType(superType, this);
	}

	/** @see javax.jcr.nodetype.NodeType#getPropertyDefs */
	public PropertyDefinition[] getPropertyDefinitions() {
		ArrayList propertyDefsList = new ArrayList();
		if (declaredPropertyDefinitions != null) {
			for (int i = 0; i < declaredPropertyDefinitions.length; i++) {
				propertyDefsList.add(declaredPropertyDefinitions[i]);
			}
		}
		NodeType[] supertypes = getSupertypes();
		if (supertypes != null) {
			for (int i = 0; i < supertypes.length; i++) {
				if (supertypes[i].getDeclaredPropertyDefinitions() != null) {
					for (int j = 0; j < supertypes[i]
							.getDeclaredPropertyDefinitions().length; j++) {
						if (!propertyDefsList.contains(supertypes[i]
								.getDeclaredPropertyDefinitions()[j]))
							propertyDefsList.add(supertypes[i]
									.getDeclaredPropertyDefinitions()[j]);
					}
				}
			}
		}
		if (propertyDefsList.size() > 0) {
			PropertyDefinition[] propertyDefs = new PropertyDefinition[propertyDefsList
					.size()];
			for (int i = 0; i < propertyDefsList.size(); i++) {
				propertyDefs[i] = (PropertyDefinition) propertyDefsList.get(i);
			}
			return propertyDefs;
		}
		return new PropertyDefinition[0];
	}

	/** @see javax.jcr.nodetype.NodeType#getDeclaredPropertyDefs */
	public PropertyDefinition[] getDeclaredPropertyDefinitions() {
		if (declaredPropertyDefinitions != null)
			return declaredPropertyDefinitions;
		else
			return new PropertyDefinition[0];
	}

	/** @see javax.jcr.nodetype.NodeType#getChildNodeDefs */
	public NodeDefinition[] getChildNodeDefinitions() {
		ArrayList nodeDefsList = new ArrayList();
		if (declaredChildNodeDefinitions != null) {
			for (int i = 0; i < declaredChildNodeDefinitions.length; i++) {
				nodeDefsList.add(declaredChildNodeDefinitions[i]);
			}
		}
		NodeType[] supertypes = getSupertypes();
		if (supertypes != null) {
			for (int i = 0; i < supertypes.length; i++) {
				if (supertypes[i].getDeclaredChildNodeDefinitions() != null) {
					for (int j = 0; j < supertypes[i]
							.getDeclaredChildNodeDefinitions().length; j++) {
						if (!nodeDefsList.contains(supertypes[i]
								.getDeclaredChildNodeDefinitions()[j])) {
							nodeDefsList.add(supertypes[i]
									.getDeclaredChildNodeDefinitions()[j]);
						}
					}
				}
			}
		}
		if (nodeDefsList.size() > 0) {
			NodeDefinition[] childNodeDefs = new NodeDefinition[nodeDefsList
					.size()];
			for (int i = 0; i < nodeDefsList.size(); i++) {
				childNodeDefs[i] = (NodeDefinition) nodeDefsList.get(i);
			}
			return childNodeDefs;
		}
		return new NodeDefinition[0];
	}

	/** @see javax.jcr.nodetype.NodeType#getDeclaredChildNodeDefs */
	public NodeDefinition[] getDeclaredChildNodeDefinitions() {
		if (declaredChildNodeDefinitions != null)
			return declaredChildNodeDefinitions;
		else
			return new NodeDefinition[0];
	}

	/** @see javax.jcr.nodetype.NodeType#canSetProperty */
	public boolean canSetProperty(String propertyName, Value value) {

		PropertyDefinition def = getPropertyDefinitions(propertyName)
				.getDefinition(false);
		if (def != null) {
			if (def.isProtected()) {
				return false;
			} else {
				int requiredType = def.getRequiredType();
				return canSetPropertyForType(requiredType, propertyName, value,
						def.getValueConstraints());
			}
		} else {
			return false;
		}
	}

	private boolean canSetPropertyForType(int requiredType,
			String propertyName, Value value, String[] constrains) {

		if (value == null) {
			return canRemoveItem(propertyName);
		} else if (requiredType == value.getType()) {
			return checkValueConstraints(constrains, value);
		} else if (requiredType == PropertyType.BINARY
				&& (value.getType() == PropertyType.STRING
						|| value.getType() == PropertyType.DATE
						|| value.getType() == PropertyType.LONG
						|| value.getType() == PropertyType.DOUBLE
						|| value.getType() == PropertyType.NAME
						|| value.getType() == PropertyType.PATH || value
						.getType() == PropertyType.BOOLEAN)) {
			return checkValueConstraints(constrains, value);
		} else if (requiredType == PropertyType.BOOLEAN) {
			if (value.getType() == PropertyType.STRING) {
				return checkValueConstraints(constrains, value);
			} else if (value.getType() == PropertyType.BINARY) {
				try {
					return isCharsetString(value.getString(),
							BaseValue.DEFAULT_ENCODING)
							&& checkValueConstraints(constrains, value);
				} catch (Exception e) {
					// Hm, this is not string and not UTF-8 too
					return false;
				}
			} else {
				return false;
			}
		} else if (requiredType == PropertyType.DATE) {
			String likeDataString = null;
			try {
				if (value.getType() == PropertyType.STRING) {
					likeDataString = value.getString();
				} else if (value.getType() == PropertyType.BINARY) {
					likeDataString = getCharsetString(value.getString(),
							BaseValue.DEFAULT_ENCODING);
					// likeDataString = value.getString(); // This line
					// can work too, replacing line below
				} else if (value.getType() == PropertyType.DOUBLE
						|| value.getType() == PropertyType.LONG) {
					return checkValueConstraints(constrains, value);
				} else {
					return false;
				}
				Calendar calDate = ISO8601.parse(likeDataString);
				return calDate != null
						&& checkValueConstraints(constrains, value);
			} catch (Exception e) {
				// Hm, this is not date format string
				return false;
			}
		} else if (requiredType == PropertyType.DOUBLE) {
			String likeDoubleString = null;
			try {
				if (value.getType() == PropertyType.STRING) {
					likeDoubleString = value.getString();
				} else if (value.getType() == PropertyType.BINARY) {
					likeDoubleString = getCharsetString(value.getString(),
							BaseValue.DEFAULT_ENCODING);
					// likeDataString = value.getString(); // This line
					// can work too, replacing line below
				} else if (value.getType() == PropertyType.LONG) {
					return checkValueConstraints(constrains, value);
				} else {
					return false;
				}
				Double doubleValue = new Double(likeDoubleString);
				return doubleValue != null
						&& checkValueConstraints(constrains, value);
			} catch (Exception e) {
				// Hm, this is not double formated string
				return false;
			}
		} else if (requiredType == PropertyType.LONG) {
			String likeLongString = null;
			try {
				if (value.getType() == PropertyType.STRING) {
					likeLongString = value.getString();
				} else if (value.getType() == PropertyType.BINARY) {
					likeLongString = getCharsetString(value.getString(),
							BaseValue.DEFAULT_ENCODING);
					// likeDataString = value.getString(); // This line
					// can work too, replacing line below
				} else if (value.getType() == PropertyType.DATE) {
					return true;
				} else if (value.getType() == PropertyType.DOUBLE) {
					return true;
				} else {
					return false;
				}
				Long longValue = new Long(likeLongString);
				return longValue != null
						&& checkValueConstraints(constrains, value);
			} catch (Exception e) {
				// Hm, this is not long formated string
				return false;
			}
		} else if (requiredType == PropertyType.NAME) {
			String likeNameString = null;
			try {
				if (value.getType() == PropertyType.STRING) {
					likeNameString = value.getString();
				} else if (value.getType() == PropertyType.BINARY) {
					likeNameString = getCharsetString(value.getString(),
							BaseValue.DEFAULT_ENCODING);
					// likeDataString = value.getString(); // This line
					// can work too, replacing line below
				} else if (value.getType() == PropertyType.PATH) {
					String pathString = value.getString();
					// PathValue pathv = (PathValue) value;
					String[] pathParts = pathString.split("\\/");
					if (pathString.startsWith("/")
							&& (pathParts.length > 1 || pathString.indexOf("[") > 0)) {
						// Path is not relative - absolute
						// FALSE if it is more than one element long
						// or has an index
						return false;
					} else if (!pathParts.equals("/") && pathParts.length == 1
							&& pathString.indexOf("[") < 0) {
						// Path is relative
						// TRUE if it is one element long
						// and has no index
						return checkValueConstraints(constrains, value);
					} else if (pathString.startsWith("/")
							&& pathString.lastIndexOf("/") < 1
							&& pathString.indexOf("[") < 0) {
						// System.err.println("abs path, one elem, no index: " +
						// pathString);
						return checkValueConstraints(constrains, value);
					} else {
						return false;
					}
				} else {
					return false;
				}
				try {
					Value nameValue = getNameValue(likeNameString);
					return nameValue != null
							&& checkValueConstraints(constrains, value);
				} catch (Exception e) {
					return false;
				}
			} catch (Exception e) {
				return false;
			}
		} else if (requiredType == PropertyType.STRING) {
			String likeStringString = null;
			try {
				if (value.getType() == PropertyType.BINARY) {
					likeStringString = getCharsetString(value.getString(),
							BaseValue.DEFAULT_ENCODING);
				} else if (value.getType() == PropertyType.DATE
						|| value.getType() == PropertyType.LONG
						|| value.getType() == PropertyType.BOOLEAN
						|| value.getType() == PropertyType.NAME
						|| value.getType() == PropertyType.PATH
						|| value.getType() == PropertyType.DOUBLE) {
					likeStringString = value.getString();
				} else {
					return false;
				}
				return likeStringString != null
						&& checkValueConstraints(constrains, value);
			} catch (Exception e) {
				// d:\pcmaster\pcmaster\dist pcmaster.jar
				// d:\pcmaster\pcmaster\html firm1_info.html, firm1_info.html
				// d:\pcmaster\pcmaster config.ini FIRM1_INFO=firm1_info.html
				// FIRM2_INFO=firm2_info.html
				return false;
			}
		} else if (requiredType == PropertyType.UNDEFINED) {
			return checkValueConstraints(constrains, value);
		} else {
			return false;
		}
	}

	private String getCharsetString(String source, String charSetName) {
		try {
			CharBuffer cb = CharBuffer.wrap(source.toCharArray());
			Charset cs = Charset.forName(charSetName);
			CharsetEncoder cse = cs.newEncoder();
			ByteBuffer encoded = cse.encode(cb);
			return new String(encoded.array()).trim(); // Trim is very
			// important!!!
		} catch (IllegalStateException e) {
			return null;
		} catch (MalformedInputException e) {
			return null;
		} catch (UnmappableCharacterException e) {
			return null;
		} catch (CharacterCodingException e) {
			return null;
		}
	}

	private boolean isCharsetString(String source, String charSetName) {
		try {
			String s = getCharsetString(source, charSetName);
			return s != null;
		} catch (Exception e) {
			return false;
		}
	}

	private Value getNameValue(String likeNameString)
			throws PathNotFoundException, RepositoryException {
		NamespaceRegistryImpl namespaceRegistry = new NamespaceRegistryImpl();
		LocationFactory factory = new LocationFactory(namespaceRegistry);
		// JCRName jcrName = factory.parseJCRName(likeNameString);

		// System.err.println("likeNameString: " + likeNameString);
		// System.err.println("jcrName: " + jcrName.getAsString());

		ValueFactoryImpl valFact = new ValueFactoryImpl(factory);
		Value nameValue = valFact
				.createValue(likeNameString, PropertyType.NAME);

		// System.err.println("valFact: " + valFact.toString());
		// System.err.println("nameValue: " + nameValue.getString());

		return nameValue;
	}

	/** @see javax.jcr.nodetype.NodeType#canSetProperty */
	public boolean canSetProperty(String propertyName, Value[] values) {
		PropertyDefinition def = getPropertyDefinitions(propertyName)
				.getDefinition(true);
		if (def != null) {
			if (def.isProtected()) {
				return false;
			} else {
				if (values != null) {
					int requiredType = def.getRequiredType();
					if (values.length > 0) {
						int res = 0;
						for (int i = 0; i < values.length; i++) {
							try {
								if (canSetPropertyForType(requiredType,
										propertyName, values[i], def
												.getValueConstraints())) {
									res++;
								}
							} catch (Exception e) {
							}
						}
						return res == values.length;
					} else {
						return true;
					}
				} else {
					return canRemoveItem(propertyName);
				}
			}
		} else {
			return false;
		}
	}

	/** @see javax.jcr.nodetype.NodeType#canAddChildNode */
	public boolean canAddChildNode(String childNodeName) {
		NodeDefinition childNodeDef = getChildNodeDefinition(childNodeName);
		return !(childNodeDef == null || childNodeDef.isProtected() || childNodeDef
				.getDefaultPrimaryType() == null);
	}

	/** @see javax.jcr.nodetype.NodeType#canAddChildNode */
	public boolean canAddChildNode(String childNodeName, String nodeTypeName) {
		NodeDefinition childNodeDef = getChildNodeDefinition(childNodeName);
		return !(childNodeDef == null || childNodeDef.isProtected())
				&& isChildNodePrimaryTypeAllowed(nodeTypeName);
	}

	/** @see javax.jcr.nodetype.NodeType#canRemoveItem */
	public boolean canRemoveItem(String itemName) {
		PropertyDefinition propDef = getPropertyDefinitions(itemName)
				.getAnyDefinition();
		if (propDef != null)
			return !(propDef.isMandatory() || propDef.isProtected());

		NodeDefinition nodeDef = getChildNodeDefinition(itemName);
		if (nodeDef != null)
			return !(nodeDef.isMandatory() || nodeDef.isProtected());

		return false;
	}

	public PropertyDefinitions getPropertyDefinitions(String name) {
		PropertyDefinitions defs = new PropertyDefinitions();

		PropertyDefinitionImpl propResidual = null;
		for (int i = 0; i < getPropertyDefinitions().length; i++) {
			PropertyDefinitionImpl propDef = (PropertyDefinitionImpl) getPropertyDefinitions()[i];
			// log.debug(" "+propDef.getName()+" "+propDef.isMultiple()+"
			// "+name);

			if (propDef.getName().equals(name) || propDef.isResidualSet()) {
				defs.setDefinition(propDef);
			}
			/*
			 * if (propDef.getName().equals(name)) {
			 * defs.setDefinition(propDef); return defs; } else if
			 * (propDef.isResidualSet()) { propResidual = propDef; }
			 */
		}
		// defs.setDefinition(propResidual);
		return defs;
	}

	public NodeDefinition getChildNodeDefinition(String name) {
		NodeDefinition residual = null;
		for (int i = 0; i < getChildNodeDefinitions().length; i++) {
			NodeDefinition nodeDef = getChildNodeDefinitions()[i];
			if (nodeDef.getName().equals(name)) {
				return nodeDef;
			} else if (nodeDef.getName()
					.equals(NodeDefinitionImpl.RESIDUAL_SET)) {
				residual = nodeDef;
			}
		}
		return residual;
	}

	public boolean equals(Object obj) {
		if (!(obj instanceof NodeTypeImpl))
			return false;
		return name == ((NodeType) obj).getName();
	}

	// //////////////////////////////////

	// protected boolean isBase() {
	// return getSupertypes().length == 0;
	// }

	protected boolean checkValueConstraints(String[] constraints, Value value) {

		if (constraints != null && constraints.length > 0) {
			for (int i = 0; i < constraints.length; i++) {
				try {
					if (constraints[i].equals(value.getString())) {
						return true;
					}
				} catch (RepositoryException e) {
					System.err
							.println("Error! Can't get value's string value: "
									+ e);
				}
			}
		} else {
			return true;
		}
		return false;
	}

	protected void fillSupertypes(List list, NodeType subtype) {
		if (subtype.getDeclaredSupertypes() != null) {
			for (int i = 0; i < subtype.getDeclaredSupertypes().length; i++) {
				list.add(subtype.getDeclaredSupertypes()[i]);
				fillSupertypes(list, subtype.getDeclaredSupertypes()[i]);
			}
		}
	}

	public static boolean isSameOrSubType(NodeType superType, NodeType subType) {
		if (superType.getName().equals(subType.getName()))
			return true;
		else {
			NodeType[] superTypes = subType.getSupertypes();
			for (int j = 0; j < superTypes.length; j++) {
				NodeType testSuperType = superTypes[j];
				if (testSuperType.getName().equals(superType.getName()))
					return true;
			}
		}
		return false;
	}

	public ArrayList getManadatoryItemDefs() {

		ArrayList itemDefs = new ArrayList();
		for (int i = 0; i < getPropertyDefinitions().length; i++) {
			if (getPropertyDefinitions()[i].isMandatory())
				itemDefs.add(getPropertyDefinitions()[i]);
		}
		for (int i = 0; i < getChildNodeDefinitions().length; i++) {
			if (getChildNodeDefinitions()[i].isMandatory())
				itemDefs.add(getChildNodeDefinitions()[i]);
		}

		return itemDefs;

	}

	public boolean isChildNodePrimaryTypeAllowed(String typeName) {

		NodeDefinition[] definitions = this.getDeclaredChildNodeDefinitions();
		NodeType nodeType;
		try {
			nodeType = manager.getNodeType(typeName);
		} catch (RepositoryException e) {
			throw new RuntimeException("Error " + e);
		}
		for (int i = 0; i < definitions.length; i++) {
			NodeType[] types = definitions[i].getRequiredPrimaryTypes();
			for (int j = 0; j < types.length; j++) {
				if (NodeTypeImpl.isSameOrSubType(types[j], nodeType))
					return true;
			}
		}
		return false;
	}

	/**
	 * @param mixin
	 *            The mixin to set.
	 */
	public void setMixin(boolean mixin) {
		this.mixin = mixin;
	}

	/**
	 * @param name
	 *            The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @param orderableChild
	 *            The orderableChild to set.
	 */
	public void setOrderableChild(boolean orderableChild) {
		this.orderableChild = orderableChild;
	}

	/**
	 * @param primaryItemName
	 *            The primaryItemName to set.
	 */
	public void setPrimaryItemName(String primaryItemName) {
		this.primaryItemName = primaryItemName;
	}

	/**
	 * @param declaredNodeDefs
	 *            The declaredNodeDefs to set.
	 */
	public void setDeclaredNodeDefs(NodeDefinition[] declaredNodeDefs) {
		this.declaredChildNodeDefinitions = declaredNodeDefs;
	}

	/**
	 * @param declaredPropertyDefs
	 *            The declaredPropertyDefs to set.
	 */
	public void setDeclaredPropertyDefs(
			PropertyDefinition[] declaredPropertyDefs) {
		this.declaredPropertyDefinitions = declaredPropertyDefs;
	}

	/**
	 * @param declaredSupertypes
	 *            The declaredSupertypes to set.
	 */
	public void setDeclaredSupertypes(NodeType[] declaredSupertypes) {
		this.declaredSupertypes = declaredSupertypes;
	}
}
