/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.nodetype;

import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.jcr.ItemExistsException;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFactory;
import javax.jcr.ValueFormatException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeIterator;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.nodetype.PropertyDefinition;
import javax.jcr.version.OnParentVersionAction;

import org.apache.commons.logging.Log;
import org.exoplatform.commons.Environment;
import org.exoplatform.services.jcr.access.AccessControlPolicy;
import org.exoplatform.services.jcr.config.RepositoryEntry;
import org.exoplatform.services.jcr.core.ExtendedPropertyType;
import org.exoplatform.services.jcr.core.nodetype.ExtendedNodeTypeManager;
import org.exoplatform.services.jcr.core.nodetype.NodeTypeValue;
import org.exoplatform.services.jcr.core.nodetype.NodeTypeValuesList;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.util.EntityCollection;
import org.exoplatform.services.log.LogService;
import org.jibx.runtime.BindingDirectory;
import org.jibx.runtime.IBindingFactory;
import org.jibx.runtime.IUnmarshallingContext;
import org.jibx.runtime.JiBXException;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: NodeTypeManagerImpl.java 3607 2006-01-04 11:28:33Z peterit $
 */
public class NodeTypeManagerImpl implements ExtendedNodeTypeManager {

  private static final Class[] DEFAULT_TYPES = {
      org.exoplatform.services.jcr.impl.core.nodetype.nt.BaseType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.mix.ReferenceableType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.mix.LockableType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.UnstructuredType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.HierarchyNodeType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.FileType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.FolderType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.ResourceType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.PropertyDefType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.ChildNodeDefType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.NodeTypeType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.LinkedFileType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.QueryType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.VersionLabelsType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.FrozenNodeType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.VersionType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.VersionHistoryType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.nt.VersionedChildType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.mix.VersionableType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.exo.AccessControllableType.class,
      org.exoplatform.services.jcr.impl.core.nodetype.exo.NamespaceType.class,

      org.exoplatform.services.jcr.impl.core.nodetype.exo.MockNodeType.class };

  public static final String NT_ROOT = "/jcr:system/jcr:nodetypes";

  private ValueFactory valueFactory;

  private Map nodeTypes;

  private SessionImpl systemSession;

  private String accessControlPolicy;

  private boolean persisted = false;

  protected Log log;

  public NodeTypeManagerImpl(RepositoryEntry config, ValueFactory valueFactory, LogService logService)
      throws RepositoryException, InstantiationException {
    log = logService.getLog("org.exoplatform.services.jcr");
    this.nodeTypes = new HashMap();
    this.valueFactory = valueFactory;
    this.accessControlPolicy = config.getAccessControl();

    initDefault();
  }

  /**
   * 6.10.4 Returns the NodeType specified by nodeTypeName. If no node type by
   * that name is registered, a NoSuchNodeTypeException is thrown.
   */
  public NodeType getNodeType(String nodeTypeName)
      throws NoSuchNodeTypeException {
    NodeType nt = findNodeType(nodeTypeName);
    if (nt == null)
      throw new NoSuchNodeTypeException(
          "NodeTypeManager.getNodeType(): NodeType '" + nodeTypeName
              + "' not found.");
    else
      return nt;
  }

  private NodeType findNodeType(String nodeTypeName) {
    return (NodeType) nodeTypes.get(nodeTypeName);

  }

  public NodeType getNodeType(InternalQName qName)
      throws NoSuchNodeTypeException, RepositoryException {

    String nodeName = systemSession.getLocationFactory().createJCRName(qName)
        .getAsString();
    return getNodeType(nodeName);
  }

  /** 6.10.4 Returns all available node types. */
  public NodeTypeIterator getAllNodeTypes() {

    EntityCollection ec = new EntityCollection();
    ec.addAll(nodeTypes.values());
    return ec;
  }

  /**
   * Returns an iterator over all available primary node types.
   * 
   * @return An <code>NodeTypeIterator</code>.
   * @throws RepositoryException
   *           if an error occurs.
   */
  public NodeTypeIterator getPrimaryNodeTypes() throws RepositoryException {
    EntityCollection ec = new EntityCollection();
    NodeTypeIterator allTypes = getAllNodeTypes();
    while (allTypes.hasNext()) {
      NodeType type = allTypes.nextNodeType();
      if (!type.isMixin())
        ec.add(type);
    }
    return ec;
  }

  /**
   * Returns an iterator over all available mixin node types.
   * 
   * @return An <code>NodeTypeIterator</code>.
   * @throws RepositoryException
   *           if an error occurs.
   */
  public NodeTypeIterator getMixinNodeTypes() throws RepositoryException {
    EntityCollection ec = new EntityCollection();
    NodeTypeIterator allTypes = getAllNodeTypes();
    while (allTypes.hasNext()) {
      NodeType type = allTypes.nextNodeType();
      if (type.isMixin())
        ec.add(type);
    }
    return ec;
  }

  /**
   * Registers node type from object
   * 
   * @param nodeType -
   *          the node type object
   * @param alreadyExistsBehaviour -
   *          if node type with such a name already exists: IGNORE_IF_EXISTS -
   *          does not register new node (default) FAIL_IF_EXISTS - throws
   *          RepositoryException REPLACE_IF_EXISTS - replaces registerd type
   *          with new one
   * @throws RepositoryException
   */
  public void registerNodeType(NodeType nodeType, int alreadyExistsBehaviour)
      throws RepositoryException {

    long start = new Date().getTime();
    
    if (accessControlPolicy.equals(AccessControlPolicy.DISABLE) &&
        nodeType.getName().equals("exo:accessControllable")) {
      throw new RepositoryException("NodeType exo:accessControllable is DISABLED");
    }

    if (findNodeType(nodeType.getName()) != null) {
      if (alreadyExistsBehaviour == FAIL_IF_EXISTS)
        throw new RepositoryException("NodeType " + nodeType.getName()
            + " is already registered");
      //else if (alreadyExistsBehaviour == REPLACE_IF_EXISTS) {
      // currently unsupported
      //unregisterNodeType(nodeType.getName());
      //	throw new RepositoryException("NodeType REPLACE_IF_EXISTS
      // currently is
      // not supported");
      //}
      else {
      //  log.info("NodeType " + nodeType.getName()
      //      + " is already registered. Registration ignored.");
        return;
      }
    }
    nodeTypes.put(nodeType.getName(), nodeType);

    if (persisted) {
      NodeImpl ntRoot = (NodeImpl) systemSession.getItem(NT_ROOT);
      log.debug("NodeType " + nodeType.getName() + " makeNT start "
          + (new Date().getTime() - start) + " ms");

      Node node = makeTypeAsNode(ntRoot, nodeType);
      log.debug("NodeType " + nodeType.getName() + " save start "
          + (new Date().getTime() - start) + " ms");

      systemSession.save();
    } else
      log.warn("Persistent storage is not initialized. Cant persist NodeType "
          + nodeType.getName());

    log.info("NodeType " + nodeType.getName() + " is registered. time "
        + (new Date().getTime() - start) + " ms");
  }

  /**
   * Registers node type from class containing the NT definition The class
   * should have constructor with one parameter NodeTypeManager
   * 
   * @param nodeTypeType -
   *          Class containing node type definition
   * @param alreadyExistsBehaviour
   *          if node type with such a name already exists: IGNORE_IF_EXISTS -
   *          does not register new node (default) FAIL_IF_EXISTS - throws
   *          RepositoryException REPLACE_IF_EXISTS - replaces registerd type
   *          with new one
   * @throws RepositoryException
   */
  public void registerNodeType(Class nodeTypeType, int alreadyExistsBehaviour)
      throws RepositoryException, InstantiationException {

    registerNodeType(makeNtFromClass(nodeTypeType), alreadyExistsBehaviour);

  }

  public void registerNodeType(NodeTypeValue nodeTypeValue,
      int alreadyExistsBehaviour) throws RepositoryException {

    //log.info("registerNodeType: this: " + this + ", valueFactory: " +
    // valueFactory + ", nodeTypeValue: " + nodeTypeValue);
	
	// We have to validate node value before registering it
	nodeTypeValue.validateNodeType();
    NodeTypeImpl nodeType = new NodeTypeImpl(this, valueFactory, nodeTypeValue);
    registerNodeType(nodeType, alreadyExistsBehaviour);

  }

  /**
   * Registers node type using xml document
   * 
   * @param xml
   * @param alreadyExistsBehaviour
   * @throws RepositoryException
   * 
   * @see org.exoplatform.services.jcr.core.nodetype.ExtendedNodeTypeManager#registerNodeType(java.io.InputStream,
   *      int)
   */
  public void registerAllNodeTypes(InputStream xml, int alreadyExistsBehaviour)
      throws RepositoryException {

    try {
      IBindingFactory factory = BindingDirectory
          .getFactory(NodeTypeValuesList.class);
      IUnmarshallingContext uctx = factory.createUnmarshallingContext();
      NodeTypeValuesList nodeTypeValuesList = (NodeTypeValuesList) uctx
          .unmarshalDocument(xml, null);
      ArrayList ntvList = nodeTypeValuesList.getNodeTypeValuesList();

      //log.info("registerNodeType, values size: " + ntvList.size());
      for (int i = 0; i < ntvList.size(); i++) {
        if (ntvList.get(i) != null) {
          NodeTypeValue nodeTypeValue = (NodeTypeValue) ntvList.get(i);
          nodeTypeValue.validateNodeType();
          log.info("Registering node type from xml stream: " + nodeTypeValue.getName());
          registerNodeType(nodeTypeValue, alreadyExistsBehaviour);
        } else {
          // Hm! Smth is wrong in xml document
          log.error("Empty nodeTypeValue in xml document, index: " + i
              + ", skiping...");
        }
      }
    } catch (JiBXException e) {
      e.printStackTrace();
      throw new RepositoryException("Error in config initialization " + e);
    }
  }

  private NodeType makeNtFromClass(Class nodeTypeType)
      throws InstantiationException {

    try {
      Constructor c = nodeTypeType
          .getConstructor(new Class[] { NodeTypeManager.class });
      return (NodeType) c.newInstance(new Object[] { this });
    } catch (Exception e1) {
      e1.printStackTrace();
      throw new InstantiationException(
          "Error in making istance of "
              + nodeTypeType.getName()
              + ". Class should have been NodeType subclass and have constructor with one argument NodeTypeManager type. Reason: "
              + e1.getCause());
    }
  }

  // Currently unsupported as there can be repository integrity problems after
  // deleting nodetype
  //public void unregisterNodeType(String name) throws RepositoryException {
  //	if (Environment.getInstance().getPlatform() != Environment.STAND_ALONE)
  //		throw new RepositoryException(
  //				"Currently Unregistration is supported for the testing purpose only as
  // may
  // cause integrity problems.(checking if there are nodes of this node type
  // should be involved!)");
  //	nodeTypes.remove(name);
  //	if(persisted) {
  //		NodeImpl ntRoot = (NodeImpl) systemSession.getItem(NT_ROOT);
  //		try {
  //			ntRoot.getNode(name).remove();
  //			ntRoot.save();
  //		} catch (PathNotFoundException e1) {
  //		}
  //
  //	}
  //	log.info("NodeType " + name + " is unregistered.");

  //}

  public synchronized void initStorage(Node sysNode, boolean forcePersistence)
      throws RepositoryException {

    NodeImpl ntRoot;
    try {
      ntRoot = (NodeImpl) systemSession.getItem(NT_ROOT);
    } catch (PathNotFoundException e1) {
      ntRoot = (NodeImpl) sysNode.addNode("jcr:nodetypes", "nt:unstructured");
      if(!accessControlPolicy.equals(AccessControlPolicy.DISABLE))
        //ntRoot.doAddMixin("exo:accessControllable", true, true);
        ntRoot.doAddMixin("exo:accessControllable", true, false, true);

    }

    long start = new Date().getTime();
    NodeTypeIterator i = getAllNodeTypes();

    // TODO!!!
    if (Environment.getInstance().getPlatform() == Environment.STAND_ALONE
        && !forcePersistence) {
      return;
    }

    persisted = true;
    while (i.hasNext()) {
      NodeTypeImpl nodeType = null;
      try {
        nodeType = (NodeTypeImpl) i.nextNodeType();
        Node node = makeTypeAsNode(ntRoot, nodeType);
      } catch (ItemExistsException e) {
      }
      log.info("" + nodeType.getName() + " is initialized.");

    }
    systemSession.save();
    log.info("NodeType manager is initialized, initialization time: "
        + (new Date().getTime() - start) + " ms");

  }

  public void initStorage(Node sysNode) throws RepositoryException {
    initStorage(sysNode, false);
  }

  private void initDefault() throws RepositoryException, InstantiationException {
    for (int i = 0; i < DEFAULT_TYPES.length; i++) {
      NodeType type = makeNtFromClass(DEFAULT_TYPES[i]);
      // Do not register exo:accessControllable if AccessControlPolicy == DISABLE

      if (accessControlPolicy.equals(AccessControlPolicy.DISABLE) &&
          type.getName().equals("exo:accessControllable")) {
        // skip
      } else {
         nodeTypes.put(type.getName(), type);
      }
    }
  }

  public void loadFromStorage() throws RepositoryException {
    NodeImpl ntRoot;
    try {
      ntRoot = (NodeImpl) systemSession.getItem(NT_ROOT);
    } catch (PathNotFoundException e1) {
      log
          .info("NodeTypes storage (/jcr:system/jcr:nodetypes) is not initialized. Only default nodetypes are accessible ");
      return;
    }

    NodeIterator types = ntRoot.getNodes();
    while (types.hasNext()) {
      Node nodeTypeNode = types.nextNode();
      if (this.findNodeType(nodeTypeNode.getName()) == null) {
        try {
          getNodeType(nodeTypeNode.getName());
        } catch (NoSuchNodeTypeException e) {
          if ((nodeTypeNode.isNodeType("nt:nodeType"))) {
            try {
              nodeTypes.put(nodeTypeNode.getName(),
                  makeNodeTypeFromNode(nodeTypeNode));
            } catch (RepositoryException e1) {
              log.error("Loading of node type " + nodeTypeNode.getName()
                  + " failed " + e1.getMessage());
              continue;
            }
          }
        }
      }
      log.info("Node Type " + nodeTypeNode.getName() + " is loaded");
    }

  }

  private Node makeTypeAsNode(NodeImpl parent, NodeType nodeType)
      throws PathNotFoundException, RepositoryException, ValueFormatException {

    NodeImpl node = parent.createChildNode(nodeType.getName(), "nt:nodeType",
        false, true);

    node.setProperty("jcr:nodeTypeName", valueFactory.createValue(nodeType
        .getName(), PropertyType.NAME));
    node.setProperty("jcr:isMixin", nodeType.isMixin());
    node.setProperty("jcr:hasOrderableChildNodes", nodeType
        .hasOrderableChildNodes());
    if (nodeType.getPrimaryItemName() != null)
      node.setProperty("jcr:primaryItemName", nodeType.getPrimaryItemName());
    Value[] parents = new Value[nodeType.getDeclaredSupertypes().length];
    for (int i = 0; i < parents.length; i++)
      parents[i] = valueFactory.createValue(nodeType.getDeclaredSupertypes()[i]
          .getName(), PropertyType.NAME);
    if (parents.length != 0)
      node.setProperty("jcr:supertypes", parents);
    if (nodeType.getDeclaredPropertyDefinitions().length > 0) {
      for (int i = 0; i < nodeType.getDeclaredPropertyDefinitions().length; i++) {
        NodeImpl childProps = node.createChildNode("jcr:propertyDefinition",
            "nt:propertyDefinition", false, true); // same-name sibs
        initPropertyDefProps(childProps, nodeType
            .getDeclaredPropertyDefinitions()[i]);
      }
    }

    if (nodeType.getDeclaredChildNodeDefinitions().length > 0) {
      for (int i = 0; i < nodeType.getDeclaredChildNodeDefinitions().length; i++) {
        Node childNodes = node.createChildNode("jcr:childNodeDefinition",
            "nt:childNodeDefinition", false, true); // same-name sibs
        initNodeDefProps(childNodes,
            nodeType.getDeclaredChildNodeDefinitions()[i]);
      }
    }
    return node;
  }

  private void initPropertyDefProps(Node node, PropertyDefinition def)
      throws ValueFormatException, RepositoryException {
    node.setProperty("jcr:name", valueFactory.createValue(def.getName(),
        PropertyType.NAME));
    node.setProperty("jcr:autoCreated", def.isAutoCreated());
    node.setProperty("jcr:mandatory", def.isMandatory());
    node.setProperty("jcr:onParentVersion", OnParentVersionAction
        .nameFromValue(def.getOnParentVersion()));
    node.setProperty("jcr:protected", def.isProtected());
    node.setProperty("jcr:requiredType", ExtendedPropertyType.nameFromValue(def
        .getRequiredType()));
    node.setProperty("jcr:multiple", def.isMultiple());
    node.setProperty("jcr:valueConstraints", def.getValueConstraints());
    node.setProperty("jcr:defaultValues", def.getDefaultValues());
  }

  private void initNodeDefProps(Node node, NodeDefinition def)
      throws ValueFormatException, RepositoryException {
    node.setProperty("jcr:name", valueFactory.createValue(def.getName(),
        PropertyType.NAME));
    node.setProperty("jcr:autoCreated", def.isAutoCreated());
    node.setProperty("jcr:mandatory", def.isMandatory());
    node.setProperty("jcr:onParentVersion", OnParentVersionAction
        .nameFromValue(def.getOnParentVersion()));
    node.setProperty("jcr:protected", def.isProtected());
    node.setProperty("jcr:sameNameSiblings", def.allowsSameNameSiblings());
    if (def.getDefaultPrimaryType() != null)
      node.setProperty("jcr:defaultPrimaryType", def.getDefaultPrimaryType()
          .getName());
    Value[] reqTypes = new Value[def.getRequiredPrimaryTypes().length];
    for (int i = 0; i < reqTypes.length; i++)
      reqTypes[i] = valueFactory.createValue(def.getRequiredPrimaryTypes()[i]
          .getName(), PropertyType.NAME);
    node.setProperty("jcr:requiredPrimaryTypes", reqTypes, PropertyType.NAME);
  }

  /**
   * @param systemSession
   *          The systemSession to set.
   */
  public void setSystemSession(SessionImpl systemSession) {
    this.systemSession = systemSession;
  }

  private NodeTypeImpl makeNodeTypeFromNode(Node nodeTypeNode)
      throws NoSuchNodeTypeException, ConstraintViolationException,
      RepositoryException {

    NodeTypeImpl type = new NodeTypeImpl(this);

    try {
      type.setName(nodeTypeNode.getProperty("jcr:nodeTypeName").getString());
      type.setMixin(nodeTypeNode.getProperty("jcr:isMixin").getBoolean());
      type.setOrderableChild(nodeTypeNode.getProperty(
          "jcr:hasOrderableChildNodes").getBoolean());
    } catch (PathNotFoundException e) {
      throw new ConstraintViolationException("Mandatory property does not set "
          + e);
    }
    try {
      type.setPrimaryItemName(nodeTypeNode.getProperty("jcr:primaryItemName")
          .getString());
    } catch (PathNotFoundException e) {
    }
    try {
      Value[] supertypeValues = nodeTypeNode.getProperty("jcr:supertypes")
          .getValues();
      NodeType[] declaredSupertypes = new NodeType[supertypeValues.length];
      for (int i = 0; i < supertypeValues.length; i++) {
        declaredSupertypes[i] = getNodeType(supertypeValues[i].getString());
      }
      type.setDeclaredSupertypes(declaredSupertypes);
    } catch (PathNotFoundException e) {
      throw new ConstraintViolationException("Mandatory property did not set "
          + e);
    }

    type.setDeclaredPropertyDefs(getDeclaredPropertyDefs(nodeTypeNode));

    type.setDeclaredNodeDefs(getDeclaredChildNodeDefs(nodeTypeNode));

    return type;
  }

  private PropertyDefinition[] getDeclaredPropertyDefs(Node nodeTypeNode)
      throws NoSuchNodeTypeException, ConstraintViolationException,
      RepositoryException {

    NodeIterator propertyDefs = nodeTypeNode.getNodes("jcr:propertyDefinition");
    PropertyDefinition[] declaredPropertyDefs = new PropertyDefinition[(int) propertyDefs
        .getSize()];
    int i = 0;
    while (propertyDefs.hasNext()) {
      Node propertyDefNode = propertyDefs.nextNode();
      PropertyDefinitionImpl pDef = new PropertyDefinitionImpl();
      //log.info("Prop def node "+propertyDefNode.getPath());

      try {
        pDef.setName(propertyDefNode.getProperty("jcr:name").getString());
        pDef.setAutoCreate(propertyDefNode.getProperty("jcr:autoCreated")
            .getBoolean());
        pDef.setMandatory(propertyDefNode.getProperty("jcr:mandatory")
            .getBoolean());
        pDef.setReadOnly(propertyDefNode.getProperty("jcr:protected")
            .getBoolean());
        pDef.setMultiple(propertyDefNode.getProperty("jcr:multiple")
            .getBoolean());
        pDef.setOnVersion(OnParentVersionAction.valueFromName(propertyDefNode
            .getProperty("jcr:onParentVersion").getString()));
        pDef.setDeclaringNodeType(propertyDefNode.getPrimaryNodeType());
        pDef.setRequiredType(ExtendedPropertyType.valueFromName(propertyDefNode
            .getProperty("jcr:requiredType").getString()));
      } catch (PathNotFoundException e) {
        throw new ConstraintViolationException(
            "Mandatory property did not set " + e);
      }

      try {
        Value[] valueConstraintValues = propertyDefNode.getProperty(
            "jcr:valueConstraints").getValues();
        String[] valueConstraints = new String[valueConstraintValues.length];
        for (int j = 0; j < valueConstraintValues.length; j++) {
          if (valueConstraintValues[j] != null)
            valueConstraints[i] = valueConstraintValues[j].getString();
          else
            valueConstraints[j] = null;
        }
        pDef.setValueConstraints(valueConstraints);
      } catch (PathNotFoundException e) {
      }
      try {
        pDef.setDefaultValues(propertyDefNode.getProperty("jcr:defaultValues")
            .getValues());
      } catch (PathNotFoundException e) {
      }

      declaredPropertyDefs[i++] = pDef;
    }

    return declaredPropertyDefs;

  }

  private NodeDefinition[] getDeclaredChildNodeDefs(Node nodeTypeNode)
      throws NoSuchNodeTypeException, ConstraintViolationException,
      RepositoryException {

    NodeIterator childNodeDefs = nodeTypeNode
        .getNodes("jcr:childNodeDefinition");
    NodeDefinition[] declaredNodeDefs = new NodeDefinition[(int) childNodeDefs
        .getSize()];
    int i = 0;
    while (childNodeDefs.hasNext()) {

      Node nodeDefNode = childNodeDefs.nextNode();
      NodeDefinitionImpl nDef = new NodeDefinitionImpl();

      try {
        nDef.setName(nodeDefNode.getProperty("jcr:name").getString());
        //nDef.setDefaultNodeType(getNodeType(nodeDefNode.
        //		getProperty("jcr:defaultPrimaryType").getString()));

        Value[] requiredNodeTypesValues = nodeDefNode.getProperty(
            "jcr:requiredPrimaryTypes").getValues();
        NodeType[] requiredNodeTypes = new NodeType[requiredNodeTypesValues.length];
        for (int j = 0; j < requiredNodeTypesValues.length; j++) {
          if (requiredNodeTypesValues[j] != null)
            requiredNodeTypes[j] = getNodeType(requiredNodeTypesValues[j]
                .getString());
          else
            throw new ConstraintViolationException(
                "Required node type is null ");
        }
        nDef.setRequiredNodeTypes(requiredNodeTypes);
        nDef.setDeclaringNodeType(nodeDefNode.getPrimaryNodeType());

        nDef.setAutoCreate(nodeDefNode.getProperty("jcr:autoCreated")
            .getBoolean());
        nDef
            .setMandatory(nodeDefNode.getProperty("jcr:mandatory").getBoolean());
        nDef.setReadOnly(nodeDefNode.getProperty("jcr:protected").getBoolean());
        nDef.setMultiple(nodeDefNode.getProperty("jcr:sameNameSiblings")
            .getBoolean());
        nDef.setOnVersion(OnParentVersionAction.valueFromName(nodeDefNode
            .getProperty("jcr:onParentVersion").getString()));
      } catch (PathNotFoundException e) {
        throw new ConstraintViolationException(
            "Mandatory property did not set " + e);
      }

      try {
        nDef.setDefaultNodeType(getNodeType(nodeDefNode.getProperty(
            "jcr:defaultPrimaryType").getString()));
      } catch (PathNotFoundException e) {
      }

      declaredNodeDefs[i++] = nDef;
    }

    return declaredNodeDefs;

  }

  public boolean isNodeType(InternalQName superTypeName,
      InternalQName[] typeNames) {

    for (int i = 0; i < typeNames.length; i++) {
      if (superTypeName.equals(typeNames[i]))
        return true;
      else {

        NodeType subType;
        NodeType superType;
        try {
          subType = getNodeType(systemSession.getLocationFactory()
              .createJCRName(typeNames[i]).getAsString());

          superType = getNodeType(systemSession.getLocationFactory()
              .createJCRName(superTypeName).getAsString());
        } catch (RepositoryException e) {
          log.error("Error obtaining node type " + e);
          continue;
        }

        NodeType[] superTypes = subType.getSupertypes();
        for (int j = 0; j < superTypes.length; j++) {
          NodeType testSuperType = superTypes[j];
          if (testSuperType.getName().equals(superType.getName()))
            return true;
        }

      }
    }
    return false;
  }

}