
/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.version;

import java.util.ArrayList;

import javax.jcr.AccessDeniedException;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.PropertyType;
import javax.jcr.ReferentialIntegrityException;
import javax.jcr.RepositoryException;
import javax.jcr.UnsupportedRepositoryOperationException;
import javax.jcr.Value;
import javax.jcr.version.Version;
import javax.jcr.version.VersionException;
import javax.jcr.version.VersionHistory;
import javax.jcr.version.VersionIterator;

import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.ItemChangeState;
import org.exoplatform.services.jcr.impl.core.JCRPath;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.WorkspaceImpl;
import org.exoplatform.services.jcr.impl.storage.TransientNodeData;
import org.exoplatform.services.jcr.impl.util.EntityCollection;
import org.exoplatform.services.log.LogService;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: VersionHistoryImpl.java 3547 2006-01-03 08:42:56Z geaz $
 */

public class VersionHistoryImpl extends VersionStorageDescendantNode 
  implements VersionHistory {

  public VersionHistoryImpl(NodeData data, NodeImpl parent, int state,
      SessionImpl session, LogService logService) throws PathNotFoundException,
      RepositoryException {
    super(data, parent, state, session, logService);
    if (!this.getPrimaryNodeType().isNodeType("nt:versionHistory"))
      throw new RepositoryException("Node " + getLocation().getAsString(true)
          + " is not nt:versionHistory type");
  }

  public String getVersionableUUID() throws RepositoryException {
    return this.getProperty("jcr:versionableUuid").getString();
  }

  public Version getRootVersion() throws RepositoryException {
    return (Version) this.getNode("jcr:rootVersion");
  }

  public VersionIterator getAllVersions() throws RepositoryException {
    EntityCollection versions = new EntityCollection();
    // should be first in list
    versions.add(getNode("jcr:rootVersion"));

    NodeIterator nodes = getNodes();
    while (nodes.hasNext()) {
      NodeImpl node = (NodeImpl) nodes.nextNode();
      if (!node.getName().equals("jcr:rootVersion")
          && !node.getName().equals("jcr:versionLabels")) {
        node = new VersionImpl(node.storedNodeData(), this, this.state,
            this.session, this.logService);
        versions.add(node);
        //        versions.add(new VersionImpl(node));
      }
    }
    return versions;
  }

  public Version getVersion(String versionName) throws VersionException,
      RepositoryException {
    NodeImpl node;
    try {
      node = (NodeImpl)getNode(versionName);
    } catch (PathNotFoundException e) {
      throw new VersionException("There are no version with name '" + versionName
          + "' in the version history "+getPath());
    }
    
    return new VersionImpl(node.storedNodeData(), this, this.state,
        this.session, this.logService);
  }

  public Version getVersionByLabel(String label) throws RepositoryException {

    String uuid = getUuidByLabel(label);
    if (uuid == null)
      throw new RepositoryException("No label " + label);

    NodeIterator children = this.getNodes();
    while (children.hasNext()) {
      NodeImpl node = (NodeImpl)children.nextNode();
      // version only
      if (node.isNodeType("nt:version") &&
           node.getInternalUUID().equals(uuid)) 
        return new VersionImpl(node.storedNodeData(), this, this.state,
            this.session, this.logService);
    }

    throw new RepositoryException("No version for label " + label);

  }

  public boolean hasVersionLabel(String label) throws RepositoryException {
    if (this.getUuidByLabel(label) == null)
      return false;
    else
      return true;
  }

  public boolean hasVersionLabel(Version version, String label)
      throws VersionException, RepositoryException {

    if (version.getUUID().equals(getUuidByLabel(label)))
      return true;
    else
      return false;
  }

  public String[] getVersionLabels() throws RepositoryException {
    PropertyIterator labels = getNode("jcr:versionLabels").getProperties();
    ArrayList labelStrs = new ArrayList();
    while (labels.hasNext()) {
      Property labelProperty = labels.nextProperty();
      if(labelProperty.getType() == PropertyType.REFERENCE) {
        labelStrs.add(labelProperty.getName());
      }
    }
    String[] res = new String[labelStrs.size()];
    for(int i=0; i<labelStrs.size(); i++)
      res[i] = (String)labelStrs.get(i);
    return res;
  }

  public String[] getVersionLabels(Version version) throws VersionException,
      RepositoryException {
    
    if(!isVersionBelongToThis(version))  
      throw new VersionException("No version "+version.getPath()+" in history "+getPath());

    PropertyIterator labels = getNode("jcr:versionLabels").getProperties();
    ArrayList labelStrs = new ArrayList();
    while (labels.hasNext()) {
      Property labelProperty = labels.nextProperty();
      if (labelProperty.getType() == PropertyType.REFERENCE && 
          labelProperty.getString().equals(version.getUUID()))
        labelStrs.add(labelProperty.getName());
    }
    String[] res = new String[labelStrs.size()]; 
    for(int i=0; i<labelStrs.size(); i++)
      res[i] = (String)labelStrs.get(i);
    return res;
  }

  public void removeVersion(String versionName)
      throws ReferentialIntegrityException, AccessDeniedException,
      UnsupportedRepositoryOperationException, VersionException,
      RepositoryException {
    Version ver = getVersion(versionName);
    // remove labels first
    PropertyIterator labels = getNode("jcr:versionLabels").getProperties();
    while (labels.hasNext()) {
      Property labelProperty = labels.nextProperty();
      if (labelProperty.getString().equals(ver.getUUID())) {
        labelProperty.remove();
      }
    }
    ver.remove();
    session.save();

  }

  public void addVersionLabel(String versionName, String label,
      boolean moveLabel) throws VersionException, RepositoryException {
    if (hasVersionLabel(label)) {

      if(moveLabel) {
        removeVersionLabel(label);
      } else {
        throw new VersionException("Label " + label
          + " already exists and moveLabel == false");
      }
    }
    Version ver = getVersion(versionName);
    Value[] values = new Value[] { valueFactory.createValue(ver) };
    PropertyImpl prop = ((NodeImpl) getNode("jcr:versionLabels"))
        .createChildProperty(label, values, PropertyType.REFERENCE);
    
    workspaceDataManager().add(prop);
  }

  public void removeVersionLabel(String label) throws VersionException,
      RepositoryException {

    PropertyImpl prop;
    try {
      prop = (PropertyImpl) getNode("jcr:versionLabels").getProperty(
          label);
    } catch (PathNotFoundException e) {
      throw new VersionException("Label not found " + label);
    } 
    workspaceDataManager().delete(prop);
  }

  ////////////////// impl

  public VersionImpl addVersion(NodeImpl versionableNode) throws RepositoryException {

    String uuid = uuidGenerator.generateUUID(versionableNode.getPath()).getString();
    VersionImpl version = createVersion(nextVersionName(), uuid);
    
    version.createChildProperty("jcr:primaryType", new Value[] { valueFactory
        .createValue("nt:version", PropertyType.NAME) }, PropertyType.NAME);
    version.createChildProperty("jcr:mixinTypes", new Value[] { valueFactory
        .createValue("mix:referenceable", PropertyType.NAME) }, PropertyType.NAME);
    version.createChildProperty("jcr:uuid", new Value[] { valueFactory
        .createValue(version.storedNodeData().getUUID()) },
        PropertyType.STRING);
    version.createChildProperty("jcr:created", new Value[] { valueFactory
        .createValue(workspaceDataManager().getCurrentTime()) }, PropertyType.DATE);

    Value[] predess = versionableNode.getProperty("jcr:predecessors").getValues();

    //Ns current jcr:predecessors property is copied to V
    version.createChildProperty("jcr:predecessors", predess,
        PropertyType.REFERENCE);

    for (int i = 0; i < predess.length; i++) {
      //VersionImpl predec = ((WorkspaceImpl) session.getWorkspace())
      //    .getVersionStorageManager().getVersionByUUID(predess[i].getString(), session);
      
      VersionImpl predec = (VersionImpl) session.getNodeByUUID(predess[i].getString());

      predec.addSuccessor(version);
    }
    
    // Frozen
    NodeImpl frozen = version.createChildNode("jcr:frozenNode",
        "nt:frozenNode", true, true);

    versionableNode.accept(new FrozenNodeInitializer(frozen));
    workspaceDataManager().add(version);

    //System.out.println("add frozen "+frozen.getPath()+" "+
    //    frozen.getPrimaryNodeType().getName()+" "+
    //    version.getNode("jcr:frozenNode").getPrimaryNodeType().getName());

    
    return version;

  }
  
  public VersionImpl addRootVersion(NodeImpl versionableNode) throws RepositoryException {

       
    // Root version
    VersionImpl rootVersion = createVersion("jcr:rootVersion", 
        versionableNode.getProperty("jcr:baseVersion").getString());

    rootVersion.createChildProperty("jcr:primaryType", new Value[] { valueFactory
            .createValue("nt:version", PropertyType.NAME) }, PropertyType.NAME);
    rootVersion.createChildProperty("jcr:mixinTypes", new Value[] { valueFactory
        .createValue("mix:referenceable", PropertyType.NAME) }, PropertyType.NAME);

    rootVersion.createChildProperty("jcr:uuid", new Value[] { valueFactory
        .createValue(versionableNode.getProperty("jcr:baseVersion").getString()) },
        PropertyType.STRING);

    rootVersion.createChildProperty("jcr:created", new Value[] { valueFactory
        .createValue(workspaceDataManager().getCurrentTime()) }, PropertyType.DATE);

    // Frozen
    NodeImpl frozenNode = rootVersion.createChildNode("jcr:frozenNode",
        "nt:frozenNode", false);
    frozenNode.createChildProperty("jcr:primaryType",
        new Value[] { valueFactory.createValue("nt:frozenNode",
            PropertyType.NAME) }, PropertyType.NAME);

    frozenNode.createChildProperty("jcr:frozenPrimaryType",
        new Value[] { valueFactory.createValue(versionableNode.getPrimaryNodeType()
            .getName(), PropertyType.NAME) }, PropertyType.NAME);

    Value[] mixinNodeTypes = new Value[versionableNode.getMixinNodeTypes().length];
    for (int i = 0; i < mixinNodeTypes.length; i++) {
      mixinNodeTypes[i] = valueFactory.createValue(versionableNode.getMixinNodeTypes()[i]
          .getName(), PropertyType.NAME);
    }
    frozenNode.createChildProperty("jcr:frozenMixinTypes", mixinNodeTypes,
        PropertyType.NAME);
    frozenNode.createChildProperty("jcr:frozenUuid", new Value[] { valueFactory
        .createValue(versionableNode.getUUID()) }, PropertyType.STRING);
    
    return rootVersion;

  }  
  
  public boolean isVersionBelongToThis(Version version) {
    return ((VersionImpl)version).getLocation().isDescendantOf(getLocation(), false);
  }
  
  public Version getLastVersion() throws RepositoryException {
    VersionIterator versions = getAllVersions();
    Version last = null;
    while(versions.hasNext()) {
      Version ver = versions.nextVersion();
      if(last == null || ver.getCreated().after(last.getCreated()));
        last = ver;
    }
    return last;
  }
  

  private String getUuidByLabel(String label) throws RepositoryException {
    PropertyIterator labels = getNode("jcr:versionLabels").getProperties();
    while (labels.hasNext()) {
      Property labelProperty = labels.nextProperty();
      if (labelProperty.getName().equals(label))
        return labelProperty.getValue().getString();
    }

    return null;

  }
  
  private VersionImpl createVersion(String versionName, String uuid) throws RepositoryException {
  
    // Create new NodeData inmemory
    JCRPath path = locationFactory.createJCRPath(this.getLocation(), versionName);
    //String uuid = uuidGenerator.generateUUID(path.getAsString(true)).getString();
    int orderNum = 0;
    //try {
    //  orderNum = (int) getNodes().getSize();
    //} catch (AccessDeniedException e) {
    //  log.warn("Access denied for read order num == 0 "+getPath());
    //}
    NodeData nodeData = new TransientNodeData(path.getInternalPath(), uuid, -1,
        Constants.NT_VERSION, new InternalQName[0], orderNum, 
        storedNodeData().getUUID()); 
    
    VersionImpl version = new VersionImpl(nodeData, this, this.state, session,
        logService);
    
    addChildNode(version);
    
    return version;

  }
  
  private String nextVersionName() throws RepositoryException {
    // make version name - 1-based (because of rootVersion) integer
    return new Integer((int) getAllVersions().getSize())
        .toString();
  }
  
  //private VersionLabelsNode versionLabels() {
  //}
  
  

}